﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Management;
using System.Net.NetworkInformation;
using System.Security.Principal;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Dandelion
{
	/// <summary>
	/// Represents the user device.
	/// </summary>
	class DeviceInfo
	{
		internal string _userName;
		internal string _securityIdentifier;
		internal string _OSVersion;
		internal string _OSType;
		internal string _deviceBrand;
		internal string _deviceModelName;
		internal string _deviceUuid;
		internal string _deviceChassisTypes;
		internal string _wifiMacAddress;
		internal string _iPAddress;

		/// <summary>
		/// Initializes a new instance of the DeviceUtils class.
		/// </summary>
		public DeviceInfo()
		{
			this._userName = "UnKnow";
			this._securityIdentifier = "UnKnow";
			this._OSVersion = "UnKnow";
			this._OSType = "windows";
			this._deviceBrand = "UnKnow";
			this._deviceModelName = "UnKnow";
			this._deviceUuid = "UnKnow";
			this._deviceChassisTypes = "UnKnow";
			this._wifiMacAddress = "UnKnow";
			this._iPAddress = "UnKnow";
		}

		/// <summary>
		/// Gets the user name.
		/// </summary>
		public string GetUserName()
		{
			_userName = Environment.UserName;
			return _userName;
		}

		/// <summary>
		/// Gets a security identifier (SID).
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetSecurityIdentifier()
		{
			_securityIdentifier = System.Guid.NewGuid().ToString();
			try
			{
				var account = new NTAccount(GetUserName());
				var tmpSid = (SecurityIdentifier)account.Translate(typeof(SecurityIdentifier));
				_securityIdentifier = tmpSid.ToString();
				return _securityIdentifier;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the version of the operating system of the device.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetOSVersion()
		{
			try
			{
				_OSVersion = Environment.OSVersion.Version.ToString();
				return _OSVersion;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the OS type of the device.
		/// </summary>
		public string GetOSType()
		{
			return _OSType;
		}

		/// <summary>
		/// Gets the name of a computer manufacturer.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetDeviceBrand()
		{
			try
			{
				System.Management.ManagementObjectSearcher query1 =
											new ManagementObjectSearcher("SELECT * FROM Win32_ComputerSystem");
				ManagementObjectCollection queryCollection1 = query1.Get();
				foreach (ManagementObject mo in queryCollection1)
				{
					_deviceBrand = mo["Manufacturer"].ToString();
				}
				return _deviceBrand;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the model name.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetDeviceModel()
		{
			try
			{
				System.Management.ManagementObjectSearcher query1 =
											new ManagementObjectSearcher("SELECT * FROM Win32_ComputerSystem");
				ManagementObjectCollection queryCollection1 = query1.Get();
				foreach (ManagementObject mo in queryCollection1)
				{
					_deviceModelName = mo["model"].ToString();
				}
				return _deviceModelName;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the Universally unique identifier (UUID) for this product.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetDeviceUUID()
		{
			_deviceUuid = System.Guid.NewGuid().ToString();
			try
			{
				System.Management.ManagementObjectSearcher query1 =
									new ManagementObjectSearcher("SELECT * FROM Win32_ComputerSystemProduct");
				ManagementObjectCollection queryCollection1 = query1.Get();
				foreach (ManagementObject mo in queryCollection1)
				{
					_deviceUuid = mo["UUID"].ToString();
				}
				return _deviceUuid;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the chassis types for this product.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetDeviceChassisTypes()
		{
			try
			{
				ManagementClass systemEnclosures = new ManagementClass("Win32_SystemEnclosure");
				foreach (ManagementObject obj in systemEnclosures.GetInstances())
				{
					foreach (int i in (UInt16[])(obj["ChassisTypes"]))
					{
						if (i > 0 && i < 25)
						{
							if (i == 3)
							{
								_deviceChassisTypes = "Desktop";
							}
							else if (i == 9)
							{
								_deviceChassisTypes = "Laptop";
							}
							else if (i == 10)
							{
								_deviceChassisTypes = "Notebook";
							}
							else if (i == 13)
							{
								_deviceChassisTypes = "AllInOne";
							}
						}
					}
				}
				return _deviceChassisTypes;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the mac address of the wifi.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetWifiMacAddress()
		{
			try
			{
				NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();

				foreach (var nic in nics)
				{
					if (nic.NetworkInterfaceType == NetworkInterfaceType.Wireless80211)
					{
						string macAddStr = nic.GetPhysicalAddress().ToString();
						string macAddStrNew = macAddStr;
						int insertedCount = 0;
						for (int i = 2; i < macAddStr.Length; i = i + 2)
						{
							macAddStrNew = macAddStrNew.Insert(i + insertedCount++, ":");
						}
						_wifiMacAddress = macAddStrNew;
					}
				}
				return _wifiMacAddress;
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the ip address of the wifi.
		/// </summary>
		/// <exception cref="Exception"></exception>
		public string GetIPAddress()
		{
			try
			{
				NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();

				foreach (var nic in nics)
				{
					if (nic.NetworkInterfaceType == NetworkInterfaceType.Wireless80211)
					{
						IPInterfaceProperties ip = nic.GetIPProperties();

						UnicastIPAddressInformationCollection ipCollection = ip.UnicastAddresses;
						foreach (UnicastIPAddressInformation ipadd in ipCollection)
						{
							if (ipadd.Address.AddressFamily == AddressFamily.InterNetwork)
							{
								_iPAddress = ipadd.Address.ToString();
							}
						}
					}
				}
				return _iPAddress;
			}
			catch
			{
				throw;
			}
		}

	}
}
