﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security;
using System.Security.Permissions;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Dandelion
{
	/// <summary>
	/// Provides the methods of logger.
	/// </summary>
	internal class Logger
	{
		private string _logFilePath;
		private static int _FileItems = 0;
		private static readonly object _lockFile = new object();

		/// <summary>
		/// Construction the logger component.
		/// </summary>
		/// <param name="logFilePath"> The file path of log files. </param>
		public Logger(string filePath)
		{
			string _dir = string.Empty;
			_logFilePath = null;

			if (Common.EnableOutputToLogFile)
			{
				if (!string.IsNullOrEmpty(filePath))
				{
					_logFilePath = filePath;
					try
					{
						FileInfo tmpFile = new System.IO.FileInfo(_logFilePath);
						_dir = tmpFile.DirectoryName;
						tmpFile.Directory.Create();
					}
					catch (Exception e)
					{
						Console.WriteLine("[Logger][Construction] Exception: " + e.ToString());
					}
				}

				if (_FileItems > Common.LogFileLimit)
				{
					foreach (string d in Directory.GetFileSystemEntries(_dir))
					{
						if (File.Exists(d))
						{
							FileInfo fi = new FileInfo(d);
							if (fi.Attributes.ToString().IndexOf("ReadOnly") != -1)
								fi.Attributes = FileAttributes.Normal;
							File.Delete(d);
						}
					}

					_FileItems = 0;
				}
				else
				{
					_FileItems++;
				}
			}
		}

		/// <summary>
		/// Logs a message.
		/// </summary>
		/// <param name="message"> The log text. </param>
		public void WriteNewLine(string message)
		{
			if (Common.EnableOutputToLogFile)
			{
				if (!string.IsNullOrEmpty(_logFilePath))
				{
					try
					{
						lock (_lockFile)
						{
							var permissionSet = new PermissionSet(PermissionState.None);
							var writePermission = new FileIOPermission(FileIOPermissionAccess.Write, _logFilePath);
							permissionSet.AddPermission(writePermission);

							using (var fs = new FileStream(_logFilePath, FileMode.Append, FileAccess.Write,
																							FileShare.ReadWrite))
							{
								using (var tmpLog = new StreamWriter(fs))
								{
									string tmpTime = DateTime.Now.ToString("MM-dd HH:mm:ss.fff");
									tmpLog.WriteLine("[" + tmpTime + "] " + message);
								}
							}
						}
					}
					catch (Exception e)
					{
						Console.WriteLine("[Logger][WriteNewLine] Exception: " + e.ToString());
					}
				}
			}
		}
	}
}
