﻿using AirFusion.Common;
using AirFusion.ConnectionManager;
using AirFusion.Manager.DeviceManager;
using AirFusion.Manager.FeatureHelper;
using AirFusion.Windows;
using Microsoft.Win32;
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for App.xaml
	/// </summary>
	public partial class App : Application, INotifyPropertyChanged
	{
		#region Events

		public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

		#endregion Events

		#region Properties

		// For UI
		private TrayIconWindow _appTrayIconWindow = null;

		public TrayIconWindow AppTrayIconWindow
		{
			get
			{
				return _appTrayIconWindow;
			}

			set
			{
				_appTrayIconWindow = value;
			}
		}

		private ScreenWindow _appScreenWindow = null;

		public ScreenWindow AppScreenWindow
		{
			get
			{
				return _appScreenWindow;
			}

			set
			{
				_appScreenWindow = value;
			}
		}

		private DeviceManagerWindow _appDeviceManagerWindow = null;

		public DeviceManagerWindow AppDeviceManagerWindow
		{
			get
			{
				return _appDeviceManagerWindow;
			}

			set
			{
				_appDeviceManagerWindow = value;
			}
		}

		private double _dpiRatio = 1.0;

		public double DPIRatio
		{
			get
			{
				return _dpiRatio;
			}

			set
			{
				_dpiRatio = value;
			}
		}

		public bool _isShowTCPDisconnection = false;

		#endregion Properties

		#region Application-related Methods

		public void LoadResource()
		{
			string strLocalizationCulture = string.Empty;

			ResourceDictionary LanguageResource = new ResourceDictionary();
			ResourceDictionary StyleResource = new ResourceDictionary();

			#region load Multi-Language

			try
			{
				string CurrentUICulture = System.Globalization.CultureInfo.CurrentUICulture.ToString().ToUpper();

				if (CurrentUICulture.Equals("PT-BR"))
					strLocalizationCulture = "pt-BR";
				else if (CurrentUICulture.Equals("EN-AU"))
					strLocalizationCulture = "EN-AU";
				else if (CurrentUICulture.Equals("EN-CA"))
					strLocalizationCulture = "EN-CA";
				else if (CurrentUICulture.Equals("EN-GB"))
					strLocalizationCulture = "EN-GB";
				else if (CurrentUICulture.Equals("ES-US"))
					strLocalizationCulture = "ES-US";
				else if (CurrentUICulture.Equals("FR-CA"))
					strLocalizationCulture = "FR-CA";
				else if (CurrentUICulture.Equals("NO") || CurrentUICulture.Equals("NB-NO") || CurrentUICulture.Equals("NN-NO"))
					strLocalizationCulture = "no";
				else
					strLocalizationCulture = System.Globalization.CultureInfo.CurrentUICulture.Parent.Name;

				//string PDPath = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + "\\acer\\CareCenter\\MUIenv.dat";
				//if (File.Exists(PDPath))
				//	strLocalizationCulture = System.Globalization.CultureInfo.CurrentCulture.Parent.Name;

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[LoadResource] Localization is : " + strLocalizationCulture, "");

				//strLocalizationCulture = "zh-cht";
				//strLocalizationCulture = "EL";
				//strLocalizationCulture = "AR";

				LanguageResource.Source = new Uri(@"pack://application:,,,/AcerEXTEND;component/MUI/" + strLocalizationCulture.ToLower() + ".xaml");
			}
			catch (Exception ex)
			{
				LanguageResource.Source = new Uri(@"pack://application:,,,/AcerEXTEND;component/MUI/en.xaml");

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[LoadResource] Exception when Load MultiLanguage:" + strLocalizationCulture, "");
			}

			#endregion load Multi-Language

			#region Load DPI

			try
			{
				int i_DPI = (int)Utils.GetDPI();

				//string CurrentUICulture = System.Globalization.CultureInfo.CurrentUICulture.ToString().ToUpper();

				StyleResource.Source = new Uri(@"pack://application:,,,/AcerEXTEND;component/Styles/DPI_" + i_DPI.ToString() + ".xaml");
			}
			catch (Exception ex)
			{
				StyleResource.Source = new Uri(@"pack://application:,,,/AcerEXTEND;component/Styles/DPI_96.xaml");

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[LoadResource] Exception when Load MultiLanguage:" + strLocalizationCulture, "");
			}

			if (LanguageResource.Source != null)
				Application.Current.Resources.MergedDictionaries.Add(LanguageResource);

			if (StyleResource.Source != null)
				Application.Current.Resources.MergedDictionaries.Add(StyleResource);

			#endregion Load DPI
		}

		/// <summary>
		/// Handles the Startup event of the Application control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="StartupEventArgs"/> instance containing the event data.</param>
		private void Application_Startup(object sender, StartupEventArgs e)
		{
			LoadResource();

			#region Set Working Folder

			//0409 Johnson add for set working folder
			try
			{
				Directory.SetCurrentDirectory(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName));
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set Working folder to " + Process.GetCurrentProcess().MainModule.FileName.ToString(), "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			#endregion Set Working Folder

			#region Read Application Version

			try
			{
				//0411 Johnson add for get AirFusion installer version
				string installerVersion = @"DisplayVersion";
				object oVersion = null;
				bool isX86;
				oVersion = RegistryUtility.GetRegistry(RegistryHive.LocalMachine, Configs.AcerEXTENDInstallKeyX64, installerVersion, RegistryView.Registry64, out isX86);

				if (oVersion == null)
					oVersion = RegistryUtility.GetRegistry(RegistryHive.LocalMachine, Configs.AcerEXTENDInstallKeyX86, installerVersion, RegistryView.Registry64, out isX86);

				if (oVersion != null)
				{
					this.ApplicationVersion = oVersion.ToString();
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start, v" + this.ApplicationVersion, "");
				}
				else
				{
					// Get assembly file version
					System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
					Version temp = assembly.GetName().Version;
					this.ApplicationVersion = string.Join(".", new string[] { temp.Major.ToString(), string.Format("{0:00}", temp.Minor), string.Format("{0:0000}", temp.Build), temp.Revision.ToString() });

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start, v" + this.ApplicationVersion, "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			#endregion Read Application Version

			//Set Minimum Support Android Target Version
			this.MinimumTargetVersion = @"1.00.2013";

			try
			{
				#region check single instance

				if (!this.SingleInstance())
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "There is an existed instance!!", "");

					if (!System.Security.Principal.WindowsIdentity.GetCurrent().IsGuest)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "It is launched by normal user", "");
						this.NotifyTray();
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "It is launched by guest user", "");
					}

					this.Shutdown();
					return;
				}

				#endregion check single instance

				#region Check Account Type, the applicatio is not support guest accout

				// Show a dialog when the user is guest
				if (System.Security.Principal.WindowsIdentity.GetCurrent().IsGuest)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "It is launched by guest user, to show dialog", "");
					//StartupUri = new Uri("/AcerEXTEND;component/Windows/UserGuestWindow.xaml", UriKind.Relative);
					string GuestAccountContent = string.Format(Application.Current.TryFindResource("txtUserGuestDetail") as String,
														Application.Current.TryFindResource("txtAppName") as String);
					string ButtonContent = (string)Application.Current.TryFindResource("txtOK");
					CustomMessageBox guestMessageBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																	GuestAccountContent, CustomMessageBox.CustomImageIcon.Warring, ButtonContent));
					guestMessageBox.ShowDialog();
					return;
				}

				#endregion Check Account Type, the applicatio is not support guest accout

				// Get suitable DPI
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start getting DPI ratio", "");
				this.DPIRatio = Utils.GetDPIRatio(false);

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start initializing", "");
				this.InitializeEvents();

				this.InitializeValues();

				this.InitializeRequiredUIComponents();

				//0422 Johnon add for check launch by Startup or not
				bool _isBootLaunch = false;
				if (e.Args.Length > 0)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Args: " + e.Args[0].ToString(), "");

					if (string.Compare(e.Args[0].ToString(), "bootLaunch", true) == 0)
						_isBootLaunch = true;
				}
				this.InitializeAFManager(_isBootLaunch);
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");

				this.Shutdown();
				return;
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
		}

		private void Application_Exit(object sender, ExitEventArgs e)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

			//0415 Johnson modify because DisableWPFTabletSupport function has some issue need to clarify
			// To solve the issue of touch crash
			//try
			//{
			//	this.DisableWPFTabletSupport();
			//}
			//catch (Exception ex)
			//{
			//	LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			//}

			try
			{
				if (null != this.AppEvent)
				{
					this.AppEvent = null;
				}

				if (null != this._singleInstance)
				{
					this._singleInstance.Dispose();
				}

				//if (null != this._connectingWorker)
				//{
				//	this.ConnectingDeviceInfo = null;
				//	this.AbortConnectingThread();
				//}
				//else
				//{
				//	this.ClearAllConnection();
				//}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			try
			{
				if (null != this._manager)
				{
					this._manager.Dispose();
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
		}

		#endregion Application-related Methods

		#region Private Methods

		private async Task RestartScreen(AppDeviceInfo deviceInfo, bool isResetDevice)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for entering lock area...", "");

			await Task.Run(delegate()
			{
				// For locking triggers from different thread
				lock (_restartingScreenCastingLockObj)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

					try
					{
						if (null == this._connectingWorker)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._connectingWorker is null", "");

							this.CreateNewConnection(deviceInfo, isResetDevice);
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._connectingWorker is not null", "");

							// There is an existed connecting work and connecting failed, to abort existed connecting work.
							this.AbortConnectingThread();

							//// Wait for the cancelling work is finished.
							//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for the signal of cancelling process...", "");
							//this._resetEventForWorkCancenlled.WaitOne();
							//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Got the signal of cancelling process!", "");

							// Then to create an new one
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Create an new connection", "");
							this.CreateNewConnection(deviceInfo, isResetDevice);
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
				}
			}).ConfigureAwait(false);
		}

		private void DisableWPFTabletSupport()
		{
			// To solve the issue of crash after pressing button with touch
			// Reference: http://msdn.microsoft.com/en-us/library/vstudio/ee230087%28v=vs.100%29.aspx

			// Get a collection of the tablet devices for this window.
			TabletDeviceCollection devices = System.Windows.Input.Tablet.TabletDevices;

			if (devices.Count > 0)
			{
				// Get the Type of InputManager.
				Type inputManagerType = typeof(System.Windows.Input.InputManager);

				// Call the StylusLogic method on the InputManager.Current instance.
				object stylusLogic = inputManagerType.InvokeMember("StylusLogic", BindingFlags.GetProperty | BindingFlags.Instance | BindingFlags.NonPublic, null, InputManager.Current, null);

				if (stylusLogic != null)
				{
					//  Get the type of the stylusLogic returned from the call to StylusLogic.
					Type stylusLogicType = stylusLogic.GetType();

					// Loop until there are no more devices to remove.
					while (devices.Count > 0)
					{
						// Remove the first tablet device in the devices collection.
						stylusLogicType.InvokeMember("OnTabletRemoved",
								BindingFlags.InvokeMethod | BindingFlags.Instance | BindingFlags.NonPublic,
								null, stylusLogic, new object[] { (uint)0 });
					}
				}
			}
		}

		//private void RestartScreenCastingActions(AppDeviceInfo deviceInfo, bool isResetDevice)
		//{
		//	if (null != this.ConnectingDeviceInfo)
		//	{
		//		if (this.ConnectingDeviceInfo.Device.InstanceID.Equals(deviceInfo.Device.InstanceID) && isResetDevice == false)
		//		{
		//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "To activate screen casting window", "");
		//			this.Dispatcher.Invoke(delegate()
		//			{
		//				if (null != this.AppScreenWindow)
		//				{
		//					this.AppScreenWindow.WindowState = WindowState.Normal;
		//					this.AppScreenWindow.Activate();
		//				}
		//				else
		//				{
		//					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
		//				}
		//			});
		//		}
		//		else
		//		{
		//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Run this.RestartScreen() with isResetDevice: " + isResetDevice, "");
		//			//this.InitialShowScreencastWindow();
		//			this.RestartScreen(deviceInfo, isResetDevice);
		//		}
		//	}
		//	else
		//	{
		//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "Run this.RestartScreen() with isResetDevice: " + isResetDevice, "");
		//		//this.InitialShowScreencastWindow();
		//		this.RestartScreen(deviceInfo, isResetDevice);
		//	}
		//}

		private void CloseScreenCasting()
		{
			this._isShowTCPDisconnection = false;

			if (null != this._connectingWorker)
			{
				this.ConnectingDeviceInfo = null;

				//Johnson 0416 Add for debug Close window deadlock issue
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "WindowEventType.CloseScreenCasting to AbortConnectingThread", "");

				this.AbortConnectingThread();
			}
			else
			{
				this.ClearAllConnection();
			}
		}

		//0609 Johnon Modify it
		/// <summary>
		/// Notifies the tray icon to launch AcerEXTEND
		/// </summary>
		private void NotifyTray()
		{
			try
			{
				LogtoFile.TrackEnter();

				IntPtr hTargetWnd = Win32Api.NativeMethod.FindWindow(null, Configs.ProductTray);
				if (hTargetWnd == IntPtr.Zero)
				{
					return;
				}

				int result = Win32Api.NativeMethod.PostMessage(hTargetWnd, Win32Api.WM_ShowUp, IntPtr.Zero, IntPtr.Zero);
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "After PostMessage, result: " + result, "");

				result = Marshal.GetLastWin32Error();
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "After GetLastWin32Error, result: " + result, "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Checks the environment network. If no network, show customer message box to hint user.
		/// </summary>
		/// <returns>Bolean value</returns>
		private bool CheckEnvironmentNetwork()
		{
			LogtoFile.TrackEnter();
			try
			{
				//0408 Johnson add for Check Network Status
				if (!System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable())
				{
					string NoNetworkContent = (string)Application.Current.TryFindResource("txtNoNetwork");
					string ButtonContent = (string)Application.Current.TryFindResource("txtOK");
					CustomMessageBox NoNetworkMessageBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																	NoNetworkContent, CustomMessageBox.CustomImageIcon.Warring, ButtonContent));
					NoNetworkMessageBox.ShowDialog();
					return false;
				}
				else
					return true;
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Shows the device manager and Collapsed Screencast Window
		/// </summary>
		private void ShowDeviceManager()
		{
			LogtoFile.TrackEnter();
			// The last launched device isn't in paired device list
			this.Dispatcher.BeginInvoke(new Action(() =>
			{
				if (null != this.AppScreenWindow)
				{
					this.AppScreenWindow.Visibility = Visibility.Collapsed;
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
				}

				if (null != this.AppDeviceManagerWindow)
				{
					this.AppDeviceManagerWindow.Visibility = Visibility.Visible;
					this.AppDeviceManagerWindow.WindowState = WindowState.Normal;
					this.AppDeviceManagerWindow.Activate();
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppDeviceManagerWindow is null", "");
				}
			}));
			LogtoFile.TrackLeave();
		}

		public void ToShowDeviceManager(DeviceManagerTab deviceManagerTab = DeviceManagerTab.DeviceManager_Pairing, bool isHideScreenwindow = false)
		{
			LogtoFile.TrackEnter();

			this.Dispatcher.Invoke(delegate()
			{
				if (isHideScreenwindow)
				{
					if (null != this.AppScreenWindow)
					{
						this.AppScreenWindow.Visibility = Visibility.Collapsed;
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
					}
				}

				if (null != this.AppDeviceManagerWindow)
				{
					if (this.AppDeviceManagerWindow.Visibility != Visibility.Visible)
					{
						this.AppDeviceManagerWindow.Visibility = Visibility.Visible;
					}
					else
					{
						this.AppDeviceManagerWindow.WindowState = WindowState.Normal;
						this.AppDeviceManagerWindow.Activate();
					}

					if (deviceManagerTab == DeviceManagerTab.DeviceManager_Pairing)
						this.AppDeviceManagerWindow.rdbtnPairing.IsChecked = true;
					else if (deviceManagerTab == DeviceManagerTab.DeviceManager_Paired)
						this.AppDeviceManagerWindow.rdbtnPairedDevices.IsChecked = true;
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppDeviceManagerWindow is null", "");
				}
			});

			this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
			{
				if (null != this._appScreenWindow)
				{
					if (this._appScreenWindow.Visibility == Visibility.Visible)
					{
						System.Drawing.Rectangle currentScreenWorkingArea = this.GetScreen(this._appScreenWindow).WorkingArea;
						if (currentScreenWorkingArea.Width > (this._appScreenWindow.Left + this._appScreenWindow.Width + this.AppDeviceManagerWindow.Width))
						{
							this.AppDeviceManagerWindow.Left = this._appScreenWindow.Left + this._appScreenWindow.Width;
							this.AppDeviceManagerWindow.Top = this._appScreenWindow.Top;
						}
						else
						{
							this.AppDeviceManagerWindow.Left = this._appScreenWindow.Left - this.AppDeviceManagerWindow.Width;
							this.AppDeviceManagerWindow.Top = this._appScreenWindow.Top;
						}
					}
				}

				this.AppDeviceManagerWindow.ReadyToShow();
			});

			LogtoFile.TrackLeave();
		}

		/// <summary>
		/// Shows the Screencast window and Collapsed DeviceManager Widnows
		/// </summary>
		private void InitialShowScreencastWindow()
		{
			try
			{
				LogtoFile.TrackEnter();

				// Inform UI to launch screen casting window
				//this.SendAppEventAsync(new AppEventArgs(AppEventType.LaunchScreenCastWindow));

				this._appScreenWindow.SetInitialScreencastWindowInfo(this.ConnectingDeviceInfo, RemoteDevicePermission.NotDetect);

				// If Device Manager is open, to close it
				this.Dispatcher.BeginInvoke(new Action(() =>
				{
					if (null != this.AppDeviceManagerWindow)
					{
						this.AppDeviceManagerWindow.Visibility = Visibility.Collapsed;
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "ShowScreencastWindow Device Manager is null", "");
					}
				}));
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		public AppDeviceInfo GetLastLaunchDeviceInfo()
		{
			AppDeviceInfo deviceInfo = null;

			this.GetLastLaunchDeviceID();
			if (!this.LastLaunchID.Equals(Guid.Empty))
			{
				AFUserDevice device = this._manager.UserDeviceManager.DeviceStore.GetPairedDevices().ToList().Where(x => x.InstanceID.Equals(this.LastLaunchID)).FirstOrDefault();

				if (null != device && null != device.SessionID && null != device.Address)
				{
					deviceInfo = this.InitialAppDeviceInfo(device);
				}
			}

			return deviceInfo;
		}

		/// <summary>
		/// Prepares the Screencast connection.
		/// </summary>
		private void PrepareScreenConnection()
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

			//0609 Johnson modify for Check Network Status
			if (!this.CheckEnvironmentNetwork())
			{
				return;
			}

			try
			{
				if (!this.LastLaunchID.Equals(Guid.Empty))
				{
					// Check if the last launched device is existed in paired devices load from AFManager component or not
					AFUserDevice device = this._manager.UserDeviceManager.DeviceStore.GetPairedDevices().ToList().Where(x => x.InstanceID.Equals(this.LastLaunchID)).FirstOrDefault();

					if (null != device && null != device.SessionID && null != device.Address)
					{
						AppDeviceInfo deviceInfo = this.InitialAppDeviceInfo(device);

						//this.StartConnectRemoteDevice(deviceInfo);
						this.Dispatcher.Invoke(delegate()
						{
							if (FlowControlManager.Instance.IsAcceptNewConnection())
							{
								//this.InitialShowScreencastWindow();

								this._appScreenWindow.ConnectNewScreencast(deviceInfo);
							}
						});
					}
					else
					{
						if (null == device)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.LastLaunchID isn't in DeviceStore", "");

							// The last launched device isn't in paired device list
							this.ToShowDeviceManager();
						}
						else
						{
							if (device.SessionID == null)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device SessionID is null", "");
							}

							if (device.Address == null)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Device Address is null", "");
							}

							device.UnPair();
						}
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.LastLaunchID == Guid.Empty", "");

					// There is no last launched device, to launch device manager window
					this.ToShowDeviceManager();
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		public void StartConnectRemoteDevice(AppDeviceInfo deviceInfo)
		{
			LogtoFile.TrackEnter();

			this.ConnectingDeviceInfo = deviceInfo;

			this.InitialShowScreencastWindow();

			//this._appScreenWindow.SetInitialScreencastWindowInfo(deviceInfo, RemoteDevicePermission.NotDetect);

			// Call RestartScreen async, not await
			//this.RestartScreen(deviceInfo, true);
			//this.Dispatcher.BeginInvoke(new Action(() =>
			//{
			FlowControlManager.Instance.StartNewConnection(deviceInfo, this._appScreenWindow);
			//}));

			LogtoFile.TrackLeave();
		}

		/// <summary>
		/// Initializes the required UI components.
		/// </summary>
		private void InitializeRequiredUIComponents()
		{
			LogtoFile.TrackEnter();

			try
			{
				// Initialize tray icon
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Create TrayIconWindow Resource", "");
				this.AppTrayIconWindow = new TrayIconWindow();
				this.AppTrayIconWindow.Title = Configs.ProductTray;
				this.AppTrayIconWindow.WindowEvent += this.WindowEventCallback;
				this.AppTrayIconWindow.AppTaskbarIcon.ToolTipText = Application.Current.TryFindResource("txtAppName") as String;
				this.AppTrayIconWindow.Show();
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish TrayIconWindow Resource", "");

				// Initialize device manager window
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Create DeviceManagerWindow Resource", "");
				this.AppDeviceManagerWindow = new DeviceManagerWindow();
				this.AppDeviceManagerWindow.Title = Application.Current.TryFindResource("txtAppName") as String;
				this.AppDeviceManagerWindow.WindowEvent += this.WindowEventCallback;
				this.AppEvent += this.AppDeviceManagerWindow.AppEventCallback;
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish DeviceManagerWindow Resource", "");

				// Initialize screen window
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Create ScreenWindow Resource", "");
				this.AppScreenWindow = new ScreenWindow();
				this.AppScreenWindow.Title = Application.Current.TryFindResource("txtAppName") as String;
				this.AppScreenWindow.WindowEvent += this.WindowEventCallback;
				this.AppEvent += this.AppScreenWindow.AppEventCallback; // Let main window to subscribe App event

				//0611 Johnson add for Flow control event to send Screenwindow
				FlowControlManager.Instance.AppEvent += this.AppScreenWindow.AppEventCallback;
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish ScreenWindow Resource", "");

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Create Timer Resource", "");
				// Initialize timer of waiting for pairing
				this.WaitForPairingTimer = new System.Timers.Timer();
				this.WaitForPairingTimer.Interval = Configs.WaitForPairingTimeoutPeriod;
				this.WaitForPairingTimer.Elapsed += new ElapsedEventHandler(this.WaitForPairingTimerTickCallBack);

				// Initialize timer of waiting for receiving video data from filter
				this.WaitForReceiveDataTimer = new System.Timers.Timer();
				this.WaitForReceiveDataTimer.Interval = Configs.WaitForReceiveDataTimeoutPeriod;
				this.WaitForReceiveDataTimer.Elapsed += new ElapsedEventHandler(this.WaitForReceiveDataTimerTickCallBack);
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish Timer Resource", "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Quicks the launch acer extend.
		/// </summary>
		private void QuickLaunchAcerEXTEND()
		{
			try
			{
				LogtoFile.TrackEnter();

				if (null != this.AppScreenWindow)
				{
					if (this.AppScreenWindow.Visibility == Visibility.Visible)
					{
						this.Dispatcher.BeginInvoke(new Action(() =>
						{
							this.AppScreenWindow.WindowState = WindowState.Normal;
							this.AppScreenWindow.Activate();
						}));
					}
					else
					{
						this.PrepareScreenConnection();
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Shows the web service timeout message.
		/// </summary>
		///
		private void ShowWebServiceTimeoutMessage()
		{
			this.Dispatcher.BeginInvoke(new Action(() =>
			{
				string TimeoutContent = ReadingResoureceWording("txtGeneralAlertDetail");// "Operation Fail, please try again";
				string ButtonContent = (string)Application.Current.TryFindResource("txtOK");
				CustomMessageBox NoNetworkMessageBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																TimeoutContent, CustomMessageBox.CustomImageIcon.Warring, ButtonContent));
				NoNetworkMessageBox.ShowDialog();
			}));
		}

		#endregion Private Methods

		#region Protected Methods

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion Protected Methods

		#region Public Methods

		/// <summary>
		/// Doble click tray icon event
		/// </summary>
		public void TrayDoubleClicked()
		{
			this.QuickLaunchAcerEXTEND();
		}

		/// <summary>
		/// For use click check for updates event
		/// </summary>
		/// <returns></returns>
		public async Task UserCheckforUpdatesAsync()
		{
			UpgradeTool.ExitValue exitValue = await Task<UpgradeTool.ExitValue>.Run(() =>
			{
				return UpgradeTool.SelfCheckUpdate(UpgradeTool.LaunchType.UserCheckUpdate);
			});

			if (exitValue == UpgradeTool.ExitValue.NONetWork)
			{
				//MessageBox.Show(Application.Current.TryFindResource("txtUpgradeToolNONetWork") as String);
				string noNetworkContent = (string)Application.Current.TryFindResource("txtUpgradeToolNONetWork");
				string buttonContent = (string)Application.Current.TryFindResource("txtOK");
				CustomMessageBox noNetworkMessageBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																noNetworkContent, CustomMessageBox.CustomImageIcon.Warring, buttonContent));
				noNetworkMessageBox.Show();
			}
			else if (exitValue == UpgradeTool.ExitValue.NONewVersion)
			{
				//MessageBox.Show(Application.Current.TryFindResource("txtUpgradeToolNONewVersion") as String);
				string noNewVersionContent = (string)Application.Current.TryFindResource("txtUpgradeToolNONewVersion");
				string buttonContent = (string)Application.Current.TryFindResource("txtOK");
				CustomMessageBox noNewVersionBox = new CustomMessageBox(new AirFusion.Windows.CustomMessageBox.CustomMessageBoxStructInfo(
																noNewVersionContent, CustomMessageBox.CustomImageIcon.Warring, buttonContent));
				noNewVersionBox.Show();
			}
		}

		public void LinkToAcerEXTENDPlayStore()
		{
			try
			{
				System.Diagnostics.Process.Start("https://play.google.com/store/apps/details?id=com.acer.android.airfusion");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "click LinkToStore() exception : " + ex.ToString(), "");
			}
		}

		#endregion Public Methods

		#region Callbacks

		private async void WindowEventCallback(object sender, WindowEventArgs e)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get WindowEventType: " + e.EventType.GetStringValue(), "");
			switch (e.EventType)
			{
				case WindowEventType.UnpairResponse:
					{
						if (null != e.Data)
						{
							if (null != e.Data && e.Data is AFUserDevice)
							{
								bool isAccept = (bool)e.Args;
								if (isAccept)
								{
									AFUserDevice device = e.Data as AFUserDevice;
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start device Unpair", "");
									device.UnPair();

									if (this._connecingdeviceInfo != null)
									{
										if (device.Equals(this._connecingdeviceInfo.Device))
										{
											if (this._appScreenWindow != null)
												this._appScreenWindow.DisconnectScreencast();
										}
									}
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "End device Unpair", "");
								}
							}
						}
					}
					break;

				case WindowEventType.VNCSetScreenSize:
					{
						if (e.Data is int[])
						{
							int[] size = e.Data as int[];
							int width = size[0];
							int height = size[1];
							int xDelta = size[2];
							int yDelta = size[3];

							if (null != FlowControlManager.Instance.InputController)
							{
								if (null != FlowControlManager.Instance.InputController.VNCClient)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start this.VNCController.VNCClient.SetAirFusionScreenSize", "");
									VNC.SCREEN_SIZE_INFO screenInfo = new VNC.SCREEN_SIZE_INFO
									{
										uiWidth = (uint)width,
										uiHeight = (uint)height,
										iDeltaX = xDelta,
										iDeltaY = yDelta,
										fDpiRate = (float)(1 / this.DPIRatio),
									};

									FlowControlManager.Instance.InputController.VNCClient.SetAirFusionScreenInfo(screenInfo);
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.VNCController.VNCClient.SetAirFusionScreenSize -- Done", "");
								}
							}
						}
					}
					break;

				case WindowEventType.UploadFiles:
					{
						if (e.Data is string[])
						{
							string[] temp = e.Data as string[];
							_nTotalFileCount = temp.Length;

							// Use background worker to transfer file, not use UI thread
							BackgroundWorker bw = new BackgroundWorker();
							bw.DoWork += delegate(object s, DoWorkEventArgs k)
							{
								string[] fileOut = (string[])k.Argument;

								this.SendAppEventAsync(new AppEventArgs(AppEventType.UploadAllFileStart));

								foreach (string file in fileOut)
								{
									FlowControlManager.Instance.InputController.VNCClient.UploadOneFileToServer(file);

									// Work around, to temporarily solve timing issue of transferring files
									Thread.Sleep(1000);
								}

								this.SendAppEventAsync(new AppEventArgs(AppEventType.UploadAllFileEnd));
							};

							bw.RunWorkerAsync(temp);
						}
					}
					break;

				case WindowEventType.CloseScreenCasting:
					{
						if (this._appScreenWindow != null)
							this._appScreenWindow.DisconnectScreencast();
					}
					break;

				case WindowEventType.WakeUp:
					{
						if (null != ConnectingDeviceInfo)
						{
							try
							{
								Task task = Task.Run(async delegate()
								{
									try
									{
										await this.ConnectingDeviceInfo.Device.WakeUpAsync();
									}
									catch (Exception ex)
									{
										LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
									}
								});
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");

								this.SendAppEventAsync(new AppEventArgs(AppEventType.NoBTCapability));
							}
						}
					}
					break;

				case WindowEventType.RestartScreenCasting:
					{
						if (null != e.Data && e.Data is AppDeviceInfo)
						{
							AppDeviceInfo deviceInfo = e.Data as AppDeviceInfo;

							try
							{
								this._isShowTCPDisconnection = false;
								//this.RestartScreenCastingActions(deviceInfo, (bool)e.Args);

								if (null != deviceInfo && ((null == this.ConnectingDeviceInfo) || (null != this.ConnectingDeviceInfo && !this.ConnectingDeviceInfo.Device.InstanceID.Equals(deviceInfo.Device.InstanceID))))
								{
									this._appScreenWindow.ConnectNewScreencast(deviceInfo);
								}
								else
								{
									if (null != this.ConnectingDeviceInfo && this.ConnectingDeviceInfo.Device.InstanceID.Equals(deviceInfo.Device.InstanceID))
									{
										this.Dispatcher.Invoke(delegate()
										{
											this._appScreenWindow.WindowState = WindowState.Maximized;
										});
									}
								}
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Data is null", "");
						}
					}
					break;

				case WindowEventType.ShowDeviceManager:
					{
						if (e.Args != null)
						{
							if ((DeviceManagerTab)e.Args == DeviceManagerTab.DeviceManager_Paired)
								this.ToShowDeviceManager(DeviceManagerTab.DeviceManager_Paired);
							else
								this.ToShowDeviceManager();
						}
						else
						{
							this.ToShowDeviceManager();
						}
						//this.Dispatcher.Invoke(delegate()
						//{
						//	if (null != this.AppDeviceManagerWindow)
						//	{
						//		if (this.AppDeviceManagerWindow.Visibility != Visibility.Visible)
						//		{
						//			this.AppDeviceManagerWindow.Visibility = Visibility.Visible;
						//		}
						//		else
						//		{
						//			this.AppDeviceManagerWindow.WindowState = WindowState.Normal;
						//			this.AppDeviceManagerWindow.Activate();
						//		}

						//		if (e.Args != null)
						//		{
						//			if ((DeviceManagerTab)e.Args == DeviceManagerTab.DeviceManager_Pairing)
						//				this.AppDeviceManagerWindow.rdbtnPairing.IsChecked = true;
						//			else if ((DeviceManagerTab)e.Args == DeviceManagerTab.DeviceManager_Paired)
						//				this.AppDeviceManagerWindow.rdbtnPairedDevices.IsChecked = true;
						//		}
						//	}
						//	else
						//	{
						//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppDeviceManagerWindow is null", "");
						//	}
						//});

						//this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
						//{
						//	if (null != this._appScreenWindow)
						//	{
						//		if (this._appScreenWindow.Visibility == Visibility.Visible)
						//		{
						//			System.Drawing.Rectangle currentScreenWorkingArea = this.GetScreen(this._appScreenWindow).WorkingArea;
						//			if (currentScreenWorkingArea.Width > (this._appScreenWindow.Left + this._appScreenWindow.Width + this.AppDeviceManagerWindow.Width))
						//			{
						//				this.AppDeviceManagerWindow.Left = this._appScreenWindow.Left + this._appScreenWindow.Width;
						//				this.AppDeviceManagerWindow.Top = this._appScreenWindow.Top;
						//			}
						//			else
						//			{
						//				this.AppDeviceManagerWindow.Left = this._appScreenWindow.Left - this.AppDeviceManagerWindow.Width;
						//				this.AppDeviceManagerWindow.Top = this._appScreenWindow.Top;
						//			}

						//			this.AppDeviceManagerWindow.ReadyToShow();
						//		}
						//	}
						//});
					}
					break;

				case WindowEventType.ExistedLaunch:
					{
						this.QuickLaunchAcerEXTEND();
					}
					break;

				case WindowEventType.Exit:
					{
						//Johnson 0416 Add for debug Close window deadlock issue
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Enter WindowEventType.Exit", "");

						//Johnson 0507 add should close screen cast windows first when call close screen cast
						//the function will automatically close screen connection.
						if (null != this.AppScreenWindow)
						{
							//Johnson 0416 Add for debug Close window deadlock issue
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "WindowEventType.Exit ScreenWindow Close", "");

							this.AppScreenWindow.Close();
						}

						// To call ClearAllConnection(true) to identify calling this function is from exit event
						try
						{
							if (null != this.AppDeviceManagerWindow)
							{
								//Johnson 0416 Add for debug Close window deadlock issue
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "WindowEventType.Exit Device Manager Windows Close", "");

								this.AppDeviceManagerWindow.Close();
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppDeviceManagerWindow is null", "");
							}
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
						}
						this.Dispatcher.Invoke(delegate()
						{
							try
							{
								if (null != this.AppTrayIconWindow)
								{
									//Johnson 0416 Add for debug Close window deadlock issue
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "WindowEventType.Exit TrayIcon Close", "");

									this.AppTrayIconWindow.Close();
								}
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}

							//Johnson 0416 Add for debug Close window deadlock issue
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Exit WindowEventType.Exit", "");

							try
							{
								Application.Current.Shutdown();
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						});
					}
					break;

				case WindowEventType.SetDiscoverable:
					{
						if (null != e.Args)
						{
							bool isDiscoverable = (bool)e.Args;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "isDisoverable is : " + isDiscoverable, "");

							if (isDiscoverable)
							{
								this.WaitForPairingTimer.Interval = Configs.WaitForPairingTimeoutPeriod;
								this.WaitForPairingTimer.Start();
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start the timer of wairing for pairing", "");
								if (null != this._manager)
								{
									this._manager.UserDeviceManager.EnableDiscoverable();
								}
							}
							else
							{
								this.WaitForPairingTimer.Stop();
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Stop the timer of wairing for pairing", "");
								if (null != this._manager)
								{
									this._manager.UserDeviceManager.DisableDiscoverable();
								}
							}
						}
					}
					break;

				case WindowEventType.NavigationBackClicked:
					{
						if (null != FlowControlManager.Instance.InputController)
						{
							if (null != FlowControlManager.Instance.InputController.VNCClient)
							{
								FlowControlManager.Instance.InputController.VNCClient.SendInputVirtualKey(VNC.VIRTUAL_KEY_TYPE.ESC);
							}
						}
					}
					break;

				case WindowEventType.NavigationHomeClicked:
					{
						if (null != FlowControlManager.Instance.InputController)
						{
							if (null != FlowControlManager.Instance.InputController.VNCClient)
							{
								FlowControlManager.Instance.InputController.VNCClient.SendInputVirtualKey(VNC.VIRTUAL_KEY_TYPE.HOME);
							}
						}
					}
					break;

				case WindowEventType.NavigationRecentClicked:
					{
						if (null != FlowControlManager.Instance.InputController)
						{
							if (null != FlowControlManager.Instance.InputController.VNCClient)
							{
								FlowControlManager.Instance.InputController.VNCClient.SendInputVirtualKey(VNC.VIRTUAL_KEY_TYPE.History);
							}
						}
					}
					break;

				// 2015-05-20 Seoker: remove Auto-Login functions since it's defeatured
				/*
				case WindowEventType.SetAccountPassword:
					{
						if (null != e.Data && e.Data is AppDeviceInfo)
						{
							AppDeviceInfo deviceInfo = e.Data as AppDeviceInfo;

							if (null != e.Args)
							{
								try
								{
									string account = Utils.GetAccountDefaultName();
									string password = e.Args as string;

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Account Name: " + account, "");
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Password: " + password, "");

									if (null != _pipeManager)
									{
										this._pipeManager.RecordALInfo(deviceInfo.Device.Bluetooth.Address, account, password);
									}
								}
								catch (Exception ex)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
								}
							}
						}
					}
					break;

				case WindowEventType.DeleteAccountPassword:
					{
						if (null != e.Data && e.Data is AppDeviceInfo)
						{
							AppDeviceInfo deviceInfo = e.Data as AppDeviceInfo;

							if (null != _pipeManager)
							{
								this._pipeManager.DeleteALInfo(deviceInfo.Device.Bluetooth.Address);
							}
						}
					}
					break;
				*/

				case WindowEventType.CheckReceiveData:
					{
						Task.Run(() =>
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "To wait for starting timer", "");
							//if (this._signalOfReceiveDataTimerStarted.WaitOne(5000))
							{
								if (this._waitForReceiveDataTimer != null)
									this.WaitForReceiveDataTimer.Stop();

								if (this._signalOfReceiveDataTimerStarted != null)
									this._signalOfReceiveDataTimerStarted.Reset();

								if (null != this.AppEvent)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of receiving data completed", "");
									this._isShowTCPDisconnection = true;
									this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.ReceiveDataCompleted));
								}
								else
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppEvent is null", "");
								}
							}
							//else
							//{
							//	LogtoFile.WriteLog("", Configs.DebugLevelValue, "WindowEventType.CheckReceiveData Timer is null", "");
							//}
						});
					}
					break;

				case WindowEventType.SwitchRotationClicked:
					{
						LogtoFile.TrackEnter();
						try
						{
							if (null != this.ConnectingDeviceInfo)
							{
								if (null != this.ConnectingDeviceInfo.Device)
								{
									if (this.ConnectingDeviceInfo.Device.Display.Rotation.HasValue)
									{
										AFUserDevice.DisplayInfo.DisplayRotation targetRotation = AFUserDevice.DisplayInfo.DisplayRotation.Rotation0;
										if (this.ConnectingDeviceInfo.Device.Display.Rotation == AFUserDevice.DisplayInfo.DisplayRotation.Rotation0 ||
											this.ConnectingDeviceInfo.Device.Display.Rotation == AFUserDevice.DisplayInfo.DisplayRotation.Rotation180)
										{
											targetRotation = AFUserDevice.DisplayInfo.DisplayRotation.Rotation90;
										}

										Task t = Task.Run(async delegate()
										{
											try
											{
												await this.ConnectingDeviceInfo.Device.RotateScreenAsync(targetRotation);
											}
											catch (TimeoutException ex)
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "SwitchRotationClicked WebService Timeout : " + ex.ToString(), "");
												this.ShowWebServiceTimeoutMessage();
											}
											catch (Exception ex)
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
											}
										});
									}
									else
									{
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "Can't get rotation of device", "");
									}
								}
								else
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connecting device is null", "");
								}
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connecting device information is null", "");
							}
						}
						finally
						{
							LogtoFile.TrackLeave();
						}
					}
					break;

				case WindowEventType.SwitchKeyboardClicked:
					{
						LogtoFile.TrackEnter();
						try
						{
							if (null != this.ConnectingDeviceInfo)
							{
								if (null != this.ConnectingDeviceInfo.Device)
								{
									if (null != e.Args)
									{
										if (e.Args is bool)
										{
											bool isVitual = (bool)e.Args;
											var keyboardMode = isVitual ? KeyboardMode.Virtual : KeyboardMode.Physical;

											Task t = Task.Run(async delegate()
											{
												try
												{
													await this.ConnectingDeviceInfo.Device.SwitchKeyboardModeAsync(keyboardMode);
												}
												catch (TimeoutException ex)
												{
													LogtoFile.WriteLog("", Configs.DebugLevelValue, "SwitchKeyboardClicked WebService Timeout : " + ex.ToString(), "");
													this.ShowWebServiceTimeoutMessage();
												}
												catch (Exception ex)
												{
													LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
												}
											});
										}
									}
								}
								else
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connecting device is null", "");
								}
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connecting device information is null", "");
							}
						}
						finally
						{
							LogtoFile.TrackLeave();
						}
					}
					break;

				case WindowEventType.PairAgain:
					{
						this.Dispatcher.Invoke(delegate()
						{
							if (null != this.AppScreenWindow)
							{
								try
								{
									this.AppScreenWindow.Close();
								}
								catch (Exception ex)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
								}
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppScreenWindow is null", "");
							}

							if (null != this.AppDeviceManagerWindow)
							{
								this.AppDeviceManagerWindow.Visibility = Visibility.Visible;
								this.AppDeviceManagerWindow.WindowState = WindowState.Normal;
								this.AppDeviceManagerWindow.Activate();
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppDeviceManagerWindow is null", "");
							}
						});
					}
					break;

				case WindowEventType.GetSnapshot:
					{
						if (null != e.Data)
						{
							string filePullPath = e.Data as String;

							try
							{
								if (null != this._generalNotificationManager)
								{
									string title = Application.Current.TryFindResource("txtGetSnapshot") as String;

									BitmapSource icon = Utils.CreateBitmapSourceByFile(filePullPath);

									this._generalNotificationManager.CreateAndShowToastOfSnapshot(title, Path.GetFileName(filePullPath), icon, filePullPath);
								}
								else
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._generalNotificationManager is null", "");
								}
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						}
					}
					break;

				case WindowEventType.TCPDisconnected:
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "WindowEventType.TCPDisconnected", "");

						if (null != this.AppEvent && _isShowTCPDisconnection)
						{
							AppDeviceInfo disConnectDeviceInfo = this.ConnectingDeviceInfo;

							if (null != disConnectDeviceInfo)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Before clean, disConnectDeviceInfo is not null", "");
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Before clean, disConnectDeviceInfo is null", "");
							}

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of TCP Disconnected", "");
							this.SendAppEventAsync(new AppEventArgs(AppEventType.ScreenCastTCPDIsconnected, disConnectDeviceInfo));
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppEvent is null", "");
						}
					}
					break;

				default:
					break;
			}
		}

		/// <summary>
		/// Gets the located screen of the target window
		/// </summary>
		/// <param name="window">target window</param>
		/// <returns>located screen</returns>
		private System.Windows.Forms.Screen GetScreen(Window window)
		{
			return System.Windows.Forms.Screen.FromHandle(new WindowInteropHelper(window).Handle);
		}

		private void WaitForPairingTimerTickCallBack(object sender, System.Timers.ElapsedEventArgs e)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "The timer of waiting for pairing is ticked", "");

			(sender as System.Timers.Timer).Stop();

			if (null != this._manager)
			{
				this._manager.UserDeviceManager.DisableDiscoverable();
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._manager is null", "");
			}

			if (null != this.AppEvent)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send the event of waiting for pairing time out", "");
				this.SendAppEventAsync(new AppEventArgs(AppEventType.WaitForPairingTimeout));
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppEvent is null", "");
			}
		}

		private void WaitForReceiveDataTimerTickCallBack(object sender, System.Timers.ElapsedEventArgs e)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "The timer of waiting for receiving data is ticked", "");

			(sender as System.Timers.Timer).Stop();
			this._signalOfReceiveDataTimerStarted.Reset();

			if (null != this.AppEvent)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Failed to send the event of receiving data", "");
				this.SendAppEventAsync(new AppEventArgs(AppEventType.InitialStatus, InitialStatus.ReceiveDataFailed));
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.AppEvent is null", "");
			}
		}

		#endregion Callbacks
	}
}