﻿using Microsoft.Win32;
using System;
using System.Net;
using System.Net.NetworkInformation;
using System.Text;

namespace AirFusion
{
	public static class GoogleAnalytics
	{
		public enum SendGoogleAnalyticStatus
		{
			Success,
			InitialError,
			NoNetwork,
			HttpWebRequestError,
		}

		public struct GoogleAnalyticInfo
		{
			private string keyName;
			public string strKeyName
			{
				get
				{
					return string.IsNullOrEmpty(keyName) ? string.Empty : keyName;
				}

				set { keyName = value; }
			}

			private string keyValue;
			public string strKeyValue
			{
				get
				{
					return string.IsNullOrEmpty(keyValue) ? string.Empty : keyValue;
				}

				set { keyValue = value; }
			}

			private string applicationVersion;
			public string strApplicationVersion
			{
				get
				{
					return string.IsNullOrEmpty(applicationVersion) ? string.Empty : applicationVersion;
				}

				set { applicationVersion = value; }
			}

			private string deviceID;
			public string strDeviceID
			{
				get
				{
					return string.IsNullOrEmpty(deviceID) ? string.Empty : deviceID;
				}

				set { deviceID = value; }
			}
		}

		#region some parameter

		/// <summary>
		/// Google Analytics parameter. It need to change by each project
		/// </summary>
		private static string s_utmhn = "acerextend2.com";	//GA設定追蹤的首頁(需與ga系統上設定吻合)

		private static string s_utmac = "UA-64451741-2";	//追蹤單號(需與ga系統上設定吻合)
		private static string s_utmwv = "5.2.9";			// GA Version 5.2.9
		private static string s_utms = "1";
		private static string s_utmul = "zh-tw";			// language
		private static string s_utmr = "0";					// refer, ~=none
		private static string s_utmp = "/af/datacollect/";	// URI
		private static string c_domain_hash = "84097795";	// Domain hash
		private static int c_session = 1;					// record by client and ++?
		private static int c_source = 1;					// record by client and ++?

		#endregion some parameter

		public static SendGoogleAnalyticStatus GoogleAnalyticSend(string keyName, string value)
		{
			SendGoogleAnalyticStatus status = SendGoogleAnalyticStatus.InitialError;

			//Get AcerEXTEND version
			string UIVersion = String.Empty;
			try
			{
				bool isX86;
				object oRK = RegistryUtility.GetRegistry(RegistryHive.LocalMachine, "SOFTWARE\\Wow6432Node\\OEM\\AcerEXTEND\\Installer Property", "MSI_ProductDisplayVersion", RegistryView.Registry64, out isX86);
				UIVersion = (string)oRK;
			}
			catch
			{
				UIVersion = "unknow UIVersion";
			}

			//Get Device ID
			string DeviceID = string.Empty;
			try
			{
				bool isX86;
				object oRK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, "SOFTWARE\\OEM\\AcerEXTEND\\Dandelion", "device_id", RegistryView.Registry64, out isX86);
				DeviceID = (string)oRK;
			}
			catch
			{
				DeviceID = string.Empty;
			}

			status = GASend(new GoogleAnalyticInfo { strKeyName = keyName, strKeyValue = value, strApplicationVersion = UIVersion, strDeviceID = DeviceID });
			return status;
		}

		private static SendGoogleAnalyticStatus GASend(GoogleAnalyticInfo GAInfo)//(string set_KeyName, string set_KeyValue, string strDeviceID, string strApplicationVersion = string.Empty)
		{
			//first. check network available
			bool isNetworkAvailable = NetworkInterface.GetIsNetworkAvailable();
			if (!isNetworkAvailable)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Google Analytics must be connected to internet, but now do not connected it.", "");

				return SendGoogleAnalyticStatus.NoNetwork;
			}
			else
			{
				//second. send httprequest to google analytics.
				try
				{
					StringBuilder builder = new StringBuilder();
					builder.Append("http://www.google-analytics.com/__utm.gif");
					StringBuilder cookie = new StringBuilder();
                    builder.Append("?utmwv=" + s_utmwv); //Tracking code version
					builder.Append("&utms=" + s_utms);
					Random s_utmn = new Random(Guid.NewGuid().GetHashCode());
                    builder.Append("&utmn=" + s_utmn.Next(0, 2000000000)); //Unique ID generated for each GIF request to prevent caching of the GIF image.
                    builder.Append("&utmhn=" + s_utmhn); //Host Name, which is a URL-encoded string.

					//setting user-define
					string str_utme = "&utme=8(1!" + GAInfo.strKeyName + "*5!" + GAInfo.strKeyName + "_DeviceID" + ")9(1!" + GAInfo.strKeyValue + "*5!" + GAInfo.strDeviceID + "/" + GAInfo.strKeyValue + ")";
					builder.Append(str_utme);
					s_utmul = System.Globalization.CultureInfo.CurrentCulture.ToString();
                    builder.Append("&utmul=" + s_utmul); //Browser language.	
                    builder.Append("&utmr=" + s_utmr); //Referral, complete URL.
                    builder.Append("&utmp=" + s_utmp); //Page request of the current page.
                    builder.Append("&utmac=" + s_utmac); //Account String. Appears on all requests.

					// utam
					c_domain_hash = domainhash();
                    cookie.Append("&utmcc=__utma=" + c_domain_hash); //Cookie values. This request parameter sends all the cookies requested from the page.
					Random c_uid = new Random(Guid.NewGuid().GetHashCode());
					cookie.Append("." + c_uid.Next(0, 2000000000));
					IFormatProvider culture = new System.Globalization.CultureInfo(s_utmul);
					DateTime InitialTime = DateTime.ParseExact("20150701000000", "yyyyMMddHHmmss", culture); //GA正式統計起始日期
					DateTime CurrentTime = DateTime.Now;
					TimeSpan ts0 = new TimeSpan(InitialTime.Ticks);
					TimeSpan ts3 = new TimeSpan(CurrentTime.Ticks);
					TimeSpan Timestamp = ts3.Subtract(ts0).Duration();
					cookie.Append("." + (int)Timestamp.TotalSeconds); // First
					cookie.Append("." + (int)Timestamp.TotalSeconds); // Last
					cookie.Append("." + (int)Timestamp.TotalSeconds); // Current
					cookie.Append("." + c_session);

					// utmz
                    cookie.Append(";+__utmz=" + c_domain_hash); 

					//透過客戶開發-來源與媒介得知使用者的UI版號
					cookie.Append("." + (int)Timestamp.TotalSeconds); // First
					cookie.Append("." + c_session);
					cookie.Append("." + c_source);
					cookie.Append(".utmcsr=" + GAInfo.strDeviceID + "|utmccn=" + GAInfo.strDeviceID + "|utmcmd=" + GAInfo.strApplicationVersion);

					// utmv
					cookie.Append(";+__utmv=" + c_domain_hash);
					cookie.Append("." + GAInfo.strDeviceID);
					cookie.Append(";&utmu=q~");
					builder.Append(string.Format(cookie.ToString()));

					try
					{
						HttpWebRequest GArequest = (HttpWebRequest)WebRequest.Create(string.Format(builder.ToString()));
						GArequest.Timeout = 5000;
						GArequest.Method = "GET";
						GArequest.BeginGetResponse(new AsyncCallback(RespCallback), GArequest);
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Google analytics event (" + GAInfo.strKeyName + "=>" + GAInfo.strKeyValue + " from " + GAInfo.strDeviceID + ") .", "");
						return SendGoogleAnalyticStatus.Success;
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Google analytics http request Failure. ex:" + ex.ToString(), "");

						return SendGoogleAnalyticStatus.HttpWebRequestError;
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Google analytics GAsend() Failure. ex:" + ex.ToString(), "");

					return SendGoogleAnalyticStatus.InitialError;
				}
			}
		}

		private static void RespCallback(IAsyncResult asynchronousResult)
		{
			HttpWebRequest request = null;
			HttpWebResponse response = null;

			try
			{
				request = (HttpWebRequest)asynchronousResult.AsyncState;
				response = (HttpWebResponse)request.EndGetResponse(asynchronousResult);
                LogtoFile.WriteLog("", Configs.DebugLevelValue, "Google Analytics RespCallback() Success, the event was successful send to Server. ", "");
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Google Analytics RespCallback() Failure, the event was unsuccess. ", "");
			}
			finally
			{
				if (response != null)
					response.Close(); // Release the HttpWebResponse
			}
		}

		public static string domainhash()
		{
			string buffer;
			int shifter;
			int a = 0;
			int c = 0;
			for (int i = s_utmhn.Length - 1; i >= 0; i--)
			{
				buffer = "" + charCodeAt(i, s_utmhn);
				shifter = Int32.Parse(buffer);
				a = ((a * (int)Math.Pow(2, 6)) & 268435455) + shifter + (shifter * (int)Math.Pow(2, 14));
				c = a & 266338304;
				if (c != 0)
					a = a ^ c / (int)Math.Pow(2, 21);
			}
			return a.ToString();
		}

		public static byte charCodeAt(int n, string str)
		{
			byte[] bytes = System.Text.Encoding.UTF8.GetBytes(str);
			return bytes[n];
		}
	}
}