@ECHO off

pushd %~dp0

SET ReleaseFolder=%~1
SET SolutionFolder=%~2
SET ProjectName=%~3
SET ConfigurationName=%~4
SET PlatformName=%~5
SET ToolsFolder=%SolutionFolder%InstallerProject\Support\Tools

ECHO ReleaseFolder=%ReleaseFolder%
ECHO SolutionFolder=%SolutionFolder%
ECHO ProjectName=%ProjectName%
ECHO ConfigurationName=%ConfigurationName%
ECHO PlatformName=%PlatformName%

REM Get current OS architecture
SET ArchitectureName=X86
IF /I "%PlatformName%"=="Any CPU" (
	REM Detect OS architecture
	FOR /f %%a in ('wmic os get osarchitecture^|find /i "-bit"') DO SET "Bits=%%a"
	ECHO Current OS is %Bits%
	IF /I "%Bits%"=="64-bit" SET ArchitectureName=AMD64
) ELSE IF /I "%PlatformName%"=="x64" (
	SET ArchitectureName=AMD64
)

SET PostBuildCurrentFolder=%SolutionFolder%%ProjectName%\
SET NativeDLLFolderWithBuildPlatform=%SolutionFolder%\NativeDLL\[%ConfigurationName%]\%ArchitectureName%\
REM SET NativeDLLFiles="32feetWidcomm.dll" "btwapi.dll" "SoundTriggerforDesktop.dll" "PullSrcFilter.dll" "RotateFilter.dll" "VideoProc.dll" "InputControlHandler.dll" "AirFusionadb.exe" "AdbWinApi.dll" "AdbWinUsbApi.dll"
SET NativeDLLFiles="PullSrcFilter.dll" "InputControlHandler.dll" "AirFusionadb.exe" "AdbWinApi.dll" "AdbWinUsbApi.dll"
SET FinalReleaseFolder=%SolutionFolder%\FinalRelease\%ProjectName%\%ConfigurationName%\%PlatformName%
SET InstallerSourceX64Folder=%SolutionFolder%\FinalRelease\InstallerSource\%ConfigurationName%\AMD64
SET InstallerSourceX86Folder=%SolutionFolder%\FinalRelease\InstallerSource\%ConfigurationName%\X86

ECHO PostBuildCurrentFolder=%PostBuildCurrentFolder%
ECHO NativeDLLFolderWithBuildPlatform=%NativeDLLFolderWithBuildPlatform%
ECHO NativeDLLFiles=%NativeDLLFiles%
ECHO ArchitectureName=%ArchitectureName%
ECHO FinalReleaseFolder=%FinalReleaseFolder%
ECHO InstallerSourceX64Folder=%InstallerSourceX64Folder%
ECHO InstallerSourceX86Folder=%InstallerSourceX86Folder%

:CopyToFinalReleaseFolder
REM Create folders
IF NOT EXIST "%FinalReleaseFolder%" mkdir "%FinalReleaseFolder%"

REM Create exclusion list
ECHO exclusion.txt>exclusion.txt
ECHO .exe.config>>exclusion.txt
IF NOT "%ConfigurationName%"=="Debug" ECHO .pdb>>exclusion.txt
ECHO .vshost>>exclusion.txt
ECHO .log>>exclusion.txt
ECHO .tmp>>exclusion.txt
ECHO .xml>>exclusion.txt


REM Copy the Native LAV folder to output folder folder
REM IF NOT EXIST "%ReleaseFolder%AE" mkdir "%ReleaseFolder%AE"
xcopy "%NativeDLLFolderWithBuildPlatform%AE\." "%ReleaseFolder%AE" /S /Y /I /F /EXCLUDE:exclusion.txt


REM Copy register.bat file to output folder folder
IF EXIST "%PostBuildCurrentFolder%register.bat" xcopy "%PostBuildCurrentFolder%register.bat" "%ReleaseFolder%" /Y /I /F

REM Copy the output folder to final release folder
xcopy "%ReleaseFolder%." "%FinalReleaseFolder%" /S /Y /I /F /EXCLUDE:exclusion.txt


REM Copy native DLLs into final release folder by the current architecture
FOR %%s in (%NativeDLLFiles%) DO (
	IF EXIST "%NativeDLLFolderWithBuildPlatform%\%%~s" xcopy "%NativeDLLFolderWithBuildPlatform%\%%~s" "%ReleaseFolder%" /S /Y /I /F
)


REM Copy native DLLs into release folder by the build platform
FOR %%s in (%NativeDLLFiles%) DO (
	IF EXIST "%NativeDLLFolderWithBuildPlatform%\%%~s" xcopy "%NativeDLLFolderWithBuildPlatform%\%%~s" "%FinalReleaseFolder%" /S /Y /I /F
)


REM Copy final release folder to installer source folder (For current status, temporarily only use X86 binaries to both X86 and AMD64 source)
IF NOT EXIST "%InstallerSourceX64Folder%" mkdir "%InstallerSourceX64Folder%"
xcopy "%FinalReleaseFolder%." "%InstallerSourceX64Folder%" /S /Y /I /F /EXCLUDE:exclusion.txt
xcopy "%ToolsFolder%" "%InstallerSourceX64Folder%\Tools" /S /Y /I /F /EXCLUDE:exclusion.txt
IF NOT EXIST "%InstallerSourceX86Folder%" mkdir "%InstallerSourceX86Folder%"
xcopy "%FinalReleaseFolder%." "%InstallerSourceX86Folder%" /S /Y /I /F /EXCLUDE:exclusion.txt
xcopy "%ToolsFolder%" "%InstallerSourceX86Folder%\Tools" /S /Y /I /F /EXCLUDE:exclusion.txt

REM Delete exclusion list
DEL /F exclusion.txt

popd
EXIT