﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Reflection;

namespace AirFusion
{
	public static class UpgradeTool
	{
		#region define SelfUpdate enum value

		public enum LaunchType
		{
			[Description("-ns")]
			UserCheckUpdate,

			[Description("-s")]
			AutoCheckUpdate
		}

		public enum ExitValue
		{
			NONetWork = 0x0400 + 100,
			NONewVersion = 0x0400 + 101,
			ExistNewVersion = 0x0400 + 102,
			None = 0x0400 + 103
		}

		#endregion define SelfUpdate enum value

		/// <summary>
		/// launchType default value is UserCheckUpdate
		/// if launchType value is "UserCheckUpdate" represent this mode is non silent upgrade mode, in this mode
		/// we will show UI hint user how to do it.
		/// if launchType value is "AutoCheckUpdate" present this mode is silent upgrade mode, in this mode we
		/// will not show any UI hint use, just check app version and downlaod new version if any in backgroud
		/// </summary>
		/// <param name="_strLauncharguments"></param>
		public static ExitValue SelfCheckUpdate(LaunchType launchType = LaunchType.UserCheckUpdate)//string _strLauncharguments = "-ns")
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call CheckforUpdates()", "");
			string upgradeToolPath = string.Empty;
			upgradeToolPath = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), "OEM\\UpgradeTool\\AcerExtend\\UpgradeTool.exe");
			if (File.Exists(upgradeToolPath) == true)
			{
				try
				{
					Process proc = new Process();
					proc.StartInfo.FileName = upgradeToolPath;
					proc.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
					proc.StartInfo.Arguments = GetEnumDescription(launchType);
					proc.Start();

					//only in user check update case need to wait for show UI
					if (launchType == LaunchType.UserCheckUpdate)
					{
						proc.WaitForExit();

						if (proc.ExitCode == (int)ExitValue.NONetWork)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckforUpdates NO_NetWork", "");
							//MessageBox.Show("Please check your network or Wi-Fi connection and try again");
							return ExitValue.NONetWork;
						}
						else if (proc.ExitCode == (int)ExitValue.NONewVersion)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckforUpdates NO_NewVersion", "");
							//MessageBox.Show("You already have the latest version installed.");
							return ExitValue.NONewVersion;
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckforUpdates Prepare Update New Version", "");
							return ExitValue.ExistNewVersion;
						}
					}
					else
						return ExitValue.None;
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			}
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "CheckforUpdates Can't Find file", "");
			return ExitValue.NONewVersion;
		}

		public static string GetEnumDescription(Enum value)
		{
			FieldInfo fi = value.GetType().GetField(value.ToString());

			DescriptionAttribute[] attributes =
				(DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);

			if (attributes != null && attributes.Length > 0)
				return attributes[0].Description;
			else
				return value.ToString();
		}
	}
}