﻿using AirFusion;
using AirFusion.Common;
using AirFusion.Manager;
using AirFusion.Manager.DeviceManager;
using AirFusion.WebService;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for DeviceManagerWindow.xaml
	/// </summary>
	public partial class DeviceManagerWindow : Window
	{
		#region Events

		public event EventHandler<WindowEventArgs> WindowEvent;

		#endregion Events

		#region Constructor & Dispose

		public DeviceManagerWindow()
		{
			InitializeComponent();

			InitializeUIComponent();
		}

		#endregion Constructor & Dispose

		#region Window-related Event Handlers

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			try
			{
				double screenWidth = SystemParameters.PrimaryScreenWidth * Configs.OptimalWindowRatioinScreen;
				double screenHeight = SystemParameters.PrimaryScreenHeight * Configs.OptimalWindowRatioinScreen;

				if (this.grdDeviceManagerWindowContent.ActualWidth >= screenWidth || this.grdDeviceManagerWindowContent.ActualHeight >= screenHeight)
				{
					double widthRadio = Math.Round(screenWidth / this.grdDeviceManagerWindowContent.ActualWidth, 1);
					double heightRadio = Math.Round(screenHeight / this.grdDeviceManagerWindowContent.ActualHeight, 1);
					double finalRatio = heightRadio < widthRadio ? heightRadio : widthRadio;

					this.grdDeviceManagerWindowBackground.LayoutTransform = new ScaleTransform(finalRatio, finalRatio, this.grdDeviceManagerWindowBackground.ActualWidth / 2, this.grdDeviceManagerWindowBackground.ActualHeight / 2);
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void Window_StateChanged(object sender, EventArgs e)
		{
			if (this.WindowState == WindowState.Maximized)
			{
				this.WindowState = WindowState.Normal;
			}
			else if (this.WindowState == WindowState.Normal)
			{
				this.Opacity = 1.0;
			}
		}

		public void ReadyToShow()
		{
			if (this.Opacity != 1.0)
			{
				try
				{
					Storyboard storyboardShowingDeviceManager = this.FindResource("StoryboardShowingDeviceManager") as Storyboard;
					storyboardShowingDeviceManager.Begin();
				}
				catch (Exception ex)
				{ }
			}
		}

		private void Window_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			// For reset and override mouse cursor
			Mouse.OverrideCursor = null;

			// Here the program need to implement the logics when window is collapsed or visible
			Window target = sender as Window;
			if (target.Visibility == System.Windows.Visibility.Visible)
			{
				this.WorkForSetDiscoverable();
			}
			else if (target.Visibility == System.Windows.Visibility.Collapsed)
			{
				this.Opacity = 0.0;
				this.UpdateLayout();

				// To reset all pages
				this.rdbtnPairing.IsChecked = true;
				this.lclPairingTab.lclShakePage.Visibility = System.Windows.Visibility.Visible;
				this.lclPairingTab.lclQRCodePage.Visibility = System.Windows.Visibility.Collapsed;

				this.lclPairedDeviceTab.lclDeviceSettingsPage.TargetDeviceInfo = null;
				this.lclPairedDeviceTab.lclDeviceSettingsPage.CurrentUserAccount = string.Empty;
				this.lclPairedDeviceTab.lclDeviceSettingsPage.Visibility = System.Windows.Visibility.Collapsed;

				this.lclPairedDeviceTab.lclGlobalSettingsPage.TargetSettings = null;
				this.lclPairedDeviceTab.lclGlobalSettingsPage.Visibility = System.Windows.Visibility.Collapsed;

				this.lclNoNetworkPage.Visibility = System.Windows.Visibility.Collapsed;

				// To clear all dialogs
				this.lclTwoBtnDialog.CommandType = UserControlEventType.None;
				this.lclTwoBtnDialog.TargetDevice = null;
				this.lclTwoBtnDialog.txtDialogTitle.Text = string.Empty;
				this.lclTwoBtnDialog.txtDialogDetail.Text = string.Empty;
				this.lclTwoBtnDialog.btnLeft.Content = string.Empty;
				this.lclTwoBtnDialog.btnRight.Content = string.Empty;
				this.lclTwoBtnDialog.imgPresent.Source = null;
				this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;

				this.lclOneBtnDialog.CommandType = UserControlEventType.None;
				this.lclOneBtnDialog.txtDialogTitle.Text = string.Empty;
				this.lclOneBtnDialog.txtDialogDetail.Text = string.Empty;
				this.lclOneBtnDialog.btnAccept.Content = string.Empty;
				this.lclOneBtnDialog.imgPresent.Source = null;
				this.grdOneDlgBackground.Visibility = System.Windows.Visibility.Collapsed;

				this.lclNoBtnDialog.txtDialogTitle.Text = string.Empty;
				this.lclNoBtnDialog.txtDialogDetail.Text = string.Empty;
				this.lclNoBtnDialog.vwbProgressBar.Visibility = System.Windows.Visibility.Collapsed;
				this.lclNoBtnDialog.imgPresent.Visibility = System.Windows.Visibility.Collapsed;
				this.lclNoBtnDialog.imgPresent.Source = null;
				this.grdNoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;

				//this.lclAutoLoginDialog.CommandType = UserControlEventType.None;
				//this.lclAutoLoginDialog.TargetDeviceInfo = null;
				//this.lclAutoLoginDialog.txtDialogTitle.Text = string.Empty;
				//this.lclAutoLoginDialog.txtDialogDetail.Text = string.Empty;
				//this.lclAutoLoginDialog.txtDialogAccount.Text = string.Empty;
				//this.lclAutoLoginDialog.imgPresent.Source = null;
				//this.lclAutoLoginDialog.btnLeft.Content = string.Empty;
				//this.lclAutoLoginDialog.btnRight.Content = string.Empty;
				//this.grdAutoLoginDlgBackground.Visibility = System.Windows.Visibility.Collapsed;

				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, false));
			}
		}

		private void Window_PreviewKeyDown(object sender, KeyEventArgs e)
		{
			if (Keyboard.Modifiers == ModifierKeys.Alt && e.Key == Key.System && e.SystemKey == Key.F4)
			{
				// Handle alt + F4 hotkey event
				this.CloseWindow();
				e.Handled = true;
			}
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			this.CloseWindow();
			e.Cancel = true;
		}

		#endregion Window-related Event Handlers

		#region Private Methods

		private void InitializeUIComponent()
		{
			this.Dispatcher.Invoke(delegate()
			{
				try
				{
					this.lclPairedDeviceTab.lstvPairedDevices.ItemsSource = null;
					this.lclPairedDeviceTab.lstvPairedDevices.ItemsSource = Configs.ApplicationReference.PairedDevices;
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			});
		}

		private void WorkForSetDiscoverable()
		{
			this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, this.rdbtnPairing.IsChecked));
		}

		private void CloseWindow()
		{
			this.Dispatcher.Invoke(delegate()
			{
				try
				{
					this.Visibility = System.Windows.Visibility.Collapsed;
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			});
		}

		private void SendWindowEventAsync(object sender, WindowEventArgs eventArgs)
		{
			try
			{
				if (null != this.WindowEvent)
				{
					// To send event by using different thread, not UI thread
					//this.WindowEvent.BeginInvoke(this, eventArgs, null, null);
					this.WindowEvent.Invoke(this, eventArgs);
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		#endregion Private Methods

		#region Callbacks

		public void AppEventCallback(object sender, AppEventArgs e)
		{
			try
			{
				if (e.EventType == AppEventType.Pairing)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType.Pairing", "");

					if (null != e.Data && e.Data is AFUserDevice)
					{
						AFUserDevice device = e.Data as AFUserDevice;

						this.Dispatcher.Invoke(delegate()
						{
							this.lclNoBtnDialog.txtDialogTitle.Text = Application.Current.TryFindResource("txtPairingTitle") as String;
							this.lclNoBtnDialog.txtDialogDetail.Text = string.Format(Application.Current.TryFindResource("txtPairingDetail") as String, device.DeviceName);
							this.lclNoBtnDialog.vwbProgressBar.Visibility = System.Windows.Visibility.Visible;
							this.lclNoBtnDialog.imgPresent.Visibility = System.Windows.Visibility.Collapsed;
							this.lclNoBtnDialog.imgPresent.Source = null;
							this.grdNoDlgBackground.Visibility = System.Windows.Visibility.Visible;

							// Bring window to foreground
							this.WindowState = System.Windows.WindowState.Normal;
							this.Activate();
						});

						this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, false));
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Data is null", "");
					}
				}
				else if (e.EventType == AppEventType.PairingFinished)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType.PairingFinished", "");

					if (null != e.Args)
					{
						bool isSuccessful = (bool)e.Args;

						LogtoFile.WriteLog("", Configs.DebugLevelValue, "isSuccessful: " + isSuccessful, "");

						if (isSuccessful)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Show paired sccessfully dialog", "");

							//string localDeviceName = e.Data as string;
							AFUserDevice device = e.Data as AFUserDevice;

							// To show the dialog with paired sccessfully
							this.Dispatcher.BeginInvoke(new Action(() =>
							{
								this.lclNoBtnDialog.txtDialogTitle.Text = Application.Current.TryFindResource("txtPairedTitle") as String;
								this.lclNoBtnDialog.txtDialogDetail.Text = string.Format(Application.Current.TryFindResource("txtPairedDetail") as String, device.DeviceName);
								this.lclNoBtnDialog.vwbProgressBar.Visibility = System.Windows.Visibility.Collapsed;
								this.lclNoBtnDialog.imgPresent.Source = Application.Current.TryFindResource("imgPairingSuccessImage") as BitmapImage;
								this.lclNoBtnDialog.imgPresent.Visibility = System.Windows.Visibility.Visible;
								this.grdNoDlgBackground.Visibility = System.Windows.Visibility.Visible;

								// Bring window to foreground
								this.WindowState = System.Windows.WindowState.Normal;
								this.Activate();
							}));
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Show pairing unsccess dialog", "");

							// To show the dialog with pairing unsccessfully
							this.Dispatcher.BeginInvoke(new Action(() =>
							{
								// To collapse pairing progress dialog
								this.lclNoBtnDialog.txtDialogTitle.Text = string.Empty;
								this.lclNoBtnDialog.txtDialogDetail.Text = string.Empty;
								this.lclNoBtnDialog.vwbProgressBar.Visibility = System.Windows.Visibility.Collapsed;
								this.lclNoBtnDialog.imgPresent.Visibility = System.Windows.Visibility.Collapsed;
								this.lclNoBtnDialog.imgPresent.Source = null;
								this.grdNoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;

								this.lclTwoBtnDialog.CommandType = UserControlEventType.PairingUnsccessAction;
								this.lclTwoBtnDialog.TargetDevice = null;
								this.lclTwoBtnDialog.txtDialogTitle.Text = Application.Current.TryFindResource("txtPairingUnsuccessTitle") as String;
								this.lclTwoBtnDialog.txtDialogDetail.Text = Application.Current.TryFindResource("txtPairingUnsuccessDetail") as String;
								this.lclTwoBtnDialog.btnLeft.Content = Application.Current.TryFindResource("txtRetry") as String;
								this.lclTwoBtnDialog.btnRight.Content = Application.Current.TryFindResource("txtQRcode") as String;
								this.lclTwoBtnDialog.imgPresent.Source = Application.Current.TryFindResource("imgPairingUnSuccessImage") as BitmapImage;
								this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Visible;

								// Bring window to foreground
								this.WindowState = System.Windows.WindowState.Normal;
								this.Activate();
							}));
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Args is null", "");
					}
				}
				else if (e.EventType == AppEventType.PairedDeviceListShow)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType.PairedDeviceListShow", "");

					if (null != e.Args)
					{
						bool isShowList = (bool)e.Args;

						this.Dispatcher.Invoke(delegate()
						{
							this.lclPairedDeviceTab.grdNoPaireDevices.Visibility = isShowList ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
							this.lclPairedDeviceTab.lstvPairedDevices.Visibility = isShowList ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed;
						});
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Args is null", "");
					}
				}
				else if (e.EventType == AppEventType.WaitForPairingTimeout)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType.WaitForPairingTimeout", "");

					this.Dispatcher.Invoke(delegate()
					{
						this.lclOneBtnDialog.CommandType = UserControlEventType.WaitForPairingTimeout;
						this.lclOneBtnDialog.txtDialogTitle.Text = string.Empty;
						this.lclOneBtnDialog.txtDialogTitle.Height = 0.0;
						this.lclOneBtnDialog.txtDialogDetail.Text = string.Format(Application.Current.TryFindResource("txtWaitForPairingDetail") as String, Application.Current.TryFindResource("txtOK") as String);
						this.lclOneBtnDialog.btnAccept.Content = Application.Current.TryFindResource("txtOK") as String;
						this.lclOneBtnDialog.imgPresent.Source = Application.Current.TryFindResource("imgWaitForPairingImage") as BitmapImage;
						this.grdOneDlgBackground.Visibility = System.Windows.Visibility.Visible;
					});
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void UserControlEventCallback(object sender, UserControlEventArgs e)
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get UserControlEventType: " + e.EventType.GetStringValue(), "");

				if (e.EventType == UserControlEventType.PairingUnsccessAction)
				{
					if (null != e.Args)
					{
						bool isRetry = (bool)e.Args;

						if (isRetry)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Action: Retry", "");

							this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, true));
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Action: QR Code", "");

							// Do show QR code page
							this.Dispatcher.Invoke(delegate()
							{
								this.lclPairingTab.lclShakePage.Visibility = System.Windows.Visibility.Collapsed;
								this.lclPairingTab.lclQRCodePage.QRCodeImage = Configs.ApplicationReference.GetQRCodeImage();
								this.lclPairingTab.lclQRCodePage.Visibility = System.Windows.Visibility.Visible;
							});

							this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, false));
						}

						this.Dispatcher.Invoke(delegate()
						{
							this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;
						});
					}
				}
				else if (e.EventType == UserControlEventType.UnpairRequestResult)
				{
					if (e.Data is AFUserDevice)
					{
						AFUserDevice device = e.Data as AFUserDevice;

						if (null != e.Args)
						{
							bool isAccept = (bool)e.Args;

							if (isAccept)
							{
								this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.UnpairResponse, device, isAccept));
								this.Dispatcher.Invoke(delegate()
								{
									this.lclPairedDeviceTab.lclDeviceSettingsPage.Visibility = System.Windows.Visibility.Collapsed;
									this.lclPairedDeviceTab.grdDeviceList.Visibility = System.Windows.Visibility.Visible;
								});
							}

							this.Dispatcher.Invoke(delegate()
							{
								this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;
							});
						}
					}
				}
				else if (e.EventType == UserControlEventType.UnpairRequest)
				{
					if (e.Data is AppDeviceInfo)
					{
						AFUserDevice device = (e.Data as AppDeviceInfo).Device;

						string caption = Application.Current.TryFindResource("txtUnpairTitle") as String;
						string message = Application.Current.TryFindResource("txtUnpairDetail") as String;

						this.Dispatcher.Invoke(delegate()
						{
							this.lclTwoBtnDialog.CommandType = UserControlEventType.UnpairRequestResult;
							this.lclTwoBtnDialog.TargetDevice = device;
							this.lclTwoBtnDialog.txtDialogTitle.Text = caption;
							this.lclTwoBtnDialog.txtDialogDetail.Text = string.Format(message, device.DeviceName);
							this.lclTwoBtnDialog.btnLeft.Content = Application.Current.TryFindResource("txtOK") as String;
							this.lclTwoBtnDialog.btnRight.Content = Application.Current.TryFindResource("txtCancel") as String;
							this.lclTwoBtnDialog.imgPresent.Source = Application.Current.TryFindResource("imgUnpairImage") as BitmapImage;
							this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Visible;
						});
					}
				}
				else if (e.EventType == UserControlEventType.ShowPairingPage)
				{
					this.rdbtnPairing.IsChecked = true;
				}
				//else if (e.EventType == UserControlEventType.SetPWD)
				//{
				//	if (e.Data is AppDeviceInfo)
				//	{
				//		try
				//		{
				//			string account = Utils.GetAccountDisplayName();
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Account: " + account, "");

				//			this.Dispatcher.Invoke(delegate()
				//			{
				//				this.lclAutoLoginDialog.CommandType = UserControlEventType.SetPWDResult;
				//				this.lclAutoLoginDialog.TargetDeviceInfo = e.Data as AppDeviceInfo;
				//				this.lclAutoLoginDialog.imgPresent.Source = Application.Current.TryFindResource("imgAutoLoginDlgImage") as BitmapImage;
				//				this.lclAutoLoginDialog.txtDialogTitle.Text = Application.Current.TryFindResource("txtAutoLoginTitle") as String;
				//				this.lclAutoLoginDialog.txtDialogDetail.Text = Application.Current.TryFindResource("txtAutoLoginDlgDetail") as String;
				//				this.lclAutoLoginDialog.txtDialogAccount.Text = account;
				//				this.lclAutoLoginDialog.btnLeft.Content = Application.Current.TryFindResource("txtOK") as String;
				//				this.lclAutoLoginDialog.btnRight.Content = Application.Current.TryFindResource("txtCancel") as String;
				//				this.grdAutoLoginDlgBackground.Visibility = System.Windows.Visibility.Visible;
				//			});
				//		}
				//		catch (Exception ex)
				//		{
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				//		}
				//	}
				//}
				//else if (e.EventType == UserControlEventType.SetPWDResult)
				//{
				//	if (e.Data is AppDeviceInfo)
				//	{
				//		AppDeviceInfo deviceInfo = e.Data as AppDeviceInfo;

				//		try
				//		{
				//			if (null != e.Args)
				//			{
				//				object[] metadata = e.Args as object[];
				//				bool isSet = (bool)metadata[0];
				//				string password = metadata[1] as string;

				//				if (isSet)
				//				{
				//					this.lclPairedDeviceTab.lclDeviceSettingsPage.TargetDeviceInfo.IsAutoUnlockOn = true;
				//					this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetAccountPassword, e.Data, password));
				//				}
				//				else
				//				{
				//					this.lclPairedDeviceTab.lclDeviceSettingsPage.TargetDeviceInfo.IsAutoUnlockOn = false;
				//				}
				//			}

				//			this.Dispatcher.Invoke(delegate()
				//			{
				//				this.grdAutoLoginDlgBackground.Visibility = System.Windows.Visibility.Collapsed;
				//			});
				//		}
				//		catch (Exception ex)
				//		{
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				//		}
				//	}
				//}
				//else if (e.EventType == UserControlEventType.ChangePWD)
				//{
				//	if (e.Data is AppDeviceInfo)
				//	{
				//		try
				//		{
				//			string account = Utils.GetAccountDisplayName();
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Account: " + account, "");

				//			this.Dispatcher.Invoke(delegate()
				//			{
				//				this.lclAutoLoginDialog.CommandType = UserControlEventType.ChangePWDResult;
				//				this.lclAutoLoginDialog.TargetDeviceInfo = e.Data as AppDeviceInfo;
				//				this.lclAutoLoginDialog.imgPresent.Source = Application.Current.TryFindResource("imgAutoLoginDlgImage") as BitmapImage;
				//				this.lclAutoLoginDialog.txtDialogTitle.Text = Application.Current.TryFindResource("txtAutoLoginTitle") as String;
				//				this.lclAutoLoginDialog.txtDialogDetail.Text = Application.Current.TryFindResource("txtAutoLoginDlgDetail") as String;
				//				this.lclAutoLoginDialog.txtDialogAccount.Text = account;
				//				this.lclAutoLoginDialog.btnLeft.Content = Application.Current.TryFindResource("txtOK") as String;
				//				this.lclAutoLoginDialog.btnRight.Content = Application.Current.TryFindResource("txtCancel") as String;
				//				this.grdAutoLoginDlgBackground.Visibility = System.Windows.Visibility.Visible;
				//			});
				//		}
				//		catch (Exception ex)
				//		{
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				//		}
				//	}
				//}
				//else if (e.EventType == UserControlEventType.ChangePWDResult)
				//{
				//	if (e.Data is AppDeviceInfo)
				//	{
				//		AppDeviceInfo deviceInfo = e.Data as AppDeviceInfo;

				//		try
				//		{
				//			if (null != e.Args)
				//			{
				//				object[] metadata = e.Args as object[];
				//				bool isSet = (bool)metadata[0];
				//				string password = metadata[1] as string;

				//				if (isSet)
				//				{
				//					this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetAccountPassword, e.Data, password));
				//				}
				//			}

				//			this.Dispatcher.Invoke(delegate()
				//			{
				//				this.grdAutoLoginDlgBackground.Visibility = System.Windows.Visibility.Collapsed;
				//			});
				//		}
				//		catch (Exception ex)
				//		{
				//			LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				//		}
				//	}
				//}
				//else if (e.EventType == UserControlEventType.DeletePWD)
				//{
				//	this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.DeleteAccountPassword, e.Data));
				//}
				else if (e.EventType == UserControlEventType.WaitForPairingTimeout)
				{
					this.Dispatcher.Invoke(delegate()
					{
						this.grdOneDlgBackground.Visibility = System.Windows.Visibility.Collapsed;
					});

					this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, true));
				}
				else if (e.EventType == UserControlEventType.SetDiscoverableValue)
				{
					if (e.Args is bool)
					{
						this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SetDiscoverable, null, (bool)e.Args));
					}
				}
				else if (e.EventType == UserControlEventType.TapToLaunchDevice)
				{
					try
					{
						if (null != sender)
						{
							AppDeviceInfo selectedTarget = e.Data as AppDeviceInfo;

							if (null != Configs.ApplicationReference.ConnectingDeviceInfo && selectedTarget.Device.InstanceID == Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connection is already existed", "");
								return;
							}
							else
							{
								this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.RestartScreenCasting, selectedTarget, true));
							}
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		#endregion Callbacks

		#region UI Event Callbacks

		private void grdTitleArea_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			try
			{
				this.DragMove();
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void btnDeviceManagerWindowMinimize_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				this.WindowState = System.Windows.WindowState.Minimized;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void btnDeviceManagerWindowClose_Click(object sender, RoutedEventArgs e)
		{
			this.CloseWindow();
		}

		private void rdbtnPairing_Checked(object sender, RoutedEventArgs e)
		{
			if (null != this.lclPairingTab)
			{
				this.lclPairingTab.lclShakePage.Visibility = System.Windows.Visibility.Visible;
				this.lclPairingTab.lclQRCodePage.Visibility = System.Windows.Visibility.Collapsed;
			}

			this.WorkForSetDiscoverable();
		}

		private void rdbtnPairing_Unchecked(object sender, RoutedEventArgs e)
		{
			if (null != this.lclPairingTab)
			{
				this.lclPairingTab.lclShakePage.Visibility = System.Windows.Visibility.Visible;
				this.lclPairingTab.lclQRCodePage.Visibility = System.Windows.Visibility.Collapsed;
			}

			this.WorkForSetDiscoverable();
		}

		private void rdbtnPairedDevices_Unchecked(object sender, RoutedEventArgs e)
		{
			try
			{
				if (this.lclPairedDeviceTab.lclDeviceSettingsPage.Visibility == System.Windows.Visibility.Visible)
				{
					this.lclPairedDeviceTab.lclDeviceSettingsPage.TargetDeviceInfo = null;
					this.lclPairedDeviceTab.lclDeviceSettingsPage.CurrentUserAccount = string.Empty;
					this.lclPairedDeviceTab.lclDeviceSettingsPage.Visibility = System.Windows.Visibility.Collapsed;

					this.lclPairedDeviceTab.grdDeviceList.Visibility = System.Windows.Visibility.Visible;
				}

				if (this.lclPairedDeviceTab.lclGlobalSettingsPage.Visibility == System.Windows.Visibility.Visible)
				{
					this.lclPairedDeviceTab.lclGlobalSettingsPage.TargetSettings = null;
					this.lclPairedDeviceTab.lclGlobalSettingsPage.Visibility = System.Windows.Visibility.Collapsed;
					this.lclPairedDeviceTab.grdDeviceList.Visibility = System.Windows.Visibility.Visible;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void StoryboardShowingDeviceManager_Completed(object sender, EventArgs e)
		{
			this.Opacity = 1.0;
		}

		#endregion UI Event Callbacks
	}
}