﻿using AirFusion.Common;
using AirFusion.ConnectionManager;
using AirFusion.Manager.DeviceManager;
using Microsoft.Win32;
using System;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for ScreenWindow.xaml
	/// </summary>
	public partial class ScreenWindow : Window
	{
		#region Members

		// For hooking message
		private IntPtr _hImc;

		private WindowInteropHelper _wih;
		private WindowStater _windowStater;
		private HwndSource _hWndSource;

		private delegate void HookMessageDelete();

		// For rotation
		private AFUserDevice.DisplayInfo.DisplayRotation _currentRenderRotationAngle = AFUserDevice.DisplayInfo.DisplayRotation.Rotation0;  // Default is anticlockwise rotation 90

		private VideoSize _videoSize = new VideoSize
		{
			VideoWidth = (double)Application.Current.TryFindResource("VideoShortSide"),
			VideoHeight = (double)Application.Current.TryFindResource("VideoLongSide"),
		};

		private bool _isPortraitMode = true; // Default is portrait
		private bool _hasVirtualKey = true;   // Default has virtual key

		// For launch
		private bool _isDeviceFirstChanged = true;

		// For scale transform
		private double _finalRatio = 1; // Default is not scaling

		private bool _isToVitual = true;   // For temporary use, need to modify once AFManager can get current keyboard mode

		// For resize use
		private bool _isFirstTimeSetScreenCastingLayout = true;

		// For recording shown dialog
		private bool _hasShownDialog = false;

		// For recording removing item action
		private bool _hasRemoveItem = false;

		private ManualResetEvent _windowIsLoaded = new ManualResetEvent(false);

		private bool _isStartUploadFile = false;

		private static object _pInstanceLock = new object();

		private System.Timers.Timer _pScreenCastTimer = null;// _pScreenCastTimer = new System.Timers.Timer();
		private const int ScreenCast_MONITOR_TIME = 10 * 60 * 1000; // 10 min
		public AppDeviceInfo CurrentScreenCastdeviceInfo = null;

		public bool _IsShowDiconnectWarningMessgae = false;

		public bool _isVideoQualityFirstShow = true;

		private bool _isSetScreenSizeToVNCNeeded = false;

		private ErrorTypeMessage recordErrorType = ErrorTypeMessage.InitializeWakeUpFail;

		private const UInt32 WS_VISIBLE = 0x10000000;
		private const UInt32 WS_DISABLED = 0x8000000;
		private const int GWL_STYLE = -16;

		private Storyboard _storyboardgrdScreenSnapshot;

		#endregion Members

		#region Events

		public event EventHandler<WindowEventArgs> WindowEvent;

		#endregion Events

		#region Enumerators

		/// <summary>
		/// The Language IDs that support ImeConversionModeValues.Native (Reference the following webs)
		/// - http://msdn.microsoft.com/en-us/library/windows/desktop/ee418266(v=vs.85).aspx#simplified_chinese_ime
		/// - http://msdn.microsoft.com/zh-tw/goglobal/bb895996.aspx
		/// </summary>
		private enum NativeLanguageID
		{
			Chinese_Taiwan = 1028,
			Chinese_PRC = 2052,

			//Chinese_Hong_Kong = 3076,
			//Chinese_Singapore = 4100,
			//Chinese_Macau = 5124,
			Japanese = 1041,

			Korean = 1042,
		}

		#endregion Enumerators

		#region Constructor & Dispose

		public ScreenWindow()
		{
			InitializeComponent();
			_wih = new WindowInteropHelper(this);
			_windowStater = new WindowStater();

			// Advanced Component Initialization
			this.InitializeUIComponent();

			this.DataContext = Application.Current as App;
			this._dpiRatio = Utils.GetDPIRatio(false);

			SystemEvents.DisplaySettingsChanged += this.DisplaySettingsChangedHandler;
			this.dsElement.NetworkUnstable += this.NetworkUnstableHandler;
		}

		private void DisplaySettingsChangedHandler(object sender, EventArgs e)
		{
			if (this.Visibility == Visibility.Visible)
			{
				try
				{
					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						this.Activate();

						if (this._isMaximized)
							this.MaxmizeWindow();
						else
							this.SetScreenCastingLayout();

						this.UpdateLayout();

						if (Utils.GetDPIRatio(false) != this._dpiRatio)
						{
							this.SetScreenSizeToVNC();
							this._dpiRatio = Utils.GetDPIRatio(false);
						}
					});
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][DisplaySettingsChangedHandler] " + ex.ToString(), "");
				}
			}
		}

		#endregion Constructor & Dispose

		#region For IME Character Input TextBox

		private bool _MouseInIMETextBox = false;

		private void IMETextBoxMouseLeave(object sender, MouseEventArgs e)
		{
			this._MouseInIMETextBox = false;
		}

		private void IMETextBoxMouseEnter(object sender, MouseEventArgs e)
		{
			this._MouseInIMETextBox = true;
		}

		protected override void OnMouseDown(MouseButtonEventArgs e)
		{
			try
			{
				if (this.IMEInputGrid.Visibility == System.Windows.Visibility.Visible)
				{
					if (false == this._MouseInIMETextBox)
					{
						this.IsShowTextInputBox(false);
					}
				}

				this.ResetFunctionFieldStatus();
				this.ResetSelectDeviceFieldStatus();
			}
			catch
			{ }

			base.OnMouseDown(e);
		}

		private void IMETextKeyboardUnFocusHandler(object sender, KeyboardFocusChangedEventArgs e)
		{
			try
			{
				this.IsShowTextInputBox(false);
			}
			catch
			{ }
		}

		protected override void OnPreviewKeyDown(KeyEventArgs e)
		{
			////Debug.WriteLine("OnPreviewKeyDown(): e.ImeProcessedKey = " + e.ImeProcessedKey.ToString());
			////Debug.WriteLine("OnPreviewKeyDown(): e.Key             = " + e.Key.ToString());
			////Debug.WriteLine("OnPreviewKeyDown(): e.SystemKey       = " + e.SystemKey.ToString());
			try
			{
				if (e.ImeProcessedKey != Key.None)
				{
					if (this.IMEInputGrid.Visibility != System.Windows.Visibility.Visible)
					{
						Debug.WriteLine("OnPreviewKeyDown(): Popup IME TextBox !!");
						this.IMEInputGrid.Visibility = System.Windows.Visibility.Visible;
						Keyboard.Focus(this.txtbxTextInputContent);
					}
				}
				else
				{
					if (null != FlowControlManager.Instance.InputController)
					{
						if (e.Key == Key.Enter)
						{
							if (this.txtbxTextInputContent.Text != string.Empty)
							{
								Debug.WriteLine("OnPreviewKeyDown(): Enter => Send the string of IME TextBox !!");
								this.IsShowTextInputBox(false);
								e.Handled = true;
							}
						}
						else if (e.Key == Key.Back)
						{
							if (this.txtbxTextInputContent.Text == string.Empty)
							{
								FlowControlManager.Instance.InputController.VNCClient.SendInputVirtualKey(VNC.VIRTUAL_KEY_TYPE.BACKSPACE);
								e.Handled = true;
							}
						}
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[OnPreviewKeyDown] " + ex.ToString(), "");
			}

			base.OnPreviewKeyDown(e);
		}

		private void ToHideIMETextBox(int iWinMsg)
		{
			if (this.IMEInputGrid.Visibility == System.Windows.Visibility.Visible)
			{
				switch (iWinMsg)
				{
					case Win32Api.WM_LBUTTONDBLCLK:
					case Win32Api.WM_LBUTTONDOWN:
					case Win32Api.WM_RBUTTONDBLCLK:
					case Win32Api.WM_RBUTTONDOWN:
						{
							if (false == this._MouseInIMETextBox)
							{
								this.IsShowTextInputBox(false);
							}
						}
						break;

					default:
						break;
				}
			}
		}

		private bool IsEnteringIMEChar(int iWinMsg)
		{
			if (this.IMEInputGrid.Visibility == System.Windows.Visibility.Visible)
			{
				switch (iWinMsg)
				{
					case Win32Api.WM_KEYDOWN:
					case Win32Api.WM_KEYUP:
						{
							return true;
						}
					default:
						{
							return false;
						}
				}
			}

			return false;
		}

		#endregion For IME Character Input TextBox

		#region Private Methods

		private void InitializeUIComponent()
		{
			this.Dispatcher.Invoke(delegate()
			{
				try
				{
					this.lvDeviceList.ItemsSource = null;
					this.lvDeviceList.ItemsSource = Configs.ApplicationReference.PairedDevices;

					((INotifyCollectionChanged)this.lvDeviceList.Items).CollectionChanged += this.lvDeviceList_CollectionChanged;
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}

				//this.Visibility = System.Windows.Visibility.Visible;

				//0623 Johnon add
				if (IsEnableDebugMode())
				{
					this.txtbConnectionStatus.Visibility = Visibility.Visible;
					this.launchingProgressbar.Visibility = Visibility.Collapsed;
				}
				else
				{
					this.txtbConnectionStatus.Visibility = Visibility.Collapsed;
					this.launchingProgressbar.Visibility = Visibility.Visible;
				}
			});
		}

		private void HookMessage()
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "HookMessage()", "");

			try
			{
				this.Dispatcher.Invoke(delegate()
				{
					if (null == this._hWndSource)
					{
						this._hWndSource = HwndSource.FromHwnd(_wih.Handle);
					}

					this._hWndSource.AddHook(this.WinProc);
				});
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void RemoveHookMessage()
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "RemoveHookMessage()", "");

			try
			{
				this.Dispatcher.Invoke(delegate()
				{
					if (null != this._hWndSource)
					{
						this._hWndSource.RemoveHook(this.WinProc);
					}
				});
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void SetScreenSizeToVNC()
		{
			// Set screen casting size arguments to VNC
			int width;
			int height;

			if (this._isMaximized)
			{
				width = (int)(grdScreen.Width);
				height = (int)(grdScreen.Height);
			}
			else
			{
				width = (int)(grdScreen.Width * this._finalRatio);
				height = (int)(grdScreen.Height * this._finalRatio);
			}

			try
			{
				var target = this.grdScreen.TransformToAncestor(this.grdScreenWindowBackground);

				if (null != target)
				{
					Point relativePoint = target.Transform(new Point(0, 0));

					if (null != relativePoint)
					{
						int xDelta;
						int yDelta;

						if (this._isMaximized)
						{
							xDelta = (int)(relativePoint.X);
							yDelta = (int)(relativePoint.Y);
						}
						else
						{
							xDelta = (int)(relativePoint.X * this._finalRatio);
							yDelta = (int)(relativePoint.Y * this._finalRatio);
						}

						int[] size = new int[4];
						size[0] = width;
						size[1] = height;
						size[2] = xDelta;
						size[3] = yDelta;

						this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.VNCSetScreenSize, size));
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetScreenSizeToVNC] relativePoint is null", "");
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetScreenSizeToVNC] this.grdScreen.TransformToAncestor(this) is null", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetScreenSizeToVNC] " + ex.ToString(), "");
			}
		}

		private void SetCenterScreen()
		{
			try
			{
				System.Drawing.Rectangle currentScreenWorkingArea = this.GetScreen(this).WorkingArea;

				double screenWidth = currentScreenWorkingArea.Width / this._dpiRatio;
				double screenHeight = currentScreenWorkingArea.Height / this._dpiRatio;

				double windowWidth = this.grdScreenWindowBackground.ActualWidth * this._finalRatio;
				double windowHeight = this.grdScreenWindowBackground.ActualHeight * this._finalRatio;

				double tempLeft = currentScreenWorkingArea.X + (screenWidth / 2) - (windowWidth / 2);
				double tempTop = currentScreenWorkingArea.Y + (screenHeight / 2) - (windowHeight / 2);

				this.Left = tempLeft;
				this.Top = tempTop;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetCenterScreen] " + ex.ToString(), "");
			}
		}

		private void SetRotation(AFUserDevice device, PhoneSystemPermissionDetectionRule rule, ref bool isPortrait, ref bool hasVirtualKey, ref VideoSize videoSize, ref AFUserDevice.DisplayInfo.DisplayRotation currentRenderRotationAngle)
		{
			if (device.Display.Navigation.HasValue)
			{
				if ((bool)device.Display.Navigation)
				{
					hasVirtualKey = true;
				}
				else
				{
					hasVirtualKey = false;
				}
			}

			// Detect phone system permission
			bool isHasPermission = false;
			try
			{
				switch (rule)
				{
					case PhoneSystemPermissionDetectionRule.AutoDetect:
						{
							isHasPermission = Utils.DetectPhoneHasSystemPermission(device);
						}
						break;

					case PhoneSystemPermissionDetectionRule.ForceHas:
						{
							isHasPermission = true;
						}
						break;

					case PhoneSystemPermissionDetectionRule.ForceNotHas:
						{
							isHasPermission = false;
						}
						break;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			// Set default render angle based on phone system permission
			if (Configs.IsWithoutMiracast)
			{
				currentRenderRotationAngle = isHasPermission ? AFUserDevice.DisplayInfo.DisplayRotation.Rotation0 : AFUserDevice.DisplayInfo.DisplayRotation.Rotation0;
			}
			else
			{
				currentRenderRotationAngle = isHasPermission ? AFUserDevice.DisplayInfo.DisplayRotation.Rotation270 : AFUserDevice.DisplayInfo.DisplayRotation.Rotation0;
			}

			if (device.Display.Rotation.HasValue)
			{
				try
				{
					AFUserDevice.DisplayInfo.DisplayRotation rotation = (AFUserDevice.DisplayInfo.DisplayRotation)device.Display.Rotation;

					if (Configs.IsWithoutMiracast)
					{
						currentRenderRotationAngle = this.GetRenderRotationAngle(false, rotation);
					}
					else
					{
						currentRenderRotationAngle = this.GetRenderRotationAngle(isHasPermission, rotation);
					}

					switch (rotation)
					{
						case AFUserDevice.DisplayInfo.DisplayRotation.Rotation0:
							{
								if (!this._isMaximized)
								{
									this.Dispatcher.Invoke(delegate()
									{
										//UIElement 套圖 (0)
										this.dkpScreenBackground.Visibility = Visibility.Visible;
										this.dkpScreenBackground90.Visibility = Visibility.Collapsed;
										this.dkpScreenBackground270.Visibility = Visibility.Collapsed;

										this.grdTitleArea.Visibility = Visibility.Visible;
										this.grdLandscape90TitleArea.Visibility = Visibility.Collapsed;
										this.grdLandscape270TitleArea.Visibility = Visibility.Collapsed;
										this.grdFunction.Visibility = Visibility.Visible;
										this.grdLandscape90Function.Visibility = Visibility.Collapsed;
										this.grdLandscape270Function.Visibility = Visibility.Collapsed;

										// Portrait mode
										this.grdLandscapeNavigationButtonField.Visibility = System.Windows.Visibility.Collapsed;
										this.grdPortraitNavigationButtonField.Visibility = this._hasVirtualKey ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
									});
								}

								this._rotatedAngle = 0;
								isPortrait = true;
							}
							break;

						case AFUserDevice.DisplayInfo.DisplayRotation.Rotation90:
							{
								if (!this._isMaximized)
								{
									this.Dispatcher.Invoke(delegate()
									{
										//UIElement 套圖 (90)
										this.dkpScreenBackground.Visibility = Visibility.Collapsed;
										this.dkpScreenBackground90.Visibility = Visibility.Visible;
										this.dkpScreenBackground270.Visibility = Visibility.Collapsed;

										this.grdTitleArea.Visibility = Visibility.Collapsed;
										this.grdLandscape90TitleArea.Visibility = Visibility.Visible;
										this.grdLandscape270TitleArea.Visibility = Visibility.Collapsed;
										this.grdFunction.Visibility = Visibility.Collapsed;
										this.grdLandscape90Function.Visibility = Visibility.Visible;
										this.grdLandscape270Function.Visibility = Visibility.Collapsed;

										// Landscape mode
										this.grdPortraitNavigationButtonField.Visibility = System.Windows.Visibility.Collapsed;
										this.grdLandscapeNavigationButtonField.Visibility = this._hasVirtualKey ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
									});
								}

								this._rotatedAngle = 90;
								isPortrait = false;
							}
							break;

						case AFUserDevice.DisplayInfo.DisplayRotation.Rotation180:
							{
							}
							break;

						case AFUserDevice.DisplayInfo.DisplayRotation.Rotation270:
							{
								if (!this._isMaximized)
								{
									this.Dispatcher.Invoke(delegate()
									{
										//UIElement 套圖 (270)
										this.dkpScreenBackground.Visibility = Visibility.Collapsed;
										this.dkpScreenBackground90.Visibility = Visibility.Collapsed;
										this.dkpScreenBackground270.Visibility = Visibility.Visible;

										this.grdTitleArea.Visibility = Visibility.Collapsed;
										this.grdLandscape90TitleArea.Visibility = Visibility.Collapsed;
										this.grdLandscape270TitleArea.Visibility = Visibility.Visible;
										this.grdFunction.Visibility = Visibility.Collapsed;
										this.grdLandscape90Function.Visibility = Visibility.Collapsed;
										this.grdLandscape270Function.Visibility = Visibility.Visible;

										// Landscape mode
										this.grdPortraitNavigationButtonField.Visibility = System.Windows.Visibility.Collapsed;
										this.grdLandscapeNavigationButtonField.Visibility = this._hasVirtualKey ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
									});
								}

								this._rotatedAngle = 270;
								isPortrait = false;
							}
							break;

						default:
							break;
					}

					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						this.ResetAllFieldStauts();
					});
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "device.Display.Rotation has not value", "");
			}
		}

		private AFUserDevice.DisplayInfo.DisplayRotation GetRenderRotationAngle(bool isHasPermission, AFUserDevice.DisplayInfo.DisplayRotation rotation)
		{
			switch (rotation)
			{
				case AFUserDevice.DisplayInfo.DisplayRotation.Rotation0:
					{
						return isHasPermission ? AFUserDevice.DisplayInfo.DisplayRotation.Rotation270 : rotation;
					}
				case AFUserDevice.DisplayInfo.DisplayRotation.Rotation90:
					{
						return isHasPermission ? AFUserDevice.DisplayInfo.DisplayRotation.Rotation180 : AFUserDevice.DisplayInfo.DisplayRotation.Rotation270;
					}
				case AFUserDevice.DisplayInfo.DisplayRotation.Rotation180:
					{
						return isHasPermission ? AFUserDevice.DisplayInfo.DisplayRotation.Rotation90 : rotation;
					}
				case AFUserDevice.DisplayInfo.DisplayRotation.Rotation270:
					{
						return isHasPermission ? AFUserDevice.DisplayInfo.DisplayRotation.Rotation0 : AFUserDevice.DisplayInfo.DisplayRotation.Rotation90;
					}
			}

			return rotation;
		}

		private void SetScreenCastingLayout()
		{
			System.Drawing.Rectangle currentScreenWorkingArea = this.GetScreen(this).WorkingArea;

			double currentScreenWorkingAreaBottom = currentScreenWorkingArea.Bottom / this._dpiRatio;
			double currentScreenWorkingAreaTop = currentScreenWorkingArea.Top / this._dpiRatio;
			double currentScreenWorkingAreaLeft = currentScreenWorkingArea.Left / this._dpiRatio;
			double currentScreenWorkingAreaRight = currentScreenWorkingArea.Right / this._dpiRatio;
			double currentScreenWorkingAreaHeight = currentScreenWorkingArea.Height / this._dpiRatio;
			double currentScreenWorkingAreaWidth = SystemParameters.VirtualScreenWidth;

			if (this._isFirstTimeSetScreenCastingLayout)
			{
				bool isX86;
				object videoInitialShortSideValue = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath + @"\InitialScreenSize", "VideoShortSide", RegistryView.Registry64, out isX86);
				object videoInitialLongSideValue = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath + @"\InitialScreenSize", "VideoLongSide", RegistryView.Registry64, out isX86);

				this._videoMinLongSide = 450;

				if (videoInitialShortSideValue != null && videoInitialLongSideValue != null)
				{
					this._videoInitialLongSide = Convert.ToDouble(videoInitialLongSideValue);
					this._videoInitialShortSide = Convert.ToDouble(videoInitialShortSideValue);

					if (this._videoInitialLongSide < this._videoInitialShortSide)
					{
						double tmp = this._videoInitialLongSide;
						this._videoInitialLongSide = this._videoInitialShortSide;
						this._videoInitialShortSide = tmp;
					}

					if (this._videoInitialLongSide < this._videoMinLongSide)
					{
						this._videoInitialLongSide = this._videoSize.VideoHeight;
						this._videoInitialShortSide = this._videoSize.VideoWidth;
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetScreenCastingLayout] No Record In Registry", "");

					this._videoInitialLongSide = this._videoSize.VideoHeight;
					this._videoInitialShortSide = this._videoSize.VideoWidth;
				}

				double videoInitialSizeRatio = this._videoInitialLongSide / this._videoInitialShortSide;

				this._videoLongSide = this._videoInitialLongSide;
				this._videoShortSide = this._videoInitialShortSide;
				this._videoSizeRatio = this._videoLongSide / this._videoShortSide;
			}

			double windowHeightWithoutScreen = this.grdScreenWindowBackground.ActualHeight - this.grdScreen.ActualHeight;
			double windowWidthWithoutScreen = this.grdScreenWindowBackground.ActualWidth - this.grdScreen.ActualWidth;

			if (this._isPortraitMode)
			{
				this.grdScreen.Width = this._videoShortSide;
				this.grdScreen.Height = this._videoLongSide;
			}
			else
			{
				this.grdScreen.Width = this._videoLongSide;
				this.grdScreen.Height = this._videoShortSide;
			}

			double windowHeight = windowHeightWithoutScreen + this.grdScreen.Height;
			if (windowHeight > currentScreenWorkingAreaHeight)
			{
				if (this._isPortraitMode)
				{
					double tmp = this.grdScreen.Height - (windowHeight - currentScreenWorkingAreaHeight);
					this.grdScreen.Height = tmp >= this._videoMinLongSide ? tmp : this._videoMinLongSide;
					this.grdScreen.Width = this.grdScreen.Height / this._videoSizeRatio;
				}
				else
				{
					double tmp = (this.grdScreen.Height - (windowHeight - currentScreenWorkingAreaHeight)) * this._videoSizeRatio;
					this.grdScreen.Width = tmp >= this._videoMinLongSide ? tmp : this._videoMinLongSide;
					this.grdScreen.Height = this.grdScreen.Width / this._videoSizeRatio;
				}
			}

			double windowWidth = windowWidthWithoutScreen + this.grdScreen.Width;
			if (windowWidth > currentScreenWorkingAreaWidth)
			{
				if (this._isPortraitMode)
				{
					double tmp = (this.grdScreen.Width - (windowWidth - currentScreenWorkingAreaWidth)) * this._videoSizeRatio;
					this.grdScreen.Height = tmp >= this._videoMinLongSide ? tmp : this._videoMinLongSide;
					this.grdScreen.Width = this.grdScreen.Height / this._videoSizeRatio;
				}
				else
				{
					double tmp = this.grdScreen.Width - (windowWidth - currentScreenWorkingAreaWidth);
					this.grdScreen.Width = tmp >= this._videoMinLongSide ? tmp : this._videoMinLongSide;
					this.grdScreen.Height = this.grdScreen.Width / this._videoSizeRatio;
				}
			}

			this._videoShortSide = this._isPortraitMode ? this.grdScreen.Width : this.grdScreen.Height;
			this._videoLongSide = this._isPortraitMode ? this.grdScreen.Height : this.grdScreen.Width;

			#region ScaleTransform

			double transformRatio = this._finalRatio;
			windowHeight = windowHeightWithoutScreen + this.grdScreen.Height;

			if ((windowHeight * transformRatio) > currentScreenWorkingAreaHeight)
				transformRatio = currentScreenWorkingAreaHeight / windowHeight;

			windowWidth = windowWidthWithoutScreen + this.grdScreen.Width;

			if ((windowWidth * transformRatio) > currentScreenWorkingAreaWidth)
				transformRatio = currentScreenWorkingAreaWidth / windowWidth;

			ScaleTransform scaleTransform = new ScaleTransform();
			scaleTransform.ScaleX = transformRatio;
			scaleTransform.ScaleY = transformRatio;
			this._finalRatio = transformRatio;
			this.grdScreenWindowBackground.LayoutTransform = scaleTransform;

			#endregion ScaleTransform

			if (this.Top >= currentScreenWorkingAreaBottom || (this.Top + windowHeight * this._finalRatio) <= currentScreenWorkingAreaTop ||
				this.Left > currentScreenWorkingAreaRight || (this.Left + windowWidth * this._finalRatio) <= currentScreenWorkingAreaLeft)
				this.SetCenterScreen();
		}

		private void CloseWindow(bool _isHideWindow = true)
		{
			//Johnson 0416 Add for debug Close window deadlock issue
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "ScreenWindow Click Close Button", "");

			//Johnson 0508 add for stop timer
			ScreencastTimer(false);

			if (_isHideWindow)
			{
				this.Dispatcher.Invoke(delegate()
				{
					Storyboard storyboardClosingMainWindow = this.FindResource("StoryboardClosingMainWindow") as Storyboard;

					EventHandler StoryboardClosingMainWindowCompletedEventHandler = null;
					StoryboardClosingMainWindowCompletedEventHandler = (sender, e) =>
					{
						this.ResetWindow(_isHideWindow);
						storyboardClosingMainWindow.Completed -= StoryboardClosingMainWindowCompletedEventHandler;
					};

					storyboardClosingMainWindow.Completed += StoryboardClosingMainWindowCompletedEventHandler;

					storyboardClosingMainWindow.Begin();
				});
			}
			else
			{
				this.ResetWindow(_isHideWindow);
			}
		}

		private void ResetWindow(bool _isHideWindow)
		{
			try
			{
				this.ResetFunctionFieldStatus();
				this.ResetSelectDeviceFieldStatus();

				this.Dispatcher.Invoke(delegate()
				{
					if (_isHideWindow)
					{
						this.txtbConnectionStatus.Text = string.Empty;
						this.txtbWarrningMessage.Text = string.Empty;
					}
					this.dsElement.Visibility = System.Windows.Visibility.Hidden;

					if (this._isMaximized)
					{
						this.RecoverScreenSize();
					}
				});

				// To avoid blocking UI thread when release DirectShow filters by using background worker
				BackgroundWorker worker = new BackgroundWorker();
				DoWorkEventHandler doWorkEventHandler = null;
				doWorkEventHandler = delegate(object s, DoWorkEventArgs k)
				{
					try
					{
						this.dsElement.Close();
						this.Dispatcher.Invoke(delegate()
						{
							if (this.Visibility == System.Windows.Visibility.Visible)
							{
								// Reset UI status
								//this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitializatingStatus") as String;

								this.dsElement.Visibility = System.Windows.Visibility.Hidden;
								this.grdScreen.AllowDrop = false;

								this.lvDeviceList.SelectedIndex = -1;
								//this.lvDeviceList.SelectedValue = null;
								if (_isHideWindow)
								{
									this.Visibility = System.Windows.Visibility.Hidden;
									this.txtbConnectionStatus.Text = string.Empty;
									this.txtbWarrningMessage.Text = string.Empty;
								}
								this._isToVitual = true;

								this.brdUploadStatus.Visibility = System.Windows.Visibility.Hidden;
								this.txtbUploadStatus.Text = string.Empty;
								this.brdDownloadStatus.Visibility = System.Windows.Visibility.Hidden;
								this.txtbDownloadStatus.Text = string.Empty;

								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Send CloseScreenCasting event", "");
								this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.CloseScreenCasting));
							}
						});
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}

					worker.DoWork -= doWorkEventHandler;
				};

				worker.DoWork += doWorkEventHandler;
				worker.RunWorkerAsync();
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void SendWindowEventAsync(object sender, WindowEventArgs eventArgs)
		{
			try
			{
				if (null != this.WindowEvent)
				{
					// To send event by using different thread, not UI thread
					//this.WindowEvent.BeginInvoke(this, eventArgs, null, null);
					this.WindowEvent.Invoke(this, eventArgs);
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		/// <summary>
		/// Resets all field(Function bar and Select Device List) status.
		/// </summary>
		private void ResetAllFieldStauts()
		{
			LogtoFile.TrackEnter();

			this.ResetFunctionFieldStatus();
			this.ResetSelectDeviceFieldStatus();

			LogtoFile.TrackLeave();
		}

		private void ResetFunctionFieldStatus()
		{
			try
			{
				this.Dispatcher.Invoke(delegate
				{
					this.tglbtnFunction.IsChecked = false;
					this.tglbtnLandscape90Function.IsChecked = false;
					this.tglbtnLandscape270Function.IsChecked = false;
				});
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void ResetSelectDeviceFieldStatus()
		{
			try
			{
				this.Dispatcher.Invoke(delegate
				{
					this.tglbtnSelectDevice.IsChecked = false;
					this.tglbtnLandscape90SelectDevice.IsChecked = false;
					this.tglbtnLandscape270SelectDevice.IsChecked = false;
				});
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		/// <summary>
		/// Gets the located screen of the target window
		/// </summary>
		/// <param name="window">target window</param>
		/// <returns>located screen</returns>
		private System.Windows.Forms.Screen GetScreen(Window window)
		{
			return System.Windows.Forms.Screen.FromHandle(new WindowInteropHelper(window).Handle);
		}

		#endregion Private Methods

		#region Callbacks

		public async void AppEventCallback(object sender, AppEventArgs e)
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType: " + e.EventType.GetStringValue(), "");

				if (e.EventType == AppEventType.UploadAllFileStart)
				{
					lock (_pInstanceLock)
					{
						this._isStartUploadFile = true;
					}
				}
				else if (e.EventType == AppEventType.UploadAllFileEnd)
				{
					lock (_pInstanceLock)
					{
						this._isStartUploadFile = false;
						this.Dispatcher.Invoke(delegate()
						{
							Thread.Sleep(500);
							this.txtbUploadStatus.Text = string.Empty;
							this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
							this.grdScreen.AllowDrop = true;

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.grdScreen.AllowDrop set to true", "");
						});
					}
				}
				else if (e.EventType == AppEventType.UploadFilePercentage)
				{
					lock (_pInstanceLock)
					{
						if (this._isStartUploadFile)
						{
							if (null != e.Data && e.Data is string[])
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType: update progress", "");

								// To update progress
								string[] infos = e.Data as string[];
								string progress = infos[0];
								string fileName = infos[1];
								this.Dispatcher.Invoke(delegate()
								{
									this.txtbUploadStatus.Text = string.Format(Application.Current.TryFindResource("txtUploadProgress") as String, progress, fileName);
									this.brdUploadStatus.Visibility = System.Windows.Visibility.Visible;
								});
								if (string.Compare(progress, "100", true) == 0)
									Thread.Sleep(500);
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType: update progress finish", "");

								this.Dispatcher.Invoke(delegate()
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get AppEventType: update progress finish clean all information", "");

									this.txtbUploadStatus.Text = string.Empty;
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.txtbUploadStatus.Text clean", "");
									this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.brdUploadStatus.Visibility collapsed", "");
									this.grdScreen.AllowDrop = true;

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.grdScreen.AllowDrop set to true", "");
								});

								//0411 Johnson modify, because we should not show MessageBox, it will show notification to hint user
								//string caption = Application.Current.TryFindResource("labelUploadCompleted") as String;
								//string message = Application.Current.TryFindResource("txtUploadCompletedMessage") as String;
								//MessageBoxResult result = this.ShowInformationMessageBox(message, caption);
							}
						}
					}
				}
				else if (e.EventType == AppEventType.DownloadFilePercentage)
				{
					if (null != e.Data)
					{
						if (e.Data is string[])
						{
							// To update progress
							string[] infos = e.Data as string[];
							string progress = infos[0];
							string fileName = infos[1];

							this.Dispatcher.Invoke(delegate()
							{
								this.txtbDownloadStatus.Text = string.Format(Application.Current.TryFindResource("txtDownloadProgress") as String, progress, fileName);
								this.brdDownloadStatus.Visibility = System.Windows.Visibility.Visible;
							});
						}
						else if (e.Data is string)
						{
							this.Dispatcher.Invoke(delegate()
							{
								this.txtbDownloadStatus.Text = string.Empty;
								this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
							});
						}
					}
				}
				else if (e.EventType == AppEventType.Disconnected || e.EventType == AppEventType.ScreenCastTCPDIsconnected)
				{
					if (this._IsShowDiconnectWarningMessgae)
					{
						//0701 Johnson add

						this.Dispatcher.Invoke(delegate()
						{
							this.dsElement.Visibility = System.Windows.Visibility.Hidden;
						});

						if (e.EventType == AppEventType.Disconnected)
						{
							ShowDebugWarningMessage(ErrorTypeMessage.VNCDisconnect, true);
							//message = Application.Current.TryFindResource("txtDisconnectedMessage") as String;
						}
						else if (e.EventType == AppEventType.ScreenCastTCPDIsconnected)
						{
							ShowDebugWarningMessage(ErrorTypeMessage.TCPDisconnect, true);
							//message = Application.Current.TryFindResource("txtScreenCastDisconnectedMessage") as String;
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Don't need to hint user disconnect message", "");
					}
				}
				//else if (e.EventType == AppEventType.CloseWindow)
				//{
				//	this.CloseWindow();
				//}
				else if (e.EventType == AppEventType.Unpair || e.EventType == AppEventType.ScreenOff)
				{
					if (e.Data is AFUserDevice)
					{
						this.AnimationCloseScreenwindow();

						//AFUserDevice device = e.Data as AFUserDevice;

						//if (null != Configs.ApplicationReference.ConnectingDeviceInfo)
						//{
						//	if (Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID.Equals(device.InstanceID))
						//	{
						//		this.AnimationCloseScreenwindow();
						//	}
						//}
					}
				}
				//else if (e.EventType == AppEventType.NoBTCapability)
				//{
				//	string caption = Application.Current.TryFindResource("labelNoBT") as String;
				//	string message = Application.Current.TryFindResource("txtNoBTMessage") as String;
				//	this.ShowInformationMessageBox(message, caption);
				//}
				else if (e.EventType == AppEventType.RotateDirection)
				{
					if (null != e.Data && e.Data is AFUserDevice)
					{
						AFUserDevice device = e.Data as AFUserDevice;

						if (null != e.Args)
						{
							PhoneSystemPermissionDetectionRule rule = (PhoneSystemPermissionDetectionRule)e.Args;

							double originalCenterX = 0;
							double originalCenterY = 0;

							this.Dispatcher.Invoke(delegate()
							{
								originalCenterX = this.Left + this.ActualWidth / 2.0;
								originalCenterY = this.Top + this.ActualHeight / 2.0;
							});

							this.SetRotation(device, rule, ref this._isPortraitMode, ref this._hasVirtualKey, ref this._videoSize, ref this._currentRenderRotationAngle);

							if (this.dsElement.IsGraphRunning)
							{
								this.Dispatcher.Invoke(delegate()
								{
									if (this._isMaximized)
										this.MaxmizeWindow();
									else
										this.SetScreenCastingLayout();
								});

								this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
								{
									//this.SetCenterScreen();

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start this.dsElement rotation", "");

									double angle = Utils.MappingRotationToAngle(this._currentRenderRotationAngle);
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "angle: " + angle, "");

									this.dsElement.LayoutTransform = new RotateTransform
									{
										Angle = angle,
									};

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.dsElement rotation -- Done!", "");

									this.grdDeviceListAdjust();

									this.Top = originalCenterY - this.ActualHeight / 2.0;
									this.Left = originalCenterX - this.ActualWidth / 2.0;
								});
							}
						}
					}
				}
				else if (e.EventType == AppEventType.StartReceiving)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Check window is loaded", "");

					// Wait for loading window
					if (this._windowIsLoaded.WaitOne(5000))
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Window is loaded, to do the following things", "");

						if (null != e.Data && e.Data is ConnectionType)
						{
							ConnectionType type = (ConnectionType)e.Data;

							this.Dispatcher.Invoke(delegate()
							{
								this.dsElement.StartReceiving(type);
							});
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Window is loaded fail, please check it.", "");
					}
				}
				else if (e.EventType == AppEventType.InitialStatus)
				{
					if (null != e.Data && e.Data is InitialStatus)
					{
						InitialStatus status = (InitialStatus)e.Data;

						this.Dispatcher.Invoke(delegate()
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "InitialStatus: " + status.GetStringValue(), "");
							switch (status)
							{
								#region InitialStatus Fail Case Handle

								case InitialStatus.DandelionReconfirmIPFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.DandelionReconfirmIPFail);
									}
									break;

								case InitialStatus.InitializeWakeUpFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeWakeUpFail);
									}
									break;

								case InitialStatus.InitializeConnectToVNCFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeConnectToVNCFail);
									}
									break;

								case InitialStatus.InputDaemonNotRunning:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InputDaemonNotRunning);
									}
									break;

								case InitialStatus.InitializeScreenCastDaemonFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeScreenCastDaemonFail);
									}
									break;

								case InitialStatus.InitializeInputControl:
									{
										this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitializeInputControlStatus") as String;
									}
									break;

								case InitialStatus.InitializeInputControlFailed:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeInputControlFailed);
									}
									break;

								case InitialStatus.RunInputControlFailed:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeConnectToVNCFail);
									}
									break;

								case InitialStatus.ConnectToRemoteServerFailed:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeInputControlFailed);
									}
									break;

								case InitialStatus.ReceiveDataFailed:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.ReceiveDataFailed);
									}
									break;

								case InitialStatus.DandelionGetIPFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.DandelionGetIPFail);
									}
									break;

								case InitialStatus.InitializeGetDeviceStatusFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeGetDeviceStatusFail);
									}
									break;

								case InitialStatus.InitializeGetDeviceInformationFail:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeGetDeviceInfoFail);
									}
									break;

								case InitialStatus.InitializeGetDeviceStatusTimeout:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeGetDeviceStatusTimeout);
									}
									break;

								case InitialStatus.InitializeGetDeviceInformationTimeout:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeGetDeviceInfoTimeout);
									}
									break;

								case InitialStatus.InitializeInputControlTimeout:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeGetInputDaemonStatusTimeout);
									}
									break;

								case InitialStatus.InitializeInputControlWebServiceException:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeInputControlWebServiceException);
									}
									break;

								case InitialStatus.InitializeScreencastDaemonTimeout:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeScreencastDaemonTimeout);
									}
									break;

								case InitialStatus.InitializeScreencastDaemonWebServiceException:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializeScreencastDaemonWebServiceException);
									}
									break;

								#endregion InitialStatus Fail Case Handle

								case InitialStatus.InitializeScreenCasting:
									{
										this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitializeScreenCastingStatus") as String;
									}
									break;

								case InitialStatus.InitailizationsCompleted:
									{
										this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitailizationsCompletedStatus") as String;
									}
									break;

								case InitialStatus.ReceiveDataCompleted:
									{
										//0612 Johnon add for enable show disconnect warning message
										this._IsShowDiconnectWarningMessgae = true;

										CurrentScreenCastdeviceInfo = Configs.ApplicationReference.ConnectingDeviceInfo;
										txtbConnectionStatus.Text = Application.Current.TryFindResource("txtReceiveDataCompletedStatus") as String;

										LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start this.dsElement.Run()", "");
										this.dsElement.Run(!Configs.ApplicationReference.IsAudioDisabled);
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.dsElement.Run() -- Done!", "");

										double angle = Utils.MappingRotationToAngle(this._currentRenderRotationAngle);
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "angle: " + angle, "");

										//0630 Johnson add
										try
										{
											if (Configs.ApplicationReference.ConnectingDeviceInfo != null)
											{
												if (Configs.ApplicationReference.ConnectingDeviceInfo.Device.Display.AutoRotation.HasValue)
												{
													this.ChangeRotateButtonStatus(Configs.ApplicationReference.ConnectingDeviceInfo.Device.Display.AutoRotation.Value);
												}
											}
										}
										catch (Exception ex)
										{
											LogtoFile.WriteLog("", Configs.DebugLevelValue, "ReceiveDataCompleted ChangeRotateButton Status ex: " + ex.ToString(), "");
										}

										this.dsElement.LayoutTransform = new RotateTransform
										{
											Angle = angle,
										};

										this.dsElement.SetCanRenderred();

										this.Dispatcher.Invoke(delegate()
										{
											this.dsElement.Opacity = 0.0;
											this.dsElement.Visibility = System.Windows.Visibility.Visible;
											Storyboard storyboardShowingScreencasting = this.TryFindResource("StoryboardShowingScreencasting") as Storyboard;
											if (storyboardShowingScreencasting != null)
												storyboardShowingScreencasting.Begin();
											this.dsElement.Opacity = 1.0;
										});

										this.grdScreen.AllowDrop = true;

										this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
										{
											// Initial the Clipboard function of VNC
											if (null != FlowControlManager.Instance.InputController)
											{
												if (FlowControlManager.Instance.InputController.IsInitializationSuccess)
												{
													FlowControlManager.Instance.InputController.VNCClient.InitialClipboard(_hWndSource.Handle);
												}
											}

											this.SetScreenSizeToVNC();
											ScreencastTimer();

											//0521 Johnson add for Enable Funciton bar
											ScreencastFuncionbarStatus(true);
											ScreencastRightMenuStatus(true);
											ScreencastSelectDeviceButtonStatus(true);

											this.launchingProgressbar.Value = 0;
										});
									}
									break;

								case InitialStatus.PhoneWakeUp:
									{
										this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitialWakeUp") as String;
									}
									break;

								case InitialStatus.GetDeviceStatus:
									{
										if ((int)e.Args == 0)
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtDeviceStatus") as String;
										else
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtDeviceStatus") as String + "retry " + e.Args.ToString();
									}
									break;

								case InitialStatus.GetDeviceInformation:
									{
										if ((int)e.Args == 0)
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtDeviceInformation") as String;
										else
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtDeviceInformation") as String + "retry " + e.Args.ToString();
									}
									break;

								case InitialStatus.InitializeConnectTOVNC:
									{
										if ((int)e.Args == 0)
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtConnectToVNC") as String;
										else
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtConnectToVNC") as String + "retry " + e.Args.ToString();
									}
									break;

								case InitialStatus.InitializeCheckVNCDaemonStatus:
									{
										if ((int)e.Args == 0)
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtCheckInputDaemonStauts") as String;
										else
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtCheckInputDaemonStauts") as String + "retry " + e.Args.ToString();
									}
									break;

								case InitialStatus.InitializeCheckScreencastDaemonStatus:
									{
										if ((int)e.Args == 0)
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtCheckScreencastDaemonStauts") as String;
										else
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtCheckScreencastDaemonStauts") as String + "retry " + e.Args.ToString();
									}
									break;

								case InitialStatus.InitializePlayScreencast:
									{
										if ((int)e.Args == 0)
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtPlayScreencastStauts") as String;
										else
											this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtPlayScreencastStauts") as String + "retry " + e.Args.ToString();
									}
									break;

								case InitialStatus.PlayScreencastTimeout:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializePlayScreencastTimeout);
									}
									break;

								case InitialStatus.PlayScreencastError:
									{
										ShowDebugWarningMessage(ErrorTypeMessage.InitializePlayScreencastFail);
									}
									break;

								default:
									break;
							}
						});
					}
				}
				else if (e.EventType == AppEventType.LaunchScreenCastWindow)
				{
					//if (this.Visibility != Visibility.Visible)
					//{
					//	this.Dispatcher.Invoke(new Action(() =>
					//	{
					//		//0430 Johnson add for clean warning dialog
					//		this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;

					//		//0521 Johnon add for disable fucntion bar when initial connection
					//		ScreencastFuncionbarStatus(false);

					//		// Reset UI status
					//		this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitializatingStatus") as String;
					//		this.dsElement.Visibility = System.Windows.Visibility.Collapsed;
					//		this.grdScreen.AllowDrop = false;
					//		this.txtbWarrningMessage.Text = string.Empty;

					//		this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
					//		this.txtbUploadStatus.Text = string.Empty;
					//		this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
					//		this.txtbDownloadStatus.Text = string.Empty;

					//		this.Opacity = 0.0;
					//		this.Visibility = System.Windows.Visibility.Visible;

					//		this.WindowState = WindowState.Normal;
					//		this.Activate();

					//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetInitialScreencastWindowInfo] Set Window Visible And Activated", "");
					//	}));
					//}
				}
				else if (e.EventType == AppEventType.PairAgain)
				{
					if (e.Data is AFUserDevice)
					{
						AFUserDevice device = e.Data as AFUserDevice;

						this.Dispatcher.Invoke(delegate()
						{
							if (this._hasShownDialog) return;

							//this._hasShownDialog = true;
							//this.txtbConnectionStatus.Text = string.Format(Application.Current.TryFindResource("txtPairAgainMessage") as String, device.DeviceName);
							ShowDebugWarningMessage(ErrorTypeMessage.UnpairDevice);
							//this.ShowWarningMessageBox(string.Format(Application.Current.TryFindResource("txtPairAgainMessage") as String, device.DeviceName),
							//																	  Application.Current.TryFindResource("labelPairAgain") as String);
							this._hasShownDialog = false;

							//this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.PairAgain));
						});
					}
				}
				else if (e.EventType == AppEventType.ConnectScreenCasting)
				{
					if (null != e.Data && e.Data is AppDeviceInfo)
					{
						AppDeviceInfo deviceInfo = e.Data as AppDeviceInfo;

						if (null != e.Args)
						{
							object[] args = (object[])e.Args;
							PhoneSystemPermissionDetectionRule rule = (PhoneSystemPermissionDetectionRule)args[0];
							bool isResetDevice = (bool)args[1];

							double originalCenterX = 0;
							double originalCenterY = 0;

							this.Dispatcher.Invoke(delegate()
							{
								originalCenterX = this.Left + this.ActualWidth / 2.0;
								originalCenterY = this.Top + this.ActualHeight / 2.0;
							});

							this.SetRotation(deviceInfo.Device, rule, ref this._isPortraitMode, ref this._hasVirtualKey, ref this._videoSize, ref this._currentRenderRotationAngle);

							this.Dispatcher.Invoke(delegate()
							{
								this.SetScreenCastingLayout();

								// Reset UI status
								this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitializatingStatus") as String;
								this.dsElement.Visibility = System.Windows.Visibility.Collapsed;
								this.grdScreen.AllowDrop = false;
								this.txtbWarrningMessage.Text = string.Empty;

								this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
								this.txtbUploadStatus.Text = string.Empty;
								this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
								this.txtbDownloadStatus.Text = string.Empty;

								// Set selected item
								//if (isResetDevice)
								{
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "isResetDevice is true, set selected device", "");

									//this._isDeviceFirstChanged = true;

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "The selected device name is: " + Configs.ApplicationReference.ConnectingDeviceInfo.Device.DeviceName, "");
									LogtoFile.WriteLog("", Configs.DebugLevelValue, "The selected device instance ID is: " + Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID, "");

									LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.lvDeviceList.Items.Count: " + this.lvDeviceList.Items.Count, "");

									if (this.lvDeviceList.Items.Count != 0)
									{
										try
										{
											var targetItem = this.lvDeviceList.Items.Cast<AppDeviceInfo>().Where(i => i.Device.InstanceID == Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID).FirstOrDefault();
											if (null != targetItem)
											{
												this._hasRemoveItem = false;
												this.lvDeviceList.SelectedItem = targetItem;
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set targetItem as selection item", "");
											}
											else
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "targetItem is null", "");
											}
										}
										catch (Exception ex)
										{
											LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
										}
									}
								}

								this.Visibility = System.Windows.Visibility.Visible;

								this.WindowState = WindowState.Normal;
								this.Activate();
							});

							this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
							{
								this.grdDeviceListAdjust();

								this.Top = originalCenterY - this.ActualHeight / 2.0;
								this.Left = originalCenterX - this.ActualWidth / 2.0;
							});
						}
					}
				}
				//else if (e.EventType == AppEventType.Reconnecting)
				//{
				//	try
				//	{
				//		this.Dispatcher.Invoke(delegate()
				//		{
				//			this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtReconnectingStatus") as String;
				//			this.dsElement.Visibility = System.Windows.Visibility.Collapsed;
				//			this.grdScreen.AllowDrop = false;

				//			this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
				//			this.txtbUploadStatus.Text = string.Empty;
				//			this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
				//			this.txtbDownloadStatus.Text = string.Empty;
				//		});
				//	}
				//	catch (Exception ex)
				//	{
				//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				//	}
				//}
				else if (e.EventType == AppEventType.VNCTransferFileError)
				{
					string caption = Application.Current.TryFindResource("labelTransferFail") as String;
					string message = Application.Current.TryFindResource("txtVNCTransferFileErrorMessage") as String;

					if (this.Visibility == System.Windows.Visibility.Visible)
					{
						if (this._hasShownDialog) return;

						this._hasShownDialog = true;
						this.ShowErrorMessageBox(message, caption);
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "AppEventType.VNCTransferFileError", "");
						this._hasShownDialog = false;
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Window has been closed", "");
					}
				}
				else if (e.EventType == AppEventType.CloseWindowBySystemSleep)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");
					//this.CloseWindow(false);
					this.DisconnecScreencast();
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
					ShowDebugWarningMessage(ErrorTypeMessage.HintScreenCastStoped);
				}
				else if (e.EventType == AppEventType.NextProgress)
				{
					if (e.Data != null && e.Args != null)
					{
						InitializingProgressbarArguments key = (InitializingProgressbarArguments)e.Data;
						int value = (int)e.Args;

						if (key.Equals(InitializingProgressbarArguments.MaxValue))
						{
							this.Dispatcher.Invoke(delegate()
							{
								this.launchingProgressbar.Maximum = value;
							});
						}
						else if (key.Equals(InitializingProgressbarArguments.Value))
						{
							this.Dispatcher.Invoke(delegate()
							{
								this.launchingProgressbar.Value = value;
							});
						}
					}
					else
					{
						this.Dispatcher.Invoke(delegate()
						{
							this.launchingProgressbar.Value++;
						});
					}
				}
				else if (e.EventType == AppEventType.RotateButtonStatus)
				{
					bool isEnable = (bool)e.Data;

					this.ChangeRotateButtonStatus(isEnable);
				}
				else if (e.EventType == AppEventType.NonAcerFlowFailStatus)
				{
					ConnectControlStatusType status = (ConnectControlStatusType)e.Data;
					ErrorTypeMessage errortype;

					switch (status)
					{
						case ConnectControlStatusType.ADBRemoteDeviceNotFound:
							errortype = ErrorTypeMessage.ADBListDeviceNotFound;
							break;

						case ConnectControlStatusType.ADBPullInputDaemonFail:
							errortype = ErrorTypeMessage.ADBPullInputDaemonFail;
							break;

						case ConnectControlStatusType.ADBPushInputDaemonFail:
							errortype = ErrorTypeMessage.ADBPushInputDaemonFail;
							break;

						case ConnectControlStatusType.ADBShellSetPrivilegeInputDaemonFail:
							errortype = ErrorTypeMessage.ADBShellSetPrivilegeInputDaemonFail;
							break;

						case ConnectControlStatusType.LaunchInputDaemonByADBException:
							errortype = ErrorTypeMessage.LaunchInputDaemonByADBException;
							break;

						case ConnectControlStatusType.ADBPullScreencastDaemonFail:
							errortype = ErrorTypeMessage.ADBPullScreencastDaemonFail;
							break;

						case ConnectControlStatusType.ADBPushScreencastDaemonFail:
							errortype = ErrorTypeMessage.ADBPushScreencastDaemonFail;
							break;

						case ConnectControlStatusType.ADBShellSetPrivilegeScreencastDaemonFail:
							errortype = ErrorTypeMessage.ADBShellSetPrivilegeScreencastDaemonFail;
							break;

						case ConnectControlStatusType.LaunchScreencastDaemonByADBException:
							errortype = ErrorTypeMessage.LaunchScreencastDaemonByADBException;
							break;

						default:
							errortype = ErrorTypeMessage.LaunchScreencastDaemonByADBException;
							break;
					}

					ShowDebugWarningMessage(errortype);
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void HookMessageCallback(IAsyncResult asyncResult)
		{
		}

		private IntPtr WinProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
		{
			switch (msg)
			{
				case Win32Api.WM_DISPLAYCHANGE:
					if (this.Visibility == System.Windows.Visibility.Visible)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Display changed!!!", "");

						this.WorkaroundForDisablingMonitorUIWillHang();
						if (this.dsElement.CheckIsPrimaryDisplayAdapterLost())
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Primary display adaptor lost!!!", "");

							Task.Run(() =>
							{
								this.dsElement.Pause();
							});
							this.ConnectNewScreencast(null);
						}
					}
					break;

				case Win32Api.WM_IME_NOTIFY:
					{
						//LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), WM_IME_NOTIFY", "");
					}
					break;

				case Win32Api.WM_IME_CHAR:
					{
						//LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), WM_IME_CHAR", "");
					}
					break;

				case Win32Api.WM_IME_SETCONTEXT:
					{
						//LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), WM_IME_SETCONTEXT", "");

						if (wParam.ToInt32() == 1)
						{
							//LogtoFile.WriteLog("", Configs.DebugLevelValue, "wParam == 1", "");

							Win32Api.ImmAssociateContext(_wih.Handle, _hImc);
						}
					}
					break;

				case Win32Api.WM_INPUTLANGCHANGE:
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), WM_INPUTLANGCHANGE", "");
					}
					break;

				case Win32Api.WM_DESTROY:
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), WM_DESTROY", "");
						try
						{
							VNC.VNCClient.RemoveClipboard(_hWndSource.Handle);
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), WM_DESTROY => Exception: " + ex.ToString(), "");
						}
					}
					break;

				default:
					{
						if (this._isSetScreenSizeToVNCNeeded)
						{
							this.Dispatcher.Invoke((Action)delegate()
							{
								this.SetScreenSizeToVNC();
							});

							this._isSetScreenSizeToVNCNeeded = false;
						}

						try
						{
							if (null != FlowControlManager.Instance.InputController)
							{
								// Check if user is entering IME character
								if (IsEnteringIMEChar(msg))
									break;

								// Check if the IME textbox is visible
								ToHideIMETextBox(msg);

								// Check if user is touching the screen
								if (IsTouchingInput(msg))
									break;

								if (FlowControlManager.Instance.InputController.IsInitializationSuccess)
								{
									if ((bool)this.tglbtnFunction.IsChecked) return IntPtr.Zero;
									if ((bool)this.tglbtnLandscape90Function.IsChecked) return IntPtr.Zero;
									if ((bool)this.tglbtnLandscape270Function.IsChecked) return IntPtr.Zero;

									if ((bool)this.tglbtnSelectDevice.IsChecked) return IntPtr.Zero;
									if ((bool)this.tglbtnLandscape90SelectDevice.IsChecked) return IntPtr.Zero;
									if ((bool)this.tglbtnLandscape270SelectDevice.IsChecked) return IntPtr.Zero;

									if (this._isFromLowerRightRectangle == true && this._isMouseLeftButtonPressed == true) return IntPtr.Zero;

									if ((bool)this.btnFullScreenRotate.IsMouseOver) return IntPtr.Zero;
									if ((bool)this.btnRecoverScreenSize.IsMouseOver) return IntPtr.Zero;

									if ((bool)this.grdTwoDlgBackground.IsVisible) return IntPtr.Zero;

									//0411 Johnson, When VNC filter transfer , we will disable VNC control
									//if (!this.grdScreen.AllowDrop) return IntPtr.Zero;
									FlowControlManager.Instance.InputController.VNCClient.WinProc(hwnd, msg, wParam, lParam, ref handled);

									//0508 Johnson add for detect timing to reset Stop Screen Cast Timer
									if (msg == Win32Api.WM_LBUTTONDOWN || msg == Win32Api.WM_RBUTTONDOWN)
									{
										if (this.grdTwoDlgBackground.Visibility != System.Windows.Visibility.Visible)
										{
											if (_pScreenCastTimer != null)
											{
												_pScreenCastTimer.Stop();
												_pScreenCastTimer.Start();
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "Screen Cast Window Detect Button Down Event so reset Stop Screen Cast Timer", "");
											}
										}
									}
								}
							}
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "WinProc(), Exception: " + ex.ToString(), "");
						}
					}
					break;
			}

			return IntPtr.Zero;
		}

		#endregion Callbacks

		#region UI Event Callbacks

		private void DragMoveScreenWindow(object sender, MouseButtonEventArgs e)
		{
			try
			{
				this.DragMove();

				this.Dispatcher.Invoke(delegate()
				{
					this.SetScreenCastingLayout();
				});
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][DragMoveScreenWindow] " + ex.ToString(), "");
			}
		}

		private void grdScreen_DragEnter(object sender, DragEventArgs e)
		{
			// only support file
			e.Effects = (e.AllowedEffects & DragDropEffects.Copy) == DragDropEffects.Copy && e.Data.GetDataPresent(DataFormats.FileDrop, false) ? DragDropEffects.Copy : DragDropEffects.None;
		}

		private void grdScreen_Drop(object sender, DragEventArgs e)
		{
			if ((e.AllowedEffects & DragDropEffects.Copy) != DragDropEffects.Copy)
			{
				return;
			}

			// The app only allows the drop source not from self program.
			if (!e.Data.GetDataPresent("AirFusionFromSelf", false))
			{
				string[] filesOut = (string[])e.Data.GetData(DataFormats.FileDrop, false);

				this.grdScreen.AllowDrop = false;
				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.UploadFiles, filesOut));
			}
		}

		private void dsElement_StylusSystemGesture(object sender, StylusSystemGestureEventArgs e)
		{
			return;
		}

		private void UserControlEventCallback(object sender, UserControlEventArgs e)
		{
			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get UserControlEventCallback: " + e.EventType.GetStringValue(), "");

				//Set Dlg Visibility to collapsed
				this.Dispatcher.Invoke(delegate()
				{
					this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Collapsed;
				});

				if (e.EventType == UserControlEventType.ReconnectionScreenCast)
				{
					if ((bool)e.Args)
					{
						ConnectNewScreencast(e.Data as AppDeviceInfo);
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Reject re-conection", "");
						//this.CloseWindow();
						this.AnimationCloseScreenwindow();
					}
				}
				else if (e.EventType == UserControlEventType.StopScreenCast)
				{
					ScreencastTimer(false);

					if (!(bool)e.Args)
					{
						this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");
							//this.CloseWindow();
							this.AnimationCloseScreenwindow();
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
						});
					}
				}
				else if (e.EventType == UserControlEventType.StopScreenHint)
				{
					ScreencastTimer(false);

					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");
						//this.CloseWindow(false);
						this.DisconnecScreencast();
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
						ShowDebugWarningMessage(ErrorTypeMessage.HintScreenCastStoped);
					});
				}
				else if (e.EventType == UserControlEventType.DetailDebugMessage)
				{
					ScreencastTimer(false);

					if (this.recordErrorType == null)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.recordErrorType is null , please check it", "");
					}

					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						ShowDebugWarningMessage(this.recordErrorType, false, true);
					});
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		#region Device List

		private ScrollViewer _svDeviceList;

		private void lvDeviceList_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			try
			{
				// Center the selected item
				try
				{
					if (null != this._svDeviceList && this.lvDeviceList.SelectedIndex != -1)
					{
						this._svDeviceList.ScrollToHorizontalOffset(this.lvDeviceList.SelectedIndex);
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}

				// Collapse the select device field
				this.ResetSelectDeviceFieldStatus();

				// Get selected device
				var listview = sender as ListView;
				AppDeviceInfo selectedTarget = listview.SelectedItem as AppDeviceInfo;

				if (null != selectedTarget)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get selected item, device name: " + selectedTarget.DeviceName, "");

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._hasRemoveItem: " + this._hasRemoveItem, "");
					if (!this._hasRemoveItem)
					{
						if (null != Configs.ApplicationReference.ConnectingDeviceInfo && selectedTarget.Device.InstanceID == Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Connection is already existed", "");
							return;
						}

						ConnectNewScreencast(selectedTarget);
						// Send restart screen casting event
						//this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.RestartScreenCasting, selectedTarget, false));
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "To clear selected item", "");
						this._hasRemoveItem = false;
						var targetListViewItem = this.lvDeviceList.ItemContainerGenerator.ContainerFromItem(selectedTarget) as ListViewItem;
						if (null != targetListViewItem)
						{
							try
							{
								targetListViewItem.IsSelected = false;
							}
							catch (Exception ex)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
							}
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "targetListViewItem is null", "");
						}
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Triggered from removing selected item", "");

					// Reset status
					this._hasRemoveItem = false;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
		}

		private void lvDeviceList_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "lvDeviceList_CollectionChanged", "");

			try
			{
				if (e.Action == NotifyCollectionChangedAction.Add)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Add device item to list", "");

					AppDeviceInfo newAppDevice = e.NewItems[0] as AppDeviceInfo;

					if (null != Configs.ApplicationReference.ConnectingDeviceInfo)
					{
						if (newAppDevice.Device.InstanceID == Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "New add device is the item needs to be selected", "");

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "The selected device name is: " + Configs.ApplicationReference.ConnectingDeviceInfo.Device.DeviceName, "");
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "The selected device instance ID is: " + Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID, "");

							LogtoFile.WriteLog("", Configs.DebugLevelValue, "this.lvDeviceList.Items.Count: " + this.lvDeviceList.Items.Count, "");

							if (null != newAppDevice)
							{
								this._hasRemoveItem = false;
								this.lvDeviceList.SelectedItem = newAppDevice;
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set newAppDevice as selection item", "");
							}
							else
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "newAppDevice is null", "");
							}
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Configs.ApplicationReference.ConnectingDeviceInfo is null", "");
					}
				}
				else if (e.Action == NotifyCollectionChangedAction.Remove)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Remove device item to list", "");

					this._hasRemoveItem = true;
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "***************** Set this._hasRemoveItem = true", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
		}

		private void ScrollToLeft(object sender, RoutedEventArgs e)
		{
			if ((this._svDeviceList != null))
			{
				if (this._isPortraitMode)
				{
					if (this._svDeviceList.HorizontalOffset > 0)
						this._svDeviceList.ScrollToHorizontalOffset(this._svDeviceList.HorizontalOffset - 1);
				}
				else
				{
					if (this._svDeviceList.VerticalOffset > 0)
						this._svDeviceList.ScrollToVerticalOffset(this._svDeviceList.VerticalOffset - 1);
				}
			}
		}

		private void ScrollToRight(object sender, RoutedEventArgs e)
		{
			if ((this._svDeviceList != null))
			{
				if (this._isPortraitMode)
				{
					if (this._svDeviceList.HorizontalOffset < this._svDeviceList.ScrollableWidth)
						this._svDeviceList.ScrollToHorizontalOffset(this._svDeviceList.HorizontalOffset + 1);
				}
				else
				{
					if (this._svDeviceList.VerticalOffset < this._svDeviceList.ScrollableHeight)
						this._svDeviceList.ScrollToVerticalOffset(this._svDeviceList.VerticalOffset + 1);
				}
			}
		}

		private void AdjustScrollButtonVisibility()
		{
			if (this.grdDeviceList.Visibility == Visibility.Visible)
			{
				this._svDeviceList.UpdateLayout();

				if (this._isPortraitMode)
				{
					if (this._svDeviceList.HorizontalOffset == 0)
						this.btnScrollLeft.Visibility = Visibility.Hidden;
					else
						this.btnScrollLeft.Visibility = Visibility.Visible;

					if (this._svDeviceList.HorizontalOffset == this._svDeviceList.ScrollableWidth)
						this.btnScrollRight.Visibility = Visibility.Hidden;
					else
						this.btnScrollRight.Visibility = Visibility.Visible;
				}
				else
				{
					if (this._svDeviceList.VerticalOffset == 0)
						this.btnScrollLeft.Visibility = Visibility.Hidden;
					else
						this.btnScrollLeft.Visibility = Visibility.Visible;

					if (this._svDeviceList.VerticalOffset == this._svDeviceList.ScrollableHeight)
						this.btnScrollRight.Visibility = Visibility.Hidden;
					else
						this.btnScrollRight.Visibility = Visibility.Visible;
				}
			}
		}

		private void ScrollChangedEventHandler(object sender, ScrollChangedEventArgs e)
		{
			if (this._svDeviceList != null)
				this.AdjustScrollButtonVisibility();
		}

		private void grdDeviceList_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			if (this.grdDeviceList.Visibility == Visibility.Visible)
			{
				this.tglbtnSelectDevice.IsChecked = true;
				this.tglbtnLandscape90SelectDevice.IsChecked = true;
				this.tglbtnLandscape270SelectDevice.IsChecked = true;

				this.bdrScreenMask.Visibility = Visibility.Visible;
			}
			else
			{
				this.ResetSelectDeviceFieldStatus();

				this.bdrScreenMask.Visibility = Visibility.Collapsed;
			}

			this.grdDeviceListAdjust();

			if (this.lvDeviceList.Visibility == Visibility.Visible)
			{
				// Get the border of the listview (first child of a listview)
				Decorator border = VisualTreeHelper.GetChild(this.lvDeviceList, 0) as Decorator;
				try
				{
					if (border != null)
					{
						// Get scrollviewer
						this._svDeviceList = VisualTreeHelper.GetChild(border, 0) as ScrollViewer;

						try
						{
							this._svDeviceList.ScrollChanged -= this.ScrollChangedEventHandler;
						}
						catch (Exception ex)
						{ }

						this._svDeviceList.ScrollChanged += this.ScrollChangedEventHandler;
					}
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###] Error: " + ex.ToString(), "");
				}
			}

			this.AdjustScrollButtonVisibility();
		}

		private void RequestBringIntoViewEventHandler(object sender, RequestBringIntoViewEventArgs e)
		{
			e.Handled = true;
		}

		private void grdDeviceListAdjust()
		{
			if (this.grdDeviceList.Visibility == Visibility.Visible)
			{
				switch (this._rotatedAngle)
				{
					case 0:
						{
							this.grdDeviceList.Height = 104;
							this.grdDeviceList.Width = this.grdScreen.Width;
							this.grdDeviceList.VerticalAlignment = VerticalAlignment.Top;
							this.grdDeviceList.HorizontalAlignment = HorizontalAlignment.Center;

							Grid.SetRow(this.rectDeviceListBorder, 1);
							Grid.SetRowSpan(this.rectDeviceListBorder, 1);
							Grid.SetColumn(this.rectDeviceListBorder, 0);
							Grid.SetColumnSpan(this.rectDeviceListBorder, 3);
							this.rectDeviceListBorder.Height = 2;
							this.rectDeviceListBorder.Width = this.grdDeviceList.Width;

							Grid.SetRow(this.grdDeviceListContent, 0);
							Grid.SetRowSpan(this.grdDeviceListContent, 1);
							Grid.SetColumn(this.grdDeviceListContent, 0);
							Grid.SetColumnSpan(this.grdDeviceListContent, 3);
							this.grdDeviceListContent.Height = this.grdDeviceList.Height - 2;
							this.grdDeviceListContent.Width = this.grdDeviceList.Width;

							this.lvDeviceList.Margin = new Thickness(26, 0, 26, 0);
							this.lvDeviceList.ItemsPanel = (ItemsPanelTemplate)System.Windows.Markup.XamlReader.Parse("<ItemsPanelTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"><StackPanel Orientation=\"Horizontal\"/></ItemsPanelTemplate>");

							this.btnScrollLeft.HorizontalAlignment = HorizontalAlignment.Left;
							this.btnScrollLeft.VerticalAlignment = VerticalAlignment.Center;
							this.btnScrollLeft.Style = this.FindResource("btnScrollLeftStyle") as Style;
							this.btnScrollRight.HorizontalAlignment = HorizontalAlignment.Right;
							this.btnScrollRight.VerticalAlignment = VerticalAlignment.Center;
							this.btnScrollRight.Style = this.FindResource("btnScrollRightStyle") as Style;

							break;
						}
					case 90:
						{
							this.grdDeviceList.Height = this.grdScreen.Height;
							this.grdDeviceList.Width = 104;
							this.grdDeviceList.VerticalAlignment = VerticalAlignment.Center;
							this.grdDeviceList.HorizontalAlignment = HorizontalAlignment.Left;

							Grid.SetRow(this.rectDeviceListBorder, 0);
							Grid.SetRowSpan(this.rectDeviceListBorder, 2);
							Grid.SetColumn(this.rectDeviceListBorder, 2);
							Grid.SetColumnSpan(this.grdDeviceListContent, 1);
							this.rectDeviceListBorder.Height = this.grdDeviceList.Height;
							this.rectDeviceListBorder.Width = 2;

							Grid.SetRow(this.grdDeviceListContent, 0);
							Grid.SetRowSpan(this.grdDeviceListContent, 2);
							Grid.SetColumn(this.grdDeviceListContent, 0);
							Grid.SetColumnSpan(this.grdDeviceListContent, 2);
							this.grdDeviceListContent.Height = this.grdDeviceList.Height;
							this.grdDeviceListContent.Width = this.grdDeviceList.Width - 2;

							this.lvDeviceList.Margin = new Thickness(0, 26, 0, 26);
							this.lvDeviceList.ItemsPanel = (ItemsPanelTemplate)System.Windows.Markup.XamlReader.Parse("<ItemsPanelTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"><StackPanel Orientation=\"Vertical\"/></ItemsPanelTemplate>");

							this.btnScrollLeft.HorizontalAlignment = HorizontalAlignment.Center;
							this.btnScrollLeft.VerticalAlignment = VerticalAlignment.Top;
							this.btnScrollLeft.Style = this.FindResource("btnScrollLeftStyle270") as Style;
							this.btnScrollRight.HorizontalAlignment = HorizontalAlignment.Center;
							this.btnScrollRight.VerticalAlignment = VerticalAlignment.Bottom;
							this.btnScrollRight.Style = this.FindResource("btnScrollRightStyle270") as Style;

							break;
						}
					case 270:
						{
							this.grdDeviceList.Height = this.grdScreen.Height;
							this.grdDeviceList.Width = 104;
							this.grdDeviceList.VerticalAlignment = VerticalAlignment.Center;
							this.grdDeviceList.HorizontalAlignment = HorizontalAlignment.Right;

							Grid.SetRow(this.rectDeviceListBorder, 0);
							Grid.SetRowSpan(this.rectDeviceListBorder, 2);
							Grid.SetColumn(this.rectDeviceListBorder, 0);
							Grid.SetColumnSpan(this.rectDeviceListBorder, 1);
							this.rectDeviceListBorder.Height = this.grdDeviceList.Height;
							this.rectDeviceListBorder.Width = 2;

							Grid.SetRow(this.grdDeviceListContent, 0);
							Grid.SetRowSpan(this.grdDeviceListContent, 2);
							Grid.SetColumn(this.grdDeviceListContent, 1);
							Grid.SetColumnSpan(this.grdDeviceListContent, 2);
							this.grdDeviceListContent.Height = this.grdDeviceList.Height;
							this.grdDeviceListContent.Width = this.grdDeviceList.Width - 2;

							this.lvDeviceList.Margin = new Thickness(0, 26, 0, 26);
							this.lvDeviceList.ItemsPanel = (ItemsPanelTemplate)System.Windows.Markup.XamlReader.Parse("<ItemsPanelTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"><StackPanel Orientation=\"Vertical\"/></ItemsPanelTemplate>");

							this.btnScrollLeft.HorizontalAlignment = HorizontalAlignment.Center;
							this.btnScrollLeft.VerticalAlignment = VerticalAlignment.Top;
							this.btnScrollLeft.Style = this.FindResource("btnScrollLeftStyle270") as Style;
							this.btnScrollRight.HorizontalAlignment = HorizontalAlignment.Center;
							this.btnScrollRight.VerticalAlignment = VerticalAlignment.Bottom;
							this.btnScrollRight.Style = this.FindResource("btnScrollRightStyle270") as Style;

							break;
						}
					default:
						break;
				}

				this.lvDeviceList.UpdateLayout();
			}
		}

		private void tglbtnSelectDevice_Unchecked(object sender, RoutedEventArgs e)
		{
			this.ResetFunctionFieldStatus();
			this.ResetSelectDeviceFieldStatus();
		}

		#endregion Device List

		#endregion UI Event Callbacks

		#region Custom Window Resize

		private bool _isMouseLeftButtonPressed = false;
		private bool _isInLowerRightRectangle = false;
		private bool _isFromLowerRightRectangle = false;
		private Point _lastMousePosition;
		private int _rotatedAngle = 0;

		private double _videoInitialLongSide;
		private double _videoInitialShortSide;
		private double _videoMinLongSide;
		private double _videoLongSide;
		private double _videoShortSide;
		private double _videoSizeRatio;

		private double _offsetXtoWindowLowerRightCorenr;
		private double _offsetYtoWindowLowerRightCorenr;
		private double _widthWithoutScreen;
		private double _heightWithoutScreen;
		private double _dpiRatio;

		[DllImport("user32.dll")]
		[return: MarshalAs(UnmanagedType.Bool)]
		internal static extern bool GetCursorPos(ref Win32Point pt);

		[StructLayout(LayoutKind.Sequential)]
		internal struct Win32Point
		{
			public Int32 X;
			public Int32 Y;
		};

		public static Point GetMousePosition()
		{
			Win32Point w32Mouse = new Win32Point();
			GetCursorPos(ref w32Mouse);
			return new Point(w32Mouse.X, w32Mouse.Y);
		}

		private void ShowResizeCursor(object sender, MouseEventArgs e)
		{
			this._isInLowerRightRectangle = true;
			Mouse.OverrideCursor = Cursors.SizeNWSE;
		}

		private void ShowNormalCursor(object sender, MouseEventArgs e)
		{
			this._isInLowerRightRectangle = false;

			if (!this._isMouseLeftButtonPressed)
			{
				Mouse.OverrideCursor = null;
			}
		}

		private void StartResize(object sender, MouseButtonEventArgs e)
		{
			// Make sure the setting screen casting window is finished
			if (!this._isFirstTimeSetScreenCastingLayout)
			{
				this._isMouseLeftButtonPressed = true;
				this._isFromLowerRightRectangle = true;
				this._lastMousePosition = GetMousePosition();

				MouseHook.Enabled = true;
				MouseHook.GlobalMouseUp += new EventHandler<MouseHook.MouseEventArgs>(MouseHook_GlobalMouseUp);
				MouseHook.GlobalMouseMove += new EventHandler<MouseHook.MouseEventArgs>(MouseHook_GlobalMouseMove);

				this._offsetXtoWindowLowerRightCorenr = this.ActualWidth - e.GetPosition(this).X;
				this._offsetYtoWindowLowerRightCorenr = this.ActualHeight - e.GetPosition(this).Y;
				this._widthWithoutScreen = this.ActualWidth - this.grdScreen.ActualWidth;
				this._heightWithoutScreen = this.ActualHeight - this.grdScreen.ActualHeight;

				this._lastMousePosition.Y /= this._dpiRatio;
			}
		}

		private bool isResizable;

		private void MouseHook_GlobalMouseMove(object sender, MouseHook.MouseEventArgs e)
		{
			try
			{
				if (this._isMouseLeftButtonPressed)
				{
					System.Drawing.Rectangle currentScreenWorkingArea = this.GetScreen(this).WorkingArea;
					double currentScreenWorkingAreaBottom = currentScreenWorkingArea.Bottom / this._dpiRatio;

					Point _currentMousePosition = GetMousePosition();
					_currentMousePosition.Y /= this._dpiRatio;

					double tmpHeight = _currentMousePosition.Y - this.Top + this._offsetYtoWindowLowerRightCorenr - this._heightWithoutScreen;
					double tmpWidth = this._isPortraitMode ? tmpHeight / this._videoSizeRatio : tmpHeight * this._videoSizeRatio;
					double _currentVideoLongSide = this._isPortraitMode ? tmpHeight : tmpWidth;

					this.isResizable = false;

					if (_currentVideoLongSide >= this._videoMinLongSide)
					{
						if ((this.Top + tmpHeight + this._heightWithoutScreen) < currentScreenWorkingAreaBottom &&
							(this.Left + tmpWidth + this._widthWithoutScreen) < SystemParameters.VirtualScreenWidth)
						{
							this.isResizable = true;
						}
						else if (_currentMousePosition.Y <= this._lastMousePosition.Y)
						{
							this.isResizable = true;
						}
						else
						{
							this.isResizable = false;
						}
					}
					else if (_currentMousePosition.Y >= this._lastMousePosition.Y)
					{
						this.isResizable = true;
					}
					else
					{
						this.isResizable = false;
					}

					if (this.isResizable && tmpHeight >= 0 && tmpWidth >= 0)
					{
						this.grdScreen.Height = tmpHeight;
						this.grdScreen.Width = tmpWidth;
						this._lastMousePosition = _currentMousePosition;

						this.txtbWarrningMessage.Width = (int)(grdScreen.ActualWidth * this._finalRatio) - 60;
						this.grdDeviceListAdjust();

						if (this.bdrVideoQualityList.Visibility == Visibility.Visible)
						{
							this.UpdateLayout();
							this.AdjustVideoQualityListPosition();
						}
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][MouseHook_GlobalMouseMove] " + ex.ToString(), "");
			}
		}

		private void MouseHook_GlobalMouseUp(object sender, MouseHook.MouseEventArgs e)
		{
			if (this._isFromLowerRightRectangle == true && this._isMouseLeftButtonPressed == true)
			{
				this.StopResize();

				if (!this._isInLowerRightRectangle)
					Mouse.OverrideCursor = null;
			}
		}

		private void StopResize()
		{
			this._isMouseLeftButtonPressed = false;
			this._isFromLowerRightRectangle = false;

			MouseHook.GlobalMouseUp -= new EventHandler<MouseHook.MouseEventArgs>(MouseHook_GlobalMouseUp);
			MouseHook.GlobalMouseMove -= new EventHandler<MouseHook.MouseEventArgs>(MouseHook_GlobalMouseMove);
			MouseHook.Enabled = false;

			this._videoLongSide = this._isPortraitMode ? this.grdScreen.Height : this.grdScreen.Width;
			this._videoShortSide = this._isPortraitMode ? this.grdScreen.Width : this.grdScreen.Height;
		}

		#endregion Custom Window Resize

		#region Full Screen

		private bool _isMaximized = false;

		private void btnScreenMaximize_Click(object sender, RoutedEventArgs e)
		{
			GoogleAnalytics.GoogleAnalyticSend("ScreencastWindow", "Screencast Maximize");

			this.MaxmizeWindow();
		}

		private void MaxmizeWindow()
		{
			//0428 Johnson add
			ScreencastTimer(false);

			this.ResetFunctionFieldStatus();
			this.ResetSelectDeviceFieldStatus();

			ScaleTransform scaleTransform = new ScaleTransform();
			scaleTransform.ScaleX = 1;
			scaleTransform.ScaleY = 1;
			this.grdScreenWindowBackground.LayoutTransform = scaleTransform;

			double fullScreenWindowHeight;
			double fullScreenWindowWidth;
			double fullScreenVideoHeight;
			double fullScreenVideoWidth;
			double NavigationButtonFieldHeight = 0;

			if (this.grdLandscapeNavigationButtonField.ActualWidth != 0)
			{
				NavigationButtonFieldHeight = this.grdLandscapeNavigationButtonField.ActualWidth;
			}
			else if (this.grdPortraitNavigationButtonField.ActualHeight != 0)
			{
				NavigationButtonFieldHeight = this.grdPortraitNavigationButtonField.ActualHeight;
			}

			System.Drawing.Rectangle currentScreenBounds = this.GetScreen(this).Bounds;
			fullScreenWindowHeight = (double)currentScreenBounds.Height / this._dpiRatio;
			fullScreenWindowWidth = (double)currentScreenBounds.Width / this._dpiRatio;

			if (!this._isMaximized)
			{
				this._isMaximized = true;
				this._windowStater.Save(this, this._finalRatio);
			}

			this._windowStater.Maximize(this, currentScreenBounds.X, currentScreenBounds.Y, fullScreenWindowWidth, fullScreenWindowHeight);

			if (this._isPortraitMode)
			{
				// Portrait mode
				this.grdLandscapeNavigationButtonField.Visibility = System.Windows.Visibility.Collapsed;
				this.grdPortraitNavigationButtonField.Visibility = this._hasVirtualKey ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
			}
			else
			{
				// Landscape mode
				this.grdPortraitNavigationButtonField.Visibility = System.Windows.Visibility.Collapsed;
				this.grdLandscapeNavigationButtonField.Visibility = this._hasVirtualKey ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
			}

			if (this._isPortraitMode)
			{
				fullScreenVideoHeight = fullScreenWindowHeight;
				fullScreenVideoHeight -= NavigationButtonFieldHeight;
				fullScreenVideoWidth = fullScreenVideoHeight / this._videoSizeRatio;

				if (fullScreenVideoWidth > fullScreenWindowWidth)
				{
					fullScreenVideoWidth = fullScreenWindowWidth;
					fullScreenVideoHeight = fullScreenVideoWidth * this._videoSizeRatio;
				}
			}
			else
			{
				fullScreenVideoWidth = fullScreenWindowWidth;
				fullScreenVideoWidth -= NavigationButtonFieldHeight;
				fullScreenVideoHeight = fullScreenVideoWidth / this._videoSizeRatio;
				//MessageBox.Show("1 " + fullScreenVideoWidth);

				if (fullScreenVideoHeight > fullScreenWindowHeight)
				{
					fullScreenVideoHeight = fullScreenWindowHeight;
					fullScreenVideoWidth = fullScreenVideoHeight * this._videoSizeRatio;
					//MessageBox.Show("2 " + fullScreenVideoWidth);
				}
			}

			this.stkpScreenWindowContent.Margin = new Thickness(0, 0, 0, 0);

			this.grdTitleArea.Visibility = Visibility.Collapsed;
			this.grdLandscape90TitleArea.Visibility = Visibility.Collapsed;
			this.grdLandscape270TitleArea.Visibility = Visibility.Collapsed;
			this.grdFunction.Visibility = Visibility.Collapsed;
			this.grdLandscape90Function.Visibility = Visibility.Collapsed;
			this.grdLandscape270Function.Visibility = Visibility.Collapsed;
			this.grdResizeArea.Visibility = Visibility.Collapsed;
			this.grdFullScreenNavigationButtonField.Visibility = Visibility.Visible;

			this.ResetFunctionFieldStatus();
			this.dkpScreenBackground.Visibility = Visibility.Collapsed;
			this.dkpScreenBackground90.Visibility = Visibility.Collapsed;
			this.dkpScreenBackground270.Visibility = Visibility.Collapsed;

			this.grdScreen.Width = fullScreenVideoWidth;
			this.grdScreen.Height = fullScreenVideoHeight;
		}

		private void btnRecoverScreenSize_Click(object sender, RoutedEventArgs e)
		{
			this.RecoverScreenSize();
		}

		private void RecoverScreenSize()
		{
			this._isMaximized = false;

			this.grdFullScreenNavigationButtonField.Visibility = Visibility.Collapsed;
			this.stkpScreenWindowContent.Margin = new Thickness(20, 20, 20, 20);
			this.grdResizeArea.Visibility = Visibility.Visible;

			switch (this._rotatedAngle)
			{
				case 0:
					{
						this.grdTitleArea.Visibility = Visibility.Visible;
						this.grdFunction.Visibility = Visibility.Visible;
						this.dkpScreenBackground.Visibility = Visibility.Visible;
						break;
					}
				case 90:
					{
						this.grdLandscape90TitleArea.Visibility = Visibility.Visible;
						this.grdLandscape90Function.Visibility = Visibility.Visible;
						this.dkpScreenBackground90.Visibility = Visibility.Visible;
						break;
					}
				case 270:
					{
						this.grdLandscape270TitleArea.Visibility = Visibility.Visible;
						this.grdLandscape270Function.Visibility = Visibility.Visible;
						this.dkpScreenBackground270.Visibility = Visibility.Visible;
						break;
					}
				default:
					break;
			}

			if (this._isPortraitMode)
			{
				this.grdScreen.Height = this._videoLongSide;
				this.grdScreen.Width = this._videoShortSide;
			}
			else
			{
				this.grdScreen.Height = this._videoShortSide;
				this.grdScreen.Width = this._videoLongSide;
			}

			this._windowStater.Restore(this);
			this.SetScreenCastingLayout();

			//Johnson modify
			ScreencastTimer(true);
		}

		#endregion Full Screen

		private void btnScrollToTheClickedButton(object sender, RoutedEventArgs e)
		{
		}

		#region Video Quality

		private void AddVQOptionToVQList(string rbtnVQOptionName, Action action, bool isDefaultVQ)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "AddVQOptionToVQList: " + rbtnVQOptionName, "");

			RadioButton rbtnVQOption = new RadioButton();

			rbtnVQOption.Content = rbtnVQOptionName;
			rbtnVQOption.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
			rbtnVQOption.VerticalAlignment = System.Windows.VerticalAlignment.Center;
			rbtnVQOption.Margin = new Thickness(2);
			rbtnVQOption.Width = 70;
			rbtnVQOption.Height = 15;
			rbtnVQOption.Style = Application.Current.FindResource("rbtnVideoQualitySelectionStyle") as Style;
			if (isDefaultVQ) rbtnVQOption.IsChecked = true;

			RoutedEventHandler ClickEventHandler = null;
			ClickEventHandler = (sender, e) =>
			{
				var rbtn = sender as RadioButton;
				rbtn.IsChecked = true;
				this.ResetFunctionFieldStatus();
				this.bdrVideoQualityList.Visibility = Visibility.Collapsed;
			};

			rbtnVQOption.Click += ClickEventHandler;

			RoutedEventHandler CheckedEventHandler = null;
			CheckedEventHandler = (sender, e) =>
			{
				action();
			};

			rbtnVQOption.Checked += CheckedEventHandler;

			this.stkpVideoQualityList.Children.Add(rbtnVQOption);
		}

		public void RemoveVQOptionToVQList()
		{
			this.Dispatcher.Invoke(delegate()
			{
				if (this.stkpVideoQualityList.Children.Count > 0)
				{
					this.stkpVideoQualityList.Children.RemoveRange(0, this.stkpVideoQualityList.Children.Count);
				}
			});
		}

		private void ChangeResolutionLaunch()
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "ChangeResolutionLaunch()", "");

			try
			{
				AppDeviceInfo selectedDeviceInfo = null;

				if (null != Configs.ApplicationReference.ConnectingDeviceInfo)
				{
					selectedDeviceInfo = Configs.ApplicationReference.ConnectingDeviceInfo;
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "e.Data is null", "");
				}

				if (null != selectedDeviceInfo)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "*** selectedDeviceInfo is not null", "");
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "*** selectedDeviceInfo is null", "");
				}

				if (null != selectedDeviceInfo)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get selected item, device name: " + selectedDeviceInfo.DeviceName, "");

					LogtoFile.WriteLog("", Configs.DebugLevelValue, "this._hasRemoveItem: " + this._hasRemoveItem, "");

					// Send restart screen casting event
					ConnectNewScreencast(selectedDeviceInfo);
					//BackgroundWorker worker = new BackgroundWorker();
					//worker.DoWork += delegate(object s, DoWorkEventArgs k)
					//{
					//	try
					//	{
					//		this.dsElement.Close();

					//		Configs.ApplicationReference.ChangeVQReconnection(selectedDeviceInfo);
					//	}
					//	catch (Exception ex)
					//	{
					//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					//	};
					//};
					//worker.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs k)
					//{
					//};
					//worker.RunWorkerAsync();
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Triggered from removing selected item", "");

					// Reset status
					this._hasRemoveItem = false;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
		}

		#endregion Video Quality

		private void grdScreenButtonField_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			Grid grdTarget = sender as Grid;

			if (grdTarget.Visibility == System.Windows.Visibility.Visible)
				this.bdrScreenMask.Visibility = Visibility.Visible;
			else
			{
				this.bdrVideoQualityList.Visibility = Visibility.Collapsed;
				this.bdrScreenMask.Visibility = Visibility.Collapsed;
			}
		}

		private void ShowWarningMessage(AppDeviceInfo appdevice, ref string caption, ref string message, ref string btnLeftContent, ref string btnRightContent, WarningMessageType warningType)
		{
			switch (warningType)
			{
				case WarningMessageType.UnableToConnect:
					{
						caption = ReadingResoureceWording(@"txtConnectionFailedTitle");
						message = ReadingResoureceWording(@"txtConnectionFailedDetail");
						if (!string.IsNullOrEmpty(message) && appdevice != null)
						{
							message = string.Format(message, appdevice.DeviceName);
						}
						btnLeftContent = ReadingResoureceWording(@"txtRetry");
						btnRightContent = string.Empty;
					}
					break;

				case WarningMessageType.NotOnTheSameNetWork:
					{
						caption = ReadingResoureceWording(@"txtNotSameNetworkTitle");
						message = ReadingResoureceWording(@"txtNotSameNetworkDetail");
						btnLeftContent = ReadingResoureceWording(@"txtRetry");
						btnRightContent = string.Empty;
					}
					break;

				case WarningMessageType.ScreencastingSuspended:
					{
						caption = ReadingResoureceWording(@"txtScreencastSuspend");
						message = ReadingResoureceWording(@"txtScreencastSuspendDetail");
						btnLeftContent = ReadingResoureceWording(@"txtOK");
						btnRightContent = ReadingResoureceWording(@"txtCancel");
					}
					break;

				case WarningMessageType.ScreencastingStopped:
					{
						caption = ReadingResoureceWording(@"txtScreencastStop");
						message = ReadingResoureceWording(@"txtScreencastStopDetail");
						btnLeftContent = ReadingResoureceWording(@"txtConnect");
						btnRightContent = string.Empty;
					}
					break;

				case WarningMessageType.ConnectionFailed:
					{
						caption = ReadingResoureceWording(@"txtConnectionFail");
						message = ReadingResoureceWording(@"txtConnectionFailDetail");
						btnLeftContent = ReadingResoureceWording(@"txtRetry");
						btnRightContent = string.Empty;
					}
					break;

				case WarningMessageType.Reboot:
					{
						caption = ReadingResoureceWording(@"txtConnectionFail");
						message = ReadingResoureceWording(@"txtConnectionFailReboot");
						btnLeftContent = string.Empty;
						btnRightContent = string.Empty;
					}
					break;

				case WarningMessageType.UnpairDevice:
					{
						caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;
						if (appdevice != null)
						{
							message = string.Format(ReadingResoureceWording("txtPairAgainMessage"), appdevice.DeviceName);
						}
						else
						{
							message = ReadingResoureceWording("txtPairAgainMessage");
						}
						btnLeftContent = string.Empty;
						btnRightContent = string.Empty;
					}
					break;
			}
		}

		//0420 Johnson add for show debug use warring message
		private void ShowDebugWarningMessage(ErrorTypeMessage errorType, bool isCheckScreenCastVisibiltiy = false, bool isAlwaysDebug = false)
		{
			LogtoFile.TrackEnter();

			try
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Get ShowDebugWarningMessage: " + errorType.GetStringValue(), "");

				if (this._hasShownDialog) return;

				if (this.grdTwoDlgBackground.Visibility == System.Windows.Visibility.Visible)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Becasue previous screen has been show, so don't show this screen.", "");
					return;
				}

				if (this._windowIsLoaded.WaitOne(5000))
				{
					if (this.Visibility == System.Windows.Visibility.Visible)
					{
						if (this.dsElement.Visibility != System.Windows.Visibility.Visible || isCheckScreenCastVisibiltiy)
						{
							AppDeviceInfo ScreencastdeviceInfo = null;

							if (null != Configs.ApplicationReference.ConnectingDeviceInfo)
							{
								ScreencastdeviceInfo = Configs.ApplicationReference.ConnectingDeviceInfo;
							}
							else
							{
								ScreencastdeviceInfo = Configs.ApplicationReference.GetLastLaunchDeviceInfo();

								LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage ScreenCastdeviceInfo is null", "");
							}

							this.Dispatcher.Invoke(delegate()
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "To show warning message", "");
								this._hasShownDialog = true;
								this.txtbConnectionStatus.Text = string.Empty;

								//0522 Johnson add
								if (this._isMaximized)
									this.RecoverScreenSize();

								string caption = string.Empty, message = string.Empty, btnDebugContent = string.Empty;
								string btnLeftContent = ReadingResoureceWording("txtRetry"); //Application.Current.TryFindResource("txtRetry") as String;
								string btnRightContent = ReadingResoureceWording("txtClose"); //Application.Current.TryFindResource("txtClose") as String;
								UserControlEventType controlEventType = UserControlEventType.ReconnectionScreenCast;

								//0623 Johnon add
								this.recordErrorType = errorType;

								if (this.IsEnableDebugMode() || isAlwaysDebug)
								{
									switch (errorType)
									{
										#region Wakeup Warning Message

										case ErrorTypeMessage.DandelionReconfirmIPFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is DandelionUpdateIPFail", "");

												string strConfirmIP = string.Empty;

												if (ScreencastdeviceInfo.ReconfirmAddressResult != null)
												{
													foreach (var ipAddress in ScreencastdeviceInfo.ReconfirmAddressResult.Addresses)
													{
														strConfirmIP += " IP (" + ipAddress.ToString() + ")";
													}
												}

												caption = Application.Current.TryFindResource("txtDandelion") as String;
												message = "Failed waking up (" + ScreencastdeviceInfo.LastServerIP.ToString() + ")\nDandelion Get" + strConfirmIP + " ping Timeout.\nPlease check with RD.";//Application.Current.TryFindResource("txtDandelionUpdateIPFail") as String + strConfirmIP;
												//this.txtbWarrningMessage.Text = Application.Current.TryFindResource("txtDandelionUpdateIPFail") as String;
											}
											break;

										case ErrorTypeMessage.DandelionGetIPFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is DandelionGetIPFail", "");

												caption = Application.Current.TryFindResource("txtDandelion") as String;
												message = Application.Current.TryFindResource("txtDandelionGetIPFail") as String;
											}
											break;

										case ErrorTypeMessage.InitializeWakeUpFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeWakeUpFail", "");

												caption = Application.Current.TryFindResource("txtWakeUpTitle") as String;
												message = string.Format(Application.Current.TryFindResource("txtWakeUpFail") as String, ScreencastdeviceInfo.LastServerIP.ToString());
											}
											break;

										#endregion Wakeup Warning Message

										#region Get Device Status Warning

										case ErrorTypeMessage.InitializeGetDeviceStatusFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeGetDeviceStatusFail", "");

												caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;
												message = Application.Current.TryFindResource("txtGetDeviceStatusFail") as String;
											}
											break;

										case ErrorTypeMessage.InitializeGetDeviceStatusTimeout:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeGetDeviceStatusTimeout", "");

												caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;
												message = Application.Current.TryFindResource("txtGetDeviceStatusTimout") as String;
											}
											break;

										case ErrorTypeMessage.UnpairDevice:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeGetDeviceStatusTimeout", "");

												caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;

												if (ScreencastdeviceInfo != null)
												{
													message = string.Format(Application.Current.TryFindResource("txtPairAgainMessage") as String, ScreencastdeviceInfo.DeviceName); ;
												}
												else
												{
													message = Application.Current.TryFindResource("txtPairAgainMessage") as String;
												}

												btnLeftContent = string.Empty;
												btnRightContent = string.Empty;
											}
											break;

										#endregion Get Device Status Warning

										#region Get Device Info Warning

										case ErrorTypeMessage.InitializeGetDeviceInfoFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeGetDeviceInfoFail", "");

												caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;
												message = Application.Current.TryFindResource("txtGetDeviceInfoFail") as String;
											}
											break;

										case ErrorTypeMessage.InitializeGetDeviceInfoTimeout:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeGetDeviceInformationTimeout", "");

												caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;
												message = Application.Current.TryFindResource("txtGetDeviceInfoTimout") as String;
											}
											break;

										#endregion Get Device Info Warning

										#region Connect VNC Warning

										case ErrorTypeMessage.InitializeConnectToVNCFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeConnectToVNCFail", "");

												caption = Application.Current.TryFindResource("txtInputControlTitle") as String;
												message = Application.Current.TryFindResource("txtConnectToVNCFail") as String;
											}
											break;

										//For old RTSP Method Use
										case ErrorTypeMessage.InitializeInputControlFailed:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeInputControlFailed", "");

												caption = Application.Current.TryFindResource("txtConnectionFailedTitle") as String;
												message = Application.Current.TryFindResource("txtNotConnectableMessage") as String;
												//for Finial Version use
												//message = string.Format(Application.Current.TryFindResource("txtConnectionFailedDetail") as String, screenCastdeviceInfo.DeviceName);
											}
											break;

										#endregion Connect VNC Warning

										#region Input Daemon Status Warning

										case ErrorTypeMessage.InputDaemonNotRunning:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeVNCDaemonFail", "");

												caption = Application.Current.TryFindResource("txtInputControlTitle") as String;
												message = Application.Current.TryFindResource("txtVNCDaemonFail") as String;
											}
											break;

										case ErrorTypeMessage.InitializeGetInputDaemonStatusTimeout:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeInputControlTimeout", "");

												caption = Application.Current.TryFindResource("txtInitialFailTitle") as String;
												message = Application.Current.TryFindResource("txtInputControlStatusTimout") as String;
											}
											break;

										case ErrorTypeMessage.InitializeInputControlWebServiceException:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeInputControlWebServiceException", "");

												caption = Application.Current.TryFindResource("txtInitialControlTitle") as String;
												message = Application.Current.TryFindResource("txtInputControlStatusWebServiceException") as String;
											}
											break;

										#endregion Input Daemon Status Warning

										#region Screencast Daemon Warning

										case ErrorTypeMessage.InitializeScreenCastDaemonFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeScreenCastDaemonFail", "");

												caption = Application.Current.TryFindResource("txtScreenCastTitle") as String;
												message = Application.Current.TryFindResource("txtScreenCastDaemonFail") as String;
											}
											break;

										case ErrorTypeMessage.InitializeScreencastDaemonTimeout:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeScreencastDaemonTimeout", "");

												caption = Application.Current.TryFindResource("txtInitialControlTitle") as String;
												message = Application.Current.TryFindResource("txtScreencastStatusTimout") as String;
											}
											break;

										case ErrorTypeMessage.InitializeScreencastDaemonWebServiceException:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializeScreencastDaemonWebServiceException", "");

												caption = Application.Current.TryFindResource("txtInitialControlTitle") as String;
												message = Application.Current.TryFindResource("txtScreencastStatusWebServiceException") as String;
											}
											break;

										#endregion Screencast Daemon Warning

										#region Play Screencast Warning

										case ErrorTypeMessage.InitializePlayScreencastTimeout:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializePlayScreencastFail", "");

												caption = Application.Current.TryFindResource("txtInitialControlTitle") as String;
												message = Application.Current.TryFindResource("txtPlayScreencastTimout") as String;
											}
											break;

										case ErrorTypeMessage.InitializePlayScreencastFail:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is InitializePlayScreencastFail", "");

												caption = Application.Current.TryFindResource("txtInitialControlTitle") as String;
												message = Application.Current.TryFindResource("txtPlayScreencastError") as String;
											}
											break;

										#endregion Play Screencast Warning

										case ErrorTypeMessage.ReceiveDataFailed:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is ReceiveDataFailed", "");

												caption = Application.Current.TryFindResource("txtScreenCastTitle") as String;
												message = Application.Current.TryFindResource("txtScreenReceivedFail") as String;
											}
											break;

										#region Disconnect Warning

										case ErrorTypeMessage.VNCDisconnect:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is VNCDisconnect", "");

												caption = Application.Current.TryFindResource("labelDisconnected") as String;
												message = Application.Current.TryFindResource("txtDisconnectedMessage") as String;
											}
											break;

										case ErrorTypeMessage.TCPDisconnect:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is TCPDisconnect", "");

												caption = Application.Current.TryFindResource("labelDisconnected") as String;
												message = Application.Current.TryFindResource("txtScreenCastDisconnectedMessage") as String;
											}
											break;

										#endregion Disconnect Warning

										#region Auto Stop Screencast Warning

										case ErrorTypeMessage.TimerStopScreenCast:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is TimerStopScreenCast", "");

												caption = Application.Current.TryFindResource("txtScreencastSuspend") as String;
												message = Application.Current.TryFindResource("txtScreencastSuspendDetail") as String;
												btnLeftContent = Application.Current.TryFindResource("txtOK") as String;
												btnRightContent = Application.Current.TryFindResource("txtNo") as String;
												controlEventType = UserControlEventType.StopScreenCast;
												//this.lclTwoBtnDialog.txtRemainTimeDetail.Text = @"The Screen will be close in ";
											}
											break;

										case ErrorTypeMessage.HintScreenCastStoped:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is HintScreenCastStoped", "");

												caption = Application.Current.TryFindResource("txtScreencastStop") as String;
												message = string.Format(Application.Current.TryFindResource("txtScreencastStopDetail") as String);
												btnLeftContent = Application.Current.TryFindResource("txtConnect") as String;
											}
											break;

										#endregion Auto Stop Screencast Warning

										default:
											break;
									};
								}
								else
								{
									switch (errorType)
									{
										#region Wakeup Warning Message

										case ErrorTypeMessage.DandelionReconfirmIPFail:
										case ErrorTypeMessage.DandelionGetIPFail:
										case ErrorTypeMessage.InitializeWakeUpFail:
											{
												if (this.IsEnableDetailButton())
												{
													btnDebugContent = @"Detail Info";
												}
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.NotOnTheSameNetWork);
											}
											break;

										#endregion Wakeup Warning Message

										case ErrorTypeMessage.UnpairDevice:
											{
												//btnDebugContent = @"Detail Info";
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.UnpairDevice);
											}
											break;

										case ErrorTypeMessage.InitializeGetDeviceStatusFail:
										case ErrorTypeMessage.InitializeGetDeviceInfoFail:
										case ErrorTypeMessage.InitializeConnectToVNCFail:
										case ErrorTypeMessage.InitializeInputControlFailed:
										case ErrorTypeMessage.InitializePlayScreencastFail:
										case ErrorTypeMessage.InitializeInputControlWebServiceException:
										case ErrorTypeMessage.InitializeScreencastDaemonWebServiceException:
										case ErrorTypeMessage.ReceiveDataFailed:
											{
												if (this.IsEnableDetailButton())
												{
													btnDebugContent = @"Detail Info";
												}
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.ConnectionFailed);
											}
											break;

										case ErrorTypeMessage.InitializeGetDeviceStatusTimeout:
										case ErrorTypeMessage.InitializeGetDeviceInfoTimeout:
										case ErrorTypeMessage.InitializeGetInputDaemonStatusTimeout:
										case ErrorTypeMessage.InitializeScreencastDaemonTimeout:
										case ErrorTypeMessage.InitializePlayScreencastTimeout:
										case ErrorTypeMessage.InputDaemonNotRunning:
										case ErrorTypeMessage.InitializeScreenCastDaemonFail:
											{
												if (this.IsEnableDetailButton())
												{
													btnDebugContent = @"Detail Info";
												}
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.UnableToConnect);
											}
											break;

										//For old RTSP Method Use

										//case ErrorTypeMessage.InputDaemonNotRunning:
										//case ErrorTypeMessage.InitializeScreenCastDaemonFail:
										//	{
										//		if (this.IsEnableDetailButton())
										//		{
										//			btnDebugContent = @"Detail Info";
										//		}
										//		ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.Reboot);
										//	}
										//	break;

										case ErrorTypeMessage.VNCDisconnect:
										case ErrorTypeMessage.TCPDisconnect:
											{
												if (this.IsEnableDetailButton())
												{
													btnDebugContent = @"Detail Info";
												}
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.ConnectionFailed);
											}
											break;

										case ErrorTypeMessage.TimerStopScreenCast:
											{
												controlEventType = UserControlEventType.StopScreenCast;
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.ScreencastingSuspended);
											}
											break;

										case ErrorTypeMessage.HintScreenCastStoped:
											{
												ShowWarningMessage(ScreencastdeviceInfo, ref caption, ref message, ref btnLeftContent, ref btnRightContent, WarningMessageType.ScreencastingStopped);
											}
											break;

										case ErrorTypeMessage.ADBListDeviceError:
										case ErrorTypeMessage.ADBListDeviceNotFound:
										case ErrorTypeMessage.ADBPullInputDaemonFail:
										case ErrorTypeMessage.ADBPushInputDaemonFail:
										case ErrorTypeMessage.ADBShellSetPrivilegeInputDaemonFail:
										case ErrorTypeMessage.LaunchInputDaemonByADBException:
										case ErrorTypeMessage.ADBPullScreencastDaemonFail:
										case ErrorTypeMessage.ADBPushScreencastDaemonFail:
										case ErrorTypeMessage.ADBShellSetPrivilegeScreencastDaemonFail:
										case ErrorTypeMessage.LaunchScreencastDaemonByADBException:
											{
												LogtoFile.WriteLog("", Configs.DebugLevelValue, "DebugWarringMessage type is " + errorType.GetStringValue(), "");

												caption = "Non-Acer Flow Fail";
												message = "Fail Type is " + errorType.GetStringValue() + ", please check RD about detail info. Thanks a lot.";
											}
											break;

										default:
											break;
									};
								}

								this.lclTwoBtnDialog.CommandType = controlEventType;
								this.lclTwoBtnDialog.TargetDeviceInfo = ScreencastdeviceInfo;
								this.lclTwoBtnDialog.txtDialogTitle.Text = caption;
								this.lclTwoBtnDialog.txtDialogDetail.Text = message;
								this.lclTwoBtnDialog.btnLeft.Content = btnLeftContent;
								this.lclTwoBtnDialog.btnRight.Content = btnRightContent;
								this.lclTwoBtnDialog.btnDebug.Content = btnDebugContent;
								this.lclTwoBtnDialog.imgPresent.Source = Application.Current.TryFindResource("imgUnpairImage") as BitmapImage;
								this.grdTwoDlgBackground.Visibility = System.Windows.Visibility.Visible;
								this._hasShownDialog = false;
							});

							if (errorType != ErrorTypeMessage.TimerStopScreenCast)
							//if (isCheckScreenCastVisibiltiy == false)
							{
								LogtoFile.WriteLog("", Configs.DebugLevelValue, "To close window", "");
								//this.CloseWindow(false);
								this.DisconnecScreencast();
							}
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Not to show warning message", "");
						}
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Window has been closed", "");
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Window is loaded fail, please check it.", "");
				}
			}
			finally
			{
				ScreencastSelectDeviceButtonStatus(true);
				LogtoFile.TrackLeave();
			}
		}

		public void ConnectNewScreencast(AppDeviceInfo deviceInfo)
		{
			LogtoFile.TrackEnter();

			if (null == deviceInfo)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "ConnectNewScreencast deviceInfo is null so use registry set value to launch it.", "");

				deviceInfo = Configs.ApplicationReference.GetLastLaunchDeviceInfo();
			}

			if (null != deviceInfo)
			{
				LogtoFile.TrackEnter();

				this._IsShowDiconnectWarningMessgae = false;

				this.ResetScreenWindowsStatus();

				Task.Run(() =>
				{
					this.DisconnectDirectShowFilter();

					FlowControlManager.Instance.NewConnectionDisconnect();

					Configs.ApplicationReference.StartConnectRemoteDevice(deviceInfo);
				});

				LogtoFile.TrackLeave();
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "ReConnectScreencast deviceInfo is null", "");
			}

			LogtoFile.TrackLeave();

			//LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start ReConnectScreenCast", "");

			//BackgroundWorker worker = new BackgroundWorker();
			//worker.DoWork += delegate(object s, DoWorkEventArgs k)
			//{
			//	try
			//	{
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "Accept re-conection, Start release Comm object", "");
			//		this.dsElement.Close();
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "Accept re-conection, End release Comm object", "");

			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, "Accept re-conection", "");
			//		this.Dispatcher.Invoke(delegate()
			//		{
			//			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Before reconnecting...", "");
			//			if (null != deviceInfo)
			//			{
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "selectedDeviceInfo is not null", "");

			//				// Reset UI status
			//				this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtReconnectingStatus") as String;
			//				this.dsElement.Visibility = System.Windows.Visibility.Collapsed;
			//				this.grdScreen.AllowDrop = false;

			//				this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
			//				this.txtbUploadStatus.Text = string.Empty;
			//				this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
			//				this.txtbDownloadStatus.Text = string.Empty;

			//				// Send restart screen casting event
			//				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.RestartScreenCasting, deviceInfo, true));
			//			}
			//			else
			//			{
			//				LogtoFile.WriteLog("", Configs.DebugLevelValue, "ReConnectScreenCast deviceInfo is null", "");
			//			}
			//		});
			//	}
			//	catch (Exception ex)
			//	{
			//		LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			//	};
			//};
			//worker.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs k)
			//{
			//};
			//worker.RunWorkerAsync();
		}

		public void DisconnectScreencast()
		{
			this.AnimationCloseScreenwindow();
		}

		public void IsShowTextInputBox(bool isShowTextBox, bool isDistory = false)
		{
			this.Dispatcher.Invoke(delegate
			{
				if (isShowTextBox)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== Show Text input box ===", "");

					this.IMEInputGrid.Visibility = System.Windows.Visibility.Visible;
					Keyboard.Focus(this.txtbxTextInputContent);
				}
				else
				{
					if (this.txtbxTextInputContent.Text != string.Empty)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== send text to remote device ===", "");

						string textString = this.txtbxTextInputContent.Text;
						FlowControlManager.Instance.InputController.VNCClient.SendIMEString(textString);
						this.txtbxTextInputContent.Clear();
					}

					if (this.IsVirtualKeyboardShow() == false || isDistory)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== Hide Text input box ===", "");

						this.txtbxTextInputContent.Clear();
						this.IMEInputGrid.Visibility = System.Windows.Visibility.Hidden;
						Keyboard.Focus(this);
					}
				}
			});
		}

		public bool CheckTextInputBoxShow()
		{
			if (this.IMEInputGrid.Visibility == System.Windows.Visibility.Visible)
				return true;
			else
				return false;
		}

		public bool isForegroundWindow()
		{
			IntPtr TopWindow = IntPtr.Zero;
			TopWindow = Win32Api.NativeMethod.GetForegroundWindow();

			IntPtr ScreenWindow = IntPtr.Zero;
			ScreenWindow = Win32Api.NativeMethod.FindWindow(string.Empty, @"AcerEXTEND");

			if (IntPtr.Equals(TopWindow, this))
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< isForegroundWindow >>>>> true ", "");
				return true;
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< isForegroundWindow >>>>> false ", "");
				return false;
			}
		}

		/// <summary>
		/// Changes the rotate button status.
		/// </summary>
		/// <param name="isEnable">if set to <c>true</c> [is enable].</param>
		private void ChangeRotateButtonStatus(bool isEnable)
		{
			this.Dispatcher.Invoke(delegate()
			{
				this.btnScreenRotate.IsEnabled = isEnable;
				this.btnScreenRotate90.IsEnabled = isEnable;
				this.btnScreenRotate270.IsEnabled = isEnable;
			});
		}

		private void ScreencastTimer(bool enable = true)
		{
			//0428 Johnson add for start cal screen cast timer

			if (enable)
			{
				if (_pScreenCastTimer != null)
				{
					_pScreenCastTimer.Stop();
					_pScreenCastTimer = null;
				}

				_pScreenCastTimer = new System.Timers.Timer();

				_pScreenCastTimer.Interval = ScreenCast_MONITOR_TIME;
				_pScreenCastTimer.Elapsed += StopScreenCast;
				_pScreenCastTimer.Start();
			}
			else
			{
				if (_pScreenCastTimer != null)
				{
					_pScreenCastTimer.Stop();
				}

				_pScreenCastTimer = null;
			}
		}

		private void StopScreenCast(object sender, EventArgs e)
		{
			bool isX86;
			object oRstopscreencastK = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegVideoPath, Configs.RegDisableStopScreencastEventKey, RegistryView.Registry64, out isX86);
			int disableStopScreencast = 0;
			if (oRstopscreencastK != null)
			{
				disableStopScreencast = (int)oRstopscreencastK;
			}

			if (disableStopScreencast == 0)
				ShowDebugWarningMessage(ErrorTypeMessage.TimerStopScreenCast, true);

			ScreencastTimer(false);
		}

		private void SetADBScreencastResoultion(int videoWidth, int videoHeight)
		{
			try
			{
				FlowControlManager.Instance.m_iVideoWidth = videoWidth;
				FlowControlManager.Instance.m_iVideoHeight = videoHeight;
				//Configs.ApplicationReference._mScreenCastWidth = videoWidth;
				//Configs.ApplicationReference._mScreenCastHeight = videoHeight;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void grdScreen_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			this.WindowSizeChangedEventHandler();
		}

		private void grdScreenWindowBackground_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			this.WindowSizeChangedEventHandler();
		}

		private void WindowSizeChangedEventHandler()
		{
			this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
			{
				this._isSetScreenSizeToVNCNeeded = true;

				if (!this._isMaximized)
				{
					this.Width = double.NaN;
					this.Height = double.NaN;

					this.SizeToContent = SizeToContent.WidthAndHeight;
				}

				try
				{
					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath + @"\InitialScreenSize", "VideoShortSide", this._videoShortSide, RegistryView.Registry64);
					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath + @"\InitialScreenSize", "VideoLongSide", this._videoLongSide, RegistryView.Registry64);
				}
				catch (Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][WindowSizeChangedEventHandler] " + ex.ToString(), "");
				}
			});
		}

		private void ScreencastFuncionbarStatus(bool isEnableFunctionbar = false)
		{
			this.Dispatcher.BeginInvoke(new Action(() =>
			{
				this.tglbtnFunction.IsEnabled = isEnableFunctionbar;
				this.tglbtnLandscape90Function.IsEnabled = isEnableFunctionbar;
				this.tglbtnLandscape270Function.IsEnabled = isEnableFunctionbar;
			}));
		}

		private void ScreencastRightMenuStatus(bool isEnableRightMenu = false)
		{
			this.Dispatcher.BeginInvoke(new Action(() =>
			{
				this.RCMenu.IsEnabled = isEnableRightMenu;
				if (isEnableRightMenu == false)
				{
					this.RCMenu.Visibility = Visibility.Collapsed;
				}
				else
				{
					this.RCMenu.Visibility = Visibility.Visible;
				}
			}));
		}

		private void ScreencastSelectDeviceButtonStatus(bool isEnableSelectDevice = false)
		{
			this.Dispatcher.BeginInvoke(new Action(() =>
			{
				this.tglbtnSelectDevice.IsEnabled = isEnableSelectDevice;
				this.tglbtnLandscape90SelectDevice.IsEnabled = isEnableSelectDevice;
				this.tglbtnLandscape270SelectDevice.IsEnabled = isEnableSelectDevice;
			}));
		}

		//0611 Johnson modify remark

		public void SetInitialScreencastWindowInfo(AppDeviceInfo deviceInfo, RemoteDevicePermission rule)
		{
			this.Dispatcher.Invoke(new Action(() =>
			{
				//0430 Johnson add for clean warning dialog
				this.grdTwoDlgBackground.Visibility = Visibility.Collapsed;
				this.brdNetworkStatus.Visibility = Visibility.Collapsed;

				if (this.Visibility != Visibility.Visible)
				{
					//0521 Johnon add for disable fucntion bar when initial connection
					ScreencastFuncionbarStatus(false);
					ScreencastRightMenuStatus(false);
					// Reset UI status
					this.txtbConnectionStatus.Text = Application.Current.TryFindResource("txtInitializatingStatus") as String;
					this.dsElement.Visibility = System.Windows.Visibility.Collapsed;
					this.grdScreen.AllowDrop = false;
					this.txtbWarrningMessage.Text = string.Empty;

					this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
					this.txtbUploadStatus.Text = string.Empty;
					this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
					this.txtbDownloadStatus.Text = string.Empty;

					this.Opacity = 0.0;
					this.Visibility = System.Windows.Visibility.Visible;

					this.WindowState = WindowState.Normal;
					this.Activate();
				}

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetInitialScreencastWindowInfo] Set Window Visible And Activated", "");

				if (this.lvDeviceList.Items.Count != 0)
				{
					try
					{
						var targetItem = this.lvDeviceList.Items.Cast<AppDeviceInfo>().Where(i => i.Device.InstanceID == Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID).FirstOrDefault();
						if (null != targetItem)
						{
							this._hasRemoveItem = false;
							this.lvDeviceList.SelectedItem = targetItem;
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Set targetItem as selection item", "");
						}
						else
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "targetItem is null", "");
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					}
				}
			}));

			// Wait for loading window
			if (this._windowIsLoaded.WaitOne(5000))
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetInitialScreencastWindowInfo] Window is loaded", "");

				if (!this._isFirstTimeSetScreenCastingLayout)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetInitialScreencastWindowInfo] Is Not First Time To Set ScreenCastingLayout", "");

					double originalCenterX = 0;
					double originalCenterY = 0;

					this.Dispatcher.Invoke(delegate()
					{
						originalCenterX = this.Left + this.grdScreenWindowBackground.ActualWidth / 2.0;
						originalCenterY = this.Top + this.grdScreenWindowBackground.ActualHeight / 2.0;
					});

					if (rule != RemoteDevicePermission.NotDetect)
					{
						this.SetRotation(deviceInfo.Device, (PhoneSystemPermissionDetectionRule)rule, ref this._isPortraitMode,
							ref this._hasVirtualKey, ref this._videoSize, ref this._currentRenderRotationAngle);
					}

					this.Dispatcher.BeginInvoke(DispatcherPriority.Loaded, (Action)delegate()
					{
						this.SetScreenCastingLayout();
					});

					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						this.Left = originalCenterX - this.grdScreenWindowBackground.ActualWidth / 2.0;
						this.Top = originalCenterY - grdScreenWindowBackground.ActualHeight / 2.0;

						this.grdDeviceListAdjust();

						if (this.Opacity != 1.0)
						{
							Storyboard storyboardShowingMainWindow = this.FindResource("StoryboardShowingMainWindow") as Storyboard;
							storyboardShowingMainWindow.Begin();
							this.Opacity = 1.0;
						}
					});
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetInitialScreencastWindowInfo] Is First Time To Set ScreenCastingLayout", "");

					if (rule != RemoteDevicePermission.NotDetect)
					{
						this.SetRotation(deviceInfo.Device, (PhoneSystemPermissionDetectionRule)rule, ref this._isPortraitMode,
							ref this._hasVirtualKey, ref this._videoSize, ref this._currentRenderRotationAngle);
					}

					this.Dispatcher.BeginInvoke(DispatcherPriority.Loaded, (Action)delegate()
					{
						this.SetScreenCastingLayout();

						if (rule != RemoteDevicePermission.NotDetect)
							this._isFirstTimeSetScreenCastingLayout = false;
					});

					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						this.SetCenterScreen();

						this.grdDeviceListAdjust();

						if (this.Opacity != 1.0)
						{
							Storyboard storyboardShowingMainWindow = this.FindResource("StoryboardShowingMainWindow") as Storyboard;
							storyboardShowingMainWindow.Begin();
							this.Opacity = 1.0;
						}
					});
				}
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "[###][SetInitialScreencastWindowInfo] Window is loaded fail, please check it", "");
			}
		}

		/// <summary>
		/// The function will based on registry to return is enable debug mode
		/// default value is false
		/// </summary>
		private bool IsEnableDebugMode()
		{
			LogtoFile.TrackEnter();

			try
			{
				bool isX86;
				string displayMode = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath, "Connecting Information Display Mode", RegistryView.Registry64, out isX86) as string;

				//Set to Default value
				if (!string.IsNullOrEmpty(displayMode) && displayMode.CompareTo("Message") == 0)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Enable Debug Mode", "");

					return true;
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Disable Debug Mode", "");

					return false;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Some excepiton, so disable debug mode ex:" + ex.ToString(), "");

				return false;
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		private bool IsEnableDetailButton()
		{
			LogtoFile.TrackEnter();

			try
			{
				bool isX86;
				string displayMode = RegistryUtility.GetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath, "DetailButton", RegistryView.Registry64, out isX86) as string;

				//Set to Default value
				if (!string.IsNullOrEmpty(displayMode) && displayMode.CompareTo("Yes") == 0)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Enable Detail Button", "");

					return true;
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Disable Detail Button", "");

					return false;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Some excepiton, so disable debug mode ex:" + ex.ToString(), "");

				return false;
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		public bool IsVirtualKeyboardShow()
		{
			IntPtr VirtualKeyboardHwnd = IntPtr.Zero;
			VirtualKeyboardHwnd = Win32Api.NativeMethod.FindWindow(@"IPTip_Main_Window", string.Empty);

			if (VirtualKeyboardHwnd != IntPtr.Zero)
			{
				UInt32 VirtualKeyboardWindowStyle = (UInt32)Win32Api.NativeMethod.GetWindowLong(VirtualKeyboardHwnd, GWL_STYLE);
				if ((VirtualKeyboardWindowStyle & WS_DISABLED) == 0
					&& (VirtualKeyboardWindowStyle & WS_VISIBLE) != 0)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< IsVirtualKeyboardShow >>>>> true ", "");
					return true;
				}
			}

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "<<<<< IsVirtualKeyboardShow >>>>> false ", "");
			return false;
		}

		/// <summary>
		/// Sets the screenwindow button Tool Tip
		/// </summary>
		/// ToDoIt Johnson
		private void SetToolTipWording()
		{
			this.tglbtnSelectDeviceToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipSelectDevice");
			this.tglbtnLandscape90SelectDeviceToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipSelectDevice");
			this.tglbtnLandscape270SelectDeviceToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipSelectDevice");

			this.btnDeviceManagerToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipDeviceManager");
			this.btnLandscape90DeviceManagerToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipDeviceManager");
			this.btnLandscape270DeviceManagerToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipDeviceManager");

			this.btnScreenRotateToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipRotate");
			this.btnScreenRotate90ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipRotate");
			this.btnScreenRotate270ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipRotate");

			this.btnScreenMaximizeToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipFullScreen");
			this.btnScreenMaximize90ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipFullScreen");
			this.btnScreenMaximize270ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipFullScreen");

			this.btnScreenKeyboardToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipPCKeyboard");
			this.btnScreenKeyboard90ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipPCKeyboard");
			this.btnScreenKeyboard270ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipPCKeyboard");

			this.btnScreenSnapshotToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipSnapshot");
			this.btnScreenSnapshot90ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipSnapshot");
			this.btnScreenSnapshot270ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipSnapshot");

			this.btnScreenRecordToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipVideoQuality");
			this.btnScreenRecord90ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipVideoQuality");
			this.btnScreenRecord270ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipVideoQuality");

			this.btnScreenResizeToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipOneToOne");
			this.btnScreenResize90ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipOneToOne");
			this.btnScreenResize270ToolTip.Text = ReadingResoureceWording(@"txtScreencastToolTipOneToOne");
		}

		/// <summary>
		/// Closes the screencast window.
		/// </summary>
		private void AnimationCloseScreenwindow()
		{
			LogtoFile.TrackEnter();

			this.Dispatcher.Invoke(delegate()
			{
				Storyboard storyboardClosingMainWindow = this.FindResource("StoryboardClosingMainWindow") as Storyboard;

				EventHandler eventHandler = null;
				eventHandler = (sender, e) =>
				{
					this.DisconnecScreencast(true);
					storyboardClosingMainWindow.Completed -= eventHandler;
				};

				storyboardClosingMainWindow.Completed += eventHandler;

				storyboardClosingMainWindow.Begin();
			});

			LogtoFile.TrackLeave();
		}

		/// <summary>
		/// Disconnecs the screencast.
		/// </summary>
		private void DisconnecScreencast(bool isHideWindow = false)
		{
			LogtoFile.TrackEnter();
			this.Dispatcher.Invoke(delegate
			{
				this.ResetScreenWindowsStatus(isHideWindow);

				//ScreencastTimer(false);

				//this.dsElement.Visibility = System.Windows.Visibility.Hidden;

				//if (isHideWindow == true)
				//{
				//	this.Visibility = System.Windows.Visibility.Hidden;
				//}
			});

			Task.Run(() =>
			{
				this.DisconnectDirectShowFilter();

				FlowControlManager.Instance.NewConnectionDisconnect();

				Configs.ApplicationReference.ConnectingDeviceInfo = null;
			});

			LogtoFile.TrackLeave();
		}

		private void DisconnectDirectShowFilter()
		{
			LogtoFile.TrackEnter();

			this.dsElement.Close();

			LogtoFile.TrackLeave();
		}

		private void ResetScreenWindowsStatus(bool isHideWindow = false)
		{
			LogtoFile.TrackEnter();

			this.ResetAllFieldStauts();
			this.ClearScreenwindowMessage();

			this.grdScreen.AllowDrop = false;
			this.lvDeviceList.SelectedIndex = -1;

			ScreencastSelectDeviceButtonStatus(false);

			//0727 Johnson add for resolve not hiden dsElemnet bug for next connection
			ScreencastTimer(false);
			this.Dispatcher.Invoke(delegate
			{
				this.dsElement.Visibility = System.Windows.Visibility.Hidden;

				if (this._isMaximized)
					this.RecoverScreenSize();

				if (isHideWindow == true)
				{
					this.Visibility = System.Windows.Visibility.Hidden;
				}
			});

			LogtoFile.TrackLeave();
		}

		private void ClearScreenwindowMessage()
		{
			LogtoFile.TrackEnter();

			this.txtbConnectionStatus.Text = string.Empty;
			this.txtbWarrningMessage.Text = string.Empty;
			this.brdUploadStatus.Visibility = System.Windows.Visibility.Collapsed;
			this.txtbUploadStatus.Text = string.Empty;
			this.brdDownloadStatus.Visibility = System.Windows.Visibility.Collapsed;
			this.txtbDownloadStatus.Text = string.Empty;

			LogtoFile.TrackLeave();
		}

		#region Window-related Event Handlers

		private void Window_Activated(object sender, EventArgs e)
		{
			//Chech Touch Keyboard is show or not.

			IntPtr VirtualKeyboardHwnd = IntPtr.Zero;
			VirtualKeyboardHwnd = Win32Api.NativeMethod.FindWindow(@"IPTip_Main_Window", string.Empty);

			if (VirtualKeyboardHwnd != IntPtr.Zero && this.CheckTextInputBoxShow() == false)
			{
				UInt32 VirtualKeyboardWindowStyle = (UInt32)Win32Api.NativeMethod.GetWindowLong(VirtualKeyboardHwnd, GWL_STYLE);
				if ((VirtualKeyboardWindowStyle & WS_DISABLED) == 0 && (VirtualKeyboardWindowStyle & WS_VISIBLE) != 0)
				{
					this.IsShowTextInputBox(true);
				}
			}
		}

		private void Window_SourceInitialized(object sender, EventArgs e)
		{
			InputMethod.SetPreferredImeState(this, InputMethodState.On);
			InputMethod.SetPreferredImeConversionMode(this, ImeConversionModeValues.DoNotCare);

			this.txtbxTextInputContent.LostKeyboardFocus += new KeyboardFocusChangedEventHandler(IMETextKeyboardUnFocusHandler);
			this.txtbxTextInputContent.MouseEnter += new MouseEventHandler(IMETextBoxMouseEnter);
			this.txtbxTextInputContent.MouseLeave += new MouseEventHandler(IMETextBoxMouseLeave);

			EnableTouchHandle();
		}

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Start", "");

			// For VNC hook Windows events
			HookMessageDelete work = new HookMessageDelete(this.HookMessage);
			work.BeginInvoke(HookMessageCallback, work);
			_hImc = Win32Api.ImmGetContext(_wih.Handle);

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Hook windows events -- Done!", "");

			// This is important for getting OS message
			this.Focus();

			// For reset and override mouse cursor
			Mouse.OverrideCursor = null;

			// To set screen cast layout when this window is loaded.
			//this.SetScreenCastingLayout();

			// To fire a signal to inform others that this window is loaded.
			if (null != this._windowIsLoaded)
			{
				this._windowIsLoaded.Set();
			}

			//0420 Johnson modify, because sometime grdScreen.ActualWidth value will be a NAN value
			//so we can't set  txtbWarrningMessage width, we need to change method to set the value
			//this.txtbWarrningMessage.Width = (int)(grdScreen.ActualWidth * this._finalRatio) - 60;

			LogtoFile.WriteLog("", Configs.DebugLevelValue, "End", "");
			SetToolTipWording();
			HookManager.SubscribeToWindowEvents();
			RCMenuWindowsSize.Click += (RCMenuWindowsSizeclickSender, RCMenuWindowsSizeclickEventArgs) =>
			{
				//if(!this._isMaximized)
				//    this.MaxmizeWindow();
				//else
				//    this.RecoverScreenSize();
				this.MaxmizeWindow();
			};

			RCMenuPaste.Click += (clickSender, clickEventArgs) =>
			{
				FlowControlManager.Instance.InputController.VNCClient.RightClickToPaste();
			};
		}

		private void Window_Closed(object sender, EventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();
				//this.CollapsedScreencastWindow();
				//this.dsElement.Close();
				this.RemoveHookMessage();
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		private void Window_Closing(object sender, CancelEventArgs e)
		{
			LogtoFile.TrackEnter();

			this.AnimationCloseScreenwindow();
			e.Cancel = true;

			LogtoFile.TrackLeave();
		}

		private void Window_StateChanged(object sender, EventArgs e)
		{
			if (this.WindowState == WindowState.Maximized)
			{
				this.WindowState = this._isMaximized ? WindowState.Maximized : WindowState.Normal;
			}
		}

		private void Window_PreviewKeyDown(object sender, KeyEventArgs e)
		{
			if (Keyboard.Modifiers == ModifierKeys.Alt && e.Key == Key.System && e.SystemKey == Key.F4)
			{
				// Handle alt + F4 hotkey event
				this.AnimationCloseScreenwindow();
				e.Handled = true;
			}
			else if (e.Key == Key.Escape && this._isMaximized == true)
			{
				this.RecoverScreenSize();
				e.Handled = true;
			}
		}

		private void Window_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			// For reset and override mouse cursor
			Mouse.OverrideCursor = null;
		}

		#endregion Window-related Event Handlers

		#region Screenwindow Button Click Event Handle

		#region Select Device Button about Portrait/Landscape90/Landscape270

		private void tglbtnSelectDevice_Checked(object sender, RoutedEventArgs e)
		{
			this.ResetFunctionFieldStatus();

			if (this.tglbtnLandscape90SelectDevice.IsChecked == false && this.tglbtnLandscape270SelectDevice.IsChecked == false)
			{
				this.tglbtnLandscape90SelectDevice.IsChecked = true;
				this.tglbtnLandscape270SelectDevice.IsChecked = true;

				Storyboard storyboardShowingPortraitgrdDeviceList = this.grdDeviceList.FindResource("StoryboardShowingPortraitgrdDeviceList") as Storyboard;
				storyboardShowingPortraitgrdDeviceList.Begin();
			}
		}

		private void tglbtnLandscape90SelectDevice_Checked(object sender, RoutedEventArgs e)
		{
			this.ResetFunctionFieldStatus();

			this.ResetFunctionFieldStatus();

			if (this.tglbtnSelectDevice.IsChecked == false && this.tglbtnLandscape270SelectDevice.IsChecked == false)
			{
				this.tglbtnSelectDevice.IsChecked = true;
				this.tglbtnLandscape270SelectDevice.IsChecked = true;

				Storyboard storyboardShowingLandscapegrdDeviceList = this.grdDeviceList.FindResource("StoryboardShowingLandscapegrdDeviceList") as Storyboard;
				storyboardShowingLandscapegrdDeviceList.Begin();
			}
		}

		private void tglbtnLandscape270SelectDevice_Checked(object sender, RoutedEventArgs e)
		{
			this.ResetFunctionFieldStatus();

			if (this.tglbtnSelectDevice.IsChecked == false && this.tglbtnLandscape90SelectDevice.IsChecked == false)
			{
				this.tglbtnSelectDevice.IsChecked = true;
				this.tglbtnLandscape90SelectDevice.IsChecked = true;

				Storyboard storyboardShowingLandscapegrdDeviceList = this.grdDeviceList.FindResource("StoryboardShowingLandscapegrdDeviceList") as Storyboard;
				storyboardShowingLandscapegrdDeviceList.Begin();
			}
		}

		#endregion Select Device Button about Portrait/Landscape90/Landscape270

		/// <summary>
		/// Handles the Click event of the MainSetting control. It will show DeviceManager
		/// </summary>
		private void btnMainSetting_Click(object sender, RoutedEventArgs e)
		{
			LogtoFile.TrackEnter();
			this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.ShowDeviceManager));

			GoogleAnalytics.GoogleAnalyticSend("ScreencastWindow", "ShowDeviceManager");

			LogtoFile.TrackLeave();
		}

		/// <summary>
		/// Handles the Click event of the Minimize Button
		/// </summary>
		private void btnScreenWindowMinimize_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();
				this.ResetAllFieldStauts();
				this.WindowState = System.Windows.WindowState.Minimized;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				LogtoFile.TrackLeave();
			}
		}

		/// <summary>
		/// Handles the Click event of the screenwindow close button.
		/// </summary>
		private void btnScreenWindowClose_Click(object sender, RoutedEventArgs e)
		{
			LogtoFile.TrackEnter();
			this.AnimationCloseScreenwindow();
			LogtoFile.TrackLeave();
		}

		#region tglbFunctionChecked

		private void tglbtnFunction_Checked(object sender, RoutedEventArgs e)
		{
			this.ResetSelectDeviceFieldStatus();
			ToggleButton functionButton = (ToggleButton)sender;
			functionButton.IsChecked = true;
		}

		#endregion tglbFunctionChecked

		private void btnScreenRotate_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();

				e.Handled = true;
				this.ResetFunctionFieldStatus();
				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SwitchRotationClicked));
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				GoogleAnalytics.GoogleAnalyticSend("ScreencastWindow", "Rotate Screencast");

				LogtoFile.TrackLeave();
			}
		}

		private void btnScreenKeyboard_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();
				this.ResetFunctionFieldStatus();
				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.SwitchKeyboardClicked, null, this._isToVitual));
				this._isToVitual = !this._isToVitual;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				GoogleAnalytics.GoogleAnalyticSend("ScreencastWindow", "Switch Keyboard");

				LogtoFile.TrackLeave();
			}
		}

		private void btnScreenSnapshot_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				LogtoFile.TrackEnter();

				this.ResetFunctionFieldStatus();
				string filePullPath = this.dsElement.SnapShot(Configs.ScreenshotFileFolderPath, this._currentRenderRotationAngle);
				if (!string.IsNullOrEmpty(filePullPath))
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "file Path: " + filePullPath, "");
					this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.GetSnapshot, filePullPath));

					if (System.IO.File.Exists(filePullPath))
					{
						if (this._storyboardgrdScreenSnapshot == null)
							this._storyboardgrdScreenSnapshot = this.FindResource("StoryboardgrdScreenSnapshot") as Storyboard;

						this.imgSnapshot.Source = Utils.CreateBitmapSourceByFile(filePullPath);
						this._storyboardgrdScreenSnapshot.Stop();
						this._storyboardgrdScreenSnapshot.Begin();
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Failed to create snapshot!", "");
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
			finally
			{
				GoogleAnalytics.GoogleAnalyticSend("ScreencastWindow", "Snapshot");

				LogtoFile.TrackLeave();
			}
		}

		private void StoryboardgrdScreenSnapshot_Completed(object sender, EventArgs e)
		{
			this.imgSnapshot.Source = null;
			this.imgSnapshot.UpdateLayout();
		}

		private void btnScreenRecord_Click(object sender, RoutedEventArgs e)
		{
			//this.ResetFunctionFieldStatus();

			//0511 Johnson add for dynamic show change resolution list
			if (this.stkpVideoQualityList.Children.Count == 0)
			{
				if (Configs.ApplicationReference.ConnectingDeviceInfo != null)
				{
					//var store = Configs.ApplicationReference._manager.UserDeviceManager.DeviceStore;
					AFUserDevice.ServicesInfo.ScreencastInfo _currentScreenCastInfo = Configs.ApplicationReference.ConnectingDeviceInfo.Device.Services.Screencast;

					try
					{
						AirFusion.Manager.DeviceManager.AFUserDevice.ServicesInfo.ScreencastInfo.ScreenResolution theLastVQ = _currentScreenCastInfo.SupportedResolutions.Last();
						foreach (AirFusion.Manager.DeviceManager.AFUserDevice.ServicesInfo.ScreencastInfo.ScreenResolution resolution in _currentScreenCastInfo.SupportedResolutions)
						{
							bool _bCheckDefaultType = resolution == theLastVQ ? true : false;
							if (resolution.Width > 300)
							{
								Action action = delegate()
								{
									Configs.ApplicationReference._mManualReset = true;
									Configs.ApplicationReference.SetADBScreenRecoderParameter(BiteRateType.Low, KeyFrameValueType.Manual);
									this.SetADBScreencastResoultion(resolution.Width, resolution.Height);
									ChangeResolutionLaunch();
								};

								this.AddVQOptionToVQList(resolution.Width.ToString() + "p", action, _bCheckDefaultType);
							}
						}
					}
					catch (Exception ex)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "btnScreenRecord_Click exception: " + ex.ToString(), "");
					}
				}
				else
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Configs.ApplicationReference.ConnectingDeviceInfo value is null", "");
				}
			}

			if (this.bdrVideoQualityList.Visibility == Visibility.Collapsed)
			{
				this.bdrVideoQualityList.Opacity = 0.0;
				this.bdrVideoQualityList.Visibility = Visibility.Visible;

				if (this._isVideoQualityFirstShow)
				{
					this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
					{
						this.AdjustVideoQualityListPosition();
						this.bdrVideoQualityList.Opacity = 1.0;
					});

					this._isVideoQualityFirstShow = false;
				}
				else
				{
					this.Dispatcher.BeginInvoke((Action)delegate()
					{
						this.AdjustVideoQualityListPosition();
						this.bdrVideoQualityList.Opacity = 1.0;
					});
				}
			}
			else
				this.bdrVideoQualityList.Visibility = Visibility.Collapsed;

			GoogleAnalytics.GoogleAnalyticSend("ScreencastWindow", "Video Quality");
		}

		private void AdjustVideoQualityListPosition()
		{
			Point relativePoint;
			double padding;

			switch (this._rotatedAngle)
			{
				case 0:
					{
						this.bdrVideoQualityList.VerticalAlignment = VerticalAlignment.Bottom;
						this.bdrVideoQualityList.HorizontalAlignment = HorizontalAlignment.Left;

						relativePoint = this.btnScreenRecord.TransformToAncestor(this.grdScreenButtonField).Transform(new Point(0, 0));

						padding = relativePoint.X + (this.btnScreenRecord.ActualWidth - this.bdrVideoQualityList.ActualWidth) / 2;
						if ((padding + this.bdrVideoQualityList.ActualWidth) > this.grdScreen.ActualWidth)
							padding = this.grdScreen.ActualWidth - this.bdrVideoQualityList.ActualWidth;

						this.bdrVideoQualityList.Margin = new Thickness(padding, 0, 0, this.grdScreenButtonField.ActualHeight);
					}
					break;

				case 90:
					{
						this.bdrVideoQualityList.VerticalAlignment = VerticalAlignment.Top;
						this.bdrVideoQualityList.HorizontalAlignment = HorizontalAlignment.Right;

						relativePoint = this.btnScreenRecord90.TransformToAncestor(this.grdScreenButtonField90).Transform(new Point(0, 0));

						padding = relativePoint.Y + this.btnScreenRecord90.ActualHeight - this.bdrVideoQualityList.ActualHeight;
						if ((padding + this.bdrVideoQualityList.ActualHeight) > this.grdScreen.ActualHeight)
							padding = this.grdScreen.ActualHeight - this.bdrVideoQualityList.ActualHeight;
						else if (padding < 0)
							padding = 0;

						this.bdrVideoQualityList.Margin = new Thickness(0, padding, this.grdScreenButtonField90.ActualWidth, 0);
					}
					break;

				case 270:
					{
						this.bdrVideoQualityList.VerticalAlignment = VerticalAlignment.Top;
						this.bdrVideoQualityList.HorizontalAlignment = HorizontalAlignment.Left;

						relativePoint = this.btnScreenRecord270.TransformToAncestor(this.grdScreenButtonField270).Transform(new Point(0, 0));

						padding = relativePoint.Y + this.btnScreenRecord270.ActualHeight - this.bdrVideoQualityList.ActualHeight;
						if ((padding + this.bdrVideoQualityList.ActualHeight) > this.grdScreen.ActualHeight)
							padding = this.grdScreen.ActualHeight - this.bdrVideoQualityList.ActualHeight;
						else if (padding < 0)
							padding = 0;

						this.bdrVideoQualityList.Margin = new Thickness(this.grdScreenButtonField270.ActualWidth, padding, 0, 0);
					}
					break;

				default:
					break;
			}
		}

		private void btnScreenResize_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				this.ResetFunctionFieldStatus();

				this._videoLongSide = this._videoSize.VideoHeight;
				this._videoShortSide = this._videoSize.VideoWidth;
				this.grdScreen.Height = this._isPortraitMode ? this._videoLongSide : this._videoShortSide;
				this.grdScreen.Width = this._isPortraitMode ? this._videoShortSide : this._videoLongSide;

				this.SetScreenCastingLayout();

				try
				{
					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath + @"\InitialScreenSize", "VideoShortSide", this._videoShortSide, RegistryView.Registry64);
					RegistryUtility.SetRegistry(RegistryHive.CurrentUser, Configs.RegUIGeneralPath + @"\InitialScreenSize", "VideoLongSide", this._videoLongSide, RegistryView.Registry64);
				}
				catch (Exception ex)
				{ }

				this.Dispatcher.BeginInvoke(DispatcherPriority.Background, (Action)delegate()
				{
					this.grdDeviceListAdjust();
				});
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		#region Handle Virtual Keyboard Layout Event

		private void btnNavigationBack_Click(object sender, RoutedEventArgs e)
		{
			if (null != FlowControlManager.Instance.InputController && FlowControlManager.Instance.InputController.IsInitializationSuccess)
				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.NavigationBackClicked));
		}

		private void btnNavigationHome_Click(object sender, RoutedEventArgs e)
		{
			if (null != FlowControlManager.Instance.InputController && FlowControlManager.Instance.InputController.IsInitializationSuccess)
				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.NavigationHomeClicked));
		}

		private void btnNavigationRecent_Click(object sender, RoutedEventArgs e)
		{
			if (null != FlowControlManager.Instance.InputController && FlowControlManager.Instance.InputController.IsInitializationSuccess)
				this.SendWindowEventAsync(this, new WindowEventArgs(WindowEventType.NavigationRecentClicked));
		}

		#endregion Handle Virtual Keyboard Layout Event

		#endregion Screenwindow Button Click Event Handle

		#region MessageBox

		public MessageBoxResult ShowMessageBox(string message, string caption = null, MessageBoxButton messageBoxButton = MessageBoxButton.OK, MessageBoxImage messageBoxImage = MessageBoxImage.Information)
		{
			MessageBoxResult ret = MessageBoxResult.None;
			if (string.IsNullOrWhiteSpace(message))
				return ret;

			this.Dispatcher.Invoke(delegate()
			{
				if (string.IsNullOrWhiteSpace(caption))
					caption = this.Title;

				ret = MessageBox.Show(this, message, caption, messageBoxButton, messageBoxImage);
			});
			return ret;
		}

		public MessageBoxResult ShowErrorMessageBox(string message, string caption = null, MessageBoxButton messageBoxButton = MessageBoxButton.OK)
		{
			return this.ShowMessageBox(message, caption, messageBoxButton, MessageBoxImage.Error);
		}

		public MessageBoxResult ShowWarningMessageBox(string message, string caption = null, MessageBoxButton messageBoxButton = MessageBoxButton.OK)
		{
			return this.ShowMessageBox(message, caption, messageBoxButton, MessageBoxImage.Warning);
		}

		public MessageBoxResult ShowQuestionMessageBox(string message, string caption = null, MessageBoxButton messageBoxButton = MessageBoxButton.YesNoCancel)
		{
			return this.ShowMessageBox(message, caption, messageBoxButton, MessageBoxImage.Question);
		}

		public MessageBoxResult ShowConfirmMessageBox(string message, string caption = null, MessageBoxButton messageBoxButton = MessageBoxButton.OKCancel)
		{
			return this.ShowMessageBox(message, caption, messageBoxButton, MessageBoxImage.Question);
		}

		public MessageBoxResult ShowInformationMessageBox(string message, string caption = null, MessageBoxButton messageBoxButton = MessageBoxButton.OK)
		{
			return this.ShowMessageBox(message, caption, messageBoxButton, MessageBoxImage.Information);
		}

		#endregion MessageBox

		/// <summary>
		/// Readings the resourece wording.
		/// </summary>
		/// <param name="resourekey">The resourekey.</param>
		/// <returns></returns>
		private string ReadingResoureceWording(object resourekey)
		{
			try
			{
				return Application.Current.TryFindResource(resourekey) as String;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "ReadingResoureceWording exception : " + ex.ToString(), "");
				return string.Empty;
			}
		}

		#region For Touch Handle

		// Only can support one finger touch
		private int _Touching_ID = -1;		// -1: No touch; Other: User is touching the screen

		#region Workaround: These flags are used for separating Touch events from mouse events

		/// <summary>
		/// Avoids PC send touch down and mouse down event to Phone at the same time
		/// </summary>
		private bool _TouchDownOnce = false;

		/// <summary>
		/// Avoids PC send touch move and mouse move event to Phone at the same time
		/// </summary>
		private bool _TouchMoveOnce = false;

		/// <summary>
		/// Avoids PC send touch up and mouse up event to Phone at the same time
		/// </summary>
		private bool _TouchUpOnce = false;

		/// <summary>
		/// Avoids PC send touch leave and mouse leave event to Phone at the same time
		/// </summary>
		private bool _TouchLeaveOnce = false;

		#endregion Workaround: These flags are used for separating Touch events from mouse events

		private void EnableTouchHandle()
		{
			// Check if the PC supports touch control
			// - Not support: Do nothing
			// - Support: Hook touch events

			this.TouchDown += new EventHandler<TouchEventArgs>(MainWindow_TouchDown);
			this.TouchUp += new EventHandler<TouchEventArgs>(MainWindow_TouchUp);
			this.TouchMove += new EventHandler<TouchEventArgs>(MainWindow_TouchMove);
			this.TouchLeave += new EventHandler<TouchEventArgs>(MainWindow_TouchLeave);
		}

		#region VNC Touch Event Handle

		private void MainWindow_TouchDown(object sender, TouchEventArgs e)
		{
			if (null == FlowControlManager.Instance.InputController)
				return;

			if (!FlowControlManager.Instance.InputController.IsInitializationSuccess)
				return;

			if (this._Touching_ID < 0)
				this._Touching_ID = e.TouchDevice.Id;		// To record the first touch ID (Avoid the multi-touch)
			else
				return;

			this._TouchDownOnce = true;
			TouchPoint tPoint = e.GetTouchPoint(this);
			FlowControlManager.Instance.InputController.VNCClient.SendTouchInpugControl(VNC.TouchEventType.Download, (Int32)tPoint.Position.X, (Int32)tPoint.Position.Y);
		}

		private void MainWindow_TouchUp(object sender, TouchEventArgs e)
		{
			if (null == FlowControlManager.Instance.InputController)
				return;

			if (!FlowControlManager.Instance.InputController.IsInitializationSuccess)
				return;

			if (this._Touching_ID != e.TouchDevice.Id)
				return;		// Not match touch ID, so to exit this function handler

			this._Touching_ID = -1;			// Match the touch ID, so to disable touch handler (i.e. clear touch ID)
			this._TouchUpOnce = true;
			TouchPoint tPoint = e.GetTouchPoint(this);
			FlowControlManager.Instance.InputController.VNCClient.SendTouchInpugControl(VNC.TouchEventType.Up, (Int32)tPoint.Position.X, (Int32)tPoint.Position.Y);
		}

		private void MainWindow_TouchMove(object sender, TouchEventArgs e)
		{
			if (null == FlowControlManager.Instance.InputController)
				return;

			if (!FlowControlManager.Instance.InputController.IsInitializationSuccess)
				return;

			if (this._Touching_ID != e.TouchDevice.Id)
				return;		// Not match touch ID, so to exit this function handler

			this._TouchMoveOnce = true;
			TouchPoint tPoint = e.GetTouchPoint(this);
			FlowControlManager.Instance.InputController.VNCClient.SendTouchInpugControl(VNC.TouchEventType.Move, (Int32)tPoint.Position.X, (Int32)tPoint.Position.Y);
		}

		private void MainWindow_TouchLeave(object sender, TouchEventArgs e)
		{
			if (null == FlowControlManager.Instance.InputController)
				return;

			if (!FlowControlManager.Instance.InputController.IsInitializationSuccess)
				return;

			if (this._Touching_ID != -1)
			{
				// It's possible that the Touch Up event didn't be fired (Maybe user moves touch outisde the phone screen)
				// - Assume it is a Touch Up event
				this._Touching_ID = -1;

				this._TouchLeaveOnce = true;
				TouchPoint tPoint = e.GetTouchPoint(this);
				FlowControlManager.Instance.InputController.VNCClient.SendTouchInpugControl(VNC.TouchEventType.Up, (Int32)tPoint.Position.X, (Int32)tPoint.Position.Y);
			}
		}

		#endregion VNC Touch Event Handle

		/// <summary>
		/// Check it the windows message is created by Touch Event or not
		/// </summary>
		/// <param name="iWinMsg">Windows message</param>
		/// <returns>True: Touch created, so don't handle this message</returns>
		private bool IsTouchingInput(int iWinMsg)
		{
			switch (iWinMsg)
			{
				case Win32Api.WM_LBUTTONDOWN:
					if (this._TouchDownOnce)
					{
						this._TouchDownOnce = false;
						return true;
					}
					break;

				case Win32Api.WM_LBUTTONUP:
					if (this._TouchUpOnce)
					{
						this._TouchUpOnce = false;
						return true;
					}
					break;

				case Win32Api.WM_MOUSEMOVE:
					if (this._TouchMoveOnce)
					{
						this._TouchMoveOnce = false;
						return true;
					}
					break;

				case Win32Api.WM_MOUSELEAVE:
					if (this._TouchLeaveOnce)
					{
						this._TouchLeaveOnce = false;
						return true;
					}
					break;

				default:
					break;
			}
			return false;
		}

		#endregion For Touch Handle

		private void bdrScreenMask_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			this.ResetFunctionFieldStatus();
			this.ResetSelectDeviceFieldStatus();
		}

		#region Workarounds

		/// <summary>
		/// Workaround for disabling monitor UI will hang.
		/// </summary>
		private void WorkaroundForDisablingMonitorUIWillHang()
		{
			this.WindowState = WindowState.Maximized;
		}

		#endregion Workarounds

		private void NetworkUnstableHandler()
		{
			//this.Dispatcher.Invoke(delegate()
			//{
			//	this.brdNetworkStatus.Visibility = Visibility.Visible;
			//});
		}

		private void grdScreen_MouseRightButtonUp(object sender, MouseButtonEventArgs e)
		{
			if (!this._isMaximized)
			{
				// on 1:1
				RCMenuWindowsSize.Visibility = Visibility.Visible;
				RCMenuWindowsSize.Header = ReadingResoureceWording("txtScreencastToolTipFullScreen");
			}
			else
			{
				//on max
				//RCMenuWindowsSize.Header = "Minimize";
				RCMenuWindowsSize.Visibility = Visibility.Collapsed;
			}
		}

		private void brdNetworkStatusDoubleAnimation_Completed(object sender, EventArgs e)
		{
			this.brdNetworkStatus.Visibility = Visibility.Hidden;
		}

		private void dsElement_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			if (this.dsElement.Visibility != Visibility.Visible)
				this.brdNetworkStatus.Visibility = Visibility.Hidden;
		}

		private void brdNetworkStatus_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			try
			{
				if (this.brdNetworkStatus.Visibility == Visibility.Visible)
				{
					Storyboard storyboard = this.TryFindResource("StoryboardNetworkStatusTimeOut") as Storyboard;
					storyboard.Begin();
				}
			}
			catch (Exception ex)
			{ }
		}
	}

	public static class HookManager
	{
		public static void SubscribeToWindowEvents()
		{
			if (windowEventHook == IntPtr.Zero)
			{
				winEventProc = new WinEventProc(WindowEventCallback);
				windowEventHook = SetWinEventHook(
					(int)ObjectEventContants.EVENT_OBJECT_CREATE,				// eventMin
					(int)ObjectEventContants.EVENT_OBJECT_ACCELERATORCHANGE,	// eventMax
					IntPtr.Zero,             // hmodWinEventProc
					winEventProc,			 // lpfnWinEventProc
					0,                       // idProcess
					0,                       // idThread
					WINEVENT_OUTOFCONTEXT | WINEVENT_SKIPOWNPROCESS);

				if (windowEventHook == IntPtr.Zero)
				{
					throw new Win32Exception(Marshal.GetLastWin32Error());
				}
			}
		}

		private static void WindowEventCallback(IntPtr hWinEventHook, uint eventType, IntPtr hwnd, int idObject, int idChild, uint dwEventThread, uint dwmsEventTime)
		{
			//LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== WindowEventCallback === " + (ObjectEventContants)eventType, "");
			//StringBuilder _name = new StringBuilder(1024);
			//if (GetClassName(hwnd, _name, _name.Capacity) != 0)
			//{
			//    LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== WindowEventCallback === ClassName:" + _name.ToString(), "");

			//}

			if (Configs.ApplicationReference.AppScreenWindow.Visibility == Visibility.Visible
				&& Configs.ApplicationReference.AppScreenWindow.IsActive)
			{
				switch (eventType)
				{
					case (uint)ObjectEventContants.EVENT_OBJECT_STATECHANGE:
						{
							StringBuilder _apiResult = new StringBuilder(1024);

							if (GetClassName(hwnd, _apiResult, _apiResult.Capacity) != 0)
							{
								if (string.Compare(_apiResult.ToString(), "IPTip_Main_Window") == 0)
								{
									UInt32 VirtualKeyboardWindowStyle = (UInt32)Win32Api.NativeMethod.GetWindowLong(hwnd, GWL_STYLE);

									if ((VirtualKeyboardWindowStyle & WS_DISABLED) == 0
										&& (VirtualKeyboardWindowStyle & WS_VISIBLE) != 0
										&& Configs.ApplicationReference.AppScreenWindow.CheckTextInputBoxShow() == false
										&& Configs.ApplicationReference.AppScreenWindow.Visibility == System.Windows.Visibility.Visible)
									{
										Configs.ApplicationReference.AppScreenWindow.IsShowTextInputBox(true);
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== WindowEventCallback === EVENT_OBJECT_STATECHANGE, " + _apiResult.ToString(), "");
									}
									else if ((VirtualKeyboardWindowStyle & WS_DISABLED) != 0
										&& Configs.ApplicationReference.AppScreenWindow.CheckTextInputBoxShow() == true)
									{
										Configs.ApplicationReference.AppScreenWindow.IsShowTextInputBox(false, true);
										LogtoFile.WriteLog("", Configs.DebugLevelValue, "=== WindowEventCallback === EVENT_OBJECT_STATECHANGE, " + _apiResult.ToString(), "");
									}
								}
							}
						}
						break;

					default:
						break;
				}
			}
		}

		private static IntPtr windowEventHook;
		private static WinEventProc winEventProc;

		private delegate void WinEventProc(IntPtr hWinEventHook, uint eventType, IntPtr hwnd, int idObject, int idChild, uint dwEventThread, uint dwmsEventTime);

		[DllImport("user32.dll", SetLastError = true)]
		private static extern IntPtr SetWinEventHook(int eventMin, int eventMax, IntPtr hmodWinEventProc, WinEventProc lpfnWinEventProc, int idProcess, int idThread, int dwflags);

		[DllImport("user32.dll", SetLastError = true)]
		private static extern int UnhookWinEvent(IntPtr hWinEventHook);

		[DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
		private static extern int GetClassName(IntPtr hWnd, StringBuilder lpClassName, int nMaxCount);

		private const int WINEVENT_INCONTEXT = 4;
		private const int WINEVENT_OUTOFCONTEXT = 0;
		private const int WINEVENT_SKIPOWNPROCESS = 2;
		private const int WINEVENT_SKIPOWNTHREAD = 1;

		private const int GWL_STYLE = -16;
		private const UInt32 WS_VISIBLE = 0x10000000;
		private const UInt32 WS_DISABLED = 0x8000000;

		public enum ObjectEventContants : uint
		{
			EVENT_OBJECT_CREATE = 0x8000,
			EVENT_OBJECT_DESTROY = 0x8001,
			EVENT_OBJECT_SHOW = 0x8002,
			EVENT_OBJECT_HIDE = 0x8003,
			EVENT_OBJECT_REORDER = 0x8004,
			EVENT_OBJECT_FOCUS = 0x8005,
			EVENT_OBJECT_SELECTION = 0x8006,
			EVENT_OBJECT_SELECTIONADD = 0x8007,
			EVENT_OBJECT_SELECTIONREMOVE = 0x8008,
			EVENT_OBJECT_SELECTIONWITHIN = 0x8009,
			EVENT_OBJECT_STATECHANGE = 0x800A,
			EVENT_OBJECT_LOCATIONCHANGE = 0x800B,
			EVENT_OBJECT_NAMECHANGE = 0x800C,
			EVENT_OBJECT_DESCRIPTIONCHANGE = 0x800D,
			EVENT_OBJECT_VALUECHANGE = 0x800E,
			EVENT_OBJECT_PARENTCHANGE = 0x800F,
			EVENT_OBJECT_HELPCHANGE = 0x8010,
			EVENT_OBJECT_DEFACTIONCHANGE = 0x8011,
			EVENT_OBJECT_ACCELERATORCHANGE = 0x8012
		}
	}
}