﻿//using InTheHand.Net;
//using InTheHand.Net.Bluetooth;
using Microsoft.Win32;
using Newtonsoft.Json;
using System;
using System.Collections.Concurrent;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Text.RegularExpressions;

namespace AirFusion.Common
{
	/// <summary>
	/// Collection of common utilities.
	/// </summary>
	public static partial class CommonUtils
	{
		#region Bluetooth

		// 2015-05-20 Seoker: remove Bluetooth related functions since it's been defeatured
		/*
		/// <summary>
		/// Gets first Bluetooth address of current device.
		/// </summary>
		/// <returns></returns>
		[DebuggerStepThrough]
		public static BluetoothAddress GetBluetoothAddress()
		{
			return BluetoothRadio.PrimaryRadio != null ? BluetoothRadio.PrimaryRadio.LocalAddress : null;
		}

		/// <summary>
		/// Determines whether there's any Bluetooth chip installed on this machine and is enabled.
		/// </summary>
		/// <returns></returns>
		[DebuggerStepThrough]
		public static bool IsBluetoothSupported()
		{
			try
			{
				StaticLogger.TrackEnter();

				//return BluetoothRadio.IsSupported; // 2015-05-12 Seoker: this function might get stuck. keep watching
				return true;
			}
			finally
			{
				StaticLogger.TrackLeave();
			}
		}

		/// <summary>
		/// Determines whether there's any Bluetooth radio enabled.
		/// </summary>
		/// <returns></returns>
		[DebuggerStepThrough]
		public static bool IsBluetoothEnabled()
		{
			//return IsBluetoothSupported() && BluetoothRadio.PrimaryRadio != null; // 2015-05-14 Seoker: this function might get stuck. keep watching
			return true;
		}
		*/

		#endregion Bluetooth

		#region JSON

		/// <summary>
		/// Serializes the string into Json format.
		/// </summary>
		/// <param name="data">The source object.</param>
		/// <param name="indented">Indicates whether to output indented result.</param>
		/// <returns></returns>
		public static string SerializeAsJson(object data, bool indented = true)
		{
			return JsonConvert.SerializeObject(data, indented ? Formatting.Indented : Formatting.None,
				new JsonSerializerSettings()
				{
					ReferenceLoopHandling = ReferenceLoopHandling.Error,
					NullValueHandling = NullValueHandling.Ignore,
					TypeNameHandling = TypeNameHandling.Auto
				});
		}

		/// <summary>
		/// Deserializes the string from Json format.
		/// </summary>
		/// <typeparam name="T">The target type of object.</typeparam>
		/// <param name="text">The json-format string.</param>
		/// <returns></returns>
		public static T DeserializeFromJson<T>(string text)
		{
			// suppose this function will throw out exception if the specified text is not a valid JSON format
			return JsonConvert.DeserializeObject<T>(text,
				new JsonSerializerSettings()
				{
					MissingMemberHandling = MissingMemberHandling.Ignore,
					NullValueHandling = NullValueHandling.Include,
					ObjectCreationHandling = ObjectCreationHandling.Auto,
					TypeNameHandling = TypeNameHandling.Auto,
					Error = delegate(object sender, Newtonsoft.Json.Serialization.ErrorEventArgs args)
					{
						StaticLogger.LogWarn("Failed parsing into '" + args.ErrorContext.Path + "', exception:\r\n" + CommonUtils.GetExceptionMessages(args.ErrorContext.Error));
						args.ErrorContext.Handled = true;
					}
				});
		}

		#endregion JSON

		#region String Function

		/// <summary>
		/// Encodes non-ascii characters into \uxxxx form.
		/// </summary>
		/// <param name="value">The non-encoded string.</param>
		/// <returns></returns>
		public static string EncodeNonAsciiCharacters(string value)
		{
			StringBuilder sb = new StringBuilder();
			foreach (char c in value)
			{
				if (c > 127)
				{
					// this character is too big for ASCII
					string encodedValue = "\\u" + ((int)c).ToString("x4");
					sb.Append(encodedValue);
				}
				else
				{
					sb.Append(c);
				}
			}
			return sb.ToString();
		}

		/// <summary>
		/// Decodes encoded non ascii characters.
		/// </summary>
		/// <param name="value">The encoded string.</param>
		/// <returns></returns>
		public static string DecodeEncodedNonAsciiCharacters(string value)
		{
			return Regex.Replace(
				value,
				@"\\u(?<Value>[a-zA-Z0-9]{4})",
				m =>
				{
					return ((char)int.Parse(m.Groups["Value"].Value, NumberStyles.HexNumber)).ToString();
				});
		}

		/// <summary>
		/// Converts path with environment variables.
		/// </summary>
		/// <param name="path">The path that contains environment variables.</param>
		/// <returns></returns>
		public static string ConvertPath(string path)
		{
			if (path == null)
			{
				return null;
			}

			path = Regex.Replace(path, "%LOCALAPPDATA%", Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%APPDATA%", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%PROGRAMDATA%", Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%PROGRAMFILES%", Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%PROGRAMFILESX86%", Environment.GetFolderPath(Environment.SpecialFolder.ProgramFilesX86), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONPROGRAMFILES%", Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONPROGRAMFILESX86%", Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFilesX86), RegexOptions.IgnoreCase | RegexOptions.Compiled);

			path = Regex.Replace(path, "%STARTMENU%", Environment.GetFolderPath(Environment.SpecialFolder.StartMenu), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONSTARTMENU%", Environment.GetFolderPath(Environment.SpecialFolder.CommonStartMenu), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%DESKTOPDIRECTORY%", Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%DESKTOP%", Environment.GetFolderPath(Environment.SpecialFolder.Desktop), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONDESKTOPDIRECTORY%", Environment.GetFolderPath(Environment.SpecialFolder.CommonDesktopDirectory), RegexOptions.IgnoreCase | RegexOptions.Compiled);

			path = Regex.Replace(path, "%MYDOCUMENTS%", Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%MYMUSIC%", Environment.GetFolderPath(Environment.SpecialFolder.MyMusic), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%MYPICTURES%", Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%MYVIDEOS%", Environment.GetFolderPath(Environment.SpecialFolder.MyVideos), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%TEMPLATES%", Environment.GetFolderPath(Environment.SpecialFolder.Templates), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONDOCUMENTS%", Environment.GetFolderPath(Environment.SpecialFolder.CommonDocuments), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONMUSIC%", Environment.GetFolderPath(Environment.SpecialFolder.CommonMusic), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONPICTURES%", Environment.GetFolderPath(Environment.SpecialFolder.CommonPictures), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONVIDEOS%", Environment.GetFolderPath(Environment.SpecialFolder.CommonVideos), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COMMONTEMPLATES%", Environment.GetFolderPath(Environment.SpecialFolder.CommonTemplates), RegexOptions.IgnoreCase | RegexOptions.Compiled);

			path = Regex.Replace(path, "%PERSONAL%", Environment.GetFolderPath(Environment.SpecialFolder.Personal), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%COOKIES%", Environment.GetFolderPath(Environment.SpecialFolder.Cookies), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%FAVORITE%", Environment.GetFolderPath(Environment.SpecialFolder.Favorites), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%FONT%", Environment.GetFolderPath(Environment.SpecialFolder.Fonts), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%HISTORY%", Environment.GetFolderPath(Environment.SpecialFolder.History), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%WINDIR%", Environment.GetFolderPath(Environment.SpecialFolder.Windows), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%WINDOWS%", Environment.GetFolderPath(Environment.SpecialFolder.Windows), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%STARTUP%", Environment.GetFolderPath(Environment.SpecialFolder.Startup), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%RECENT%", Environment.GetFolderPath(Environment.SpecialFolder.Recent), RegexOptions.IgnoreCase | RegexOptions.Compiled);
			path = Regex.Replace(path, "%SENDTO%", Environment.GetFolderPath(Environment.SpecialFolder.SendTo), RegexOptions.IgnoreCase | RegexOptions.Compiled);

			return path;
		}

		#endregion String Function

		#region Registry

		/// <summary>
		/// Opens the or create the specified registry.
		/// </summary>
		/// <param name="hive">The hive.</param>
		/// <param name="view">The view.</param>
		/// <param name="path">The path.</param>
		/// <param name="createIfNotFound">Indicates whether to create the registry key if such key doesn't exist.</param>
		/// <param name="writable">Indicates the registry is writable or not.</param>
		/// <exception cref="System.ArgumentException" />
		/// <exception cref="System.ArgumentNullException" />
		/// <exception cref="System.IO.IOException" />
		/// <exception cref="System.ObjectDisposedException" />
		/// <exception cref="System.Security.SecurityException" />
		/// <exception cref="System.UnauthorizedAccessException" />
		/// <returns></returns>
		public static RegistryKey OpenOrCreateRegistry(RegistryHive hive, RegistryView view, string path, bool createIfNotFound = true, bool writable = true)
		{
			return OpenOrCreateRegistry(RegistryKey.OpenBaseKey(hive, view), path, createIfNotFound, writable);
		}

		/// <summary>
		/// Opens the or create registry.
		/// </summary>
		/// <param name="root">The root.</param>
		/// <param name="path">The path.</param>
		/// <param name="createIfNotFound">Indicates whether to create the registry key if such key doesn't exist.</param>
		/// <param name="writable">Indicates the registry is writable or not.</param>
		/// <returns></returns>
		/// <exception cref="System.ArgumentException" />
		/// <exception cref="System.ArgumentNullException" />
		/// <exception cref="System.IO.IOException" />
		/// <exception cref="System.ObjectDisposedException" />
		/// <exception cref="System.Security.SecurityException" />
		/// <exception cref="System.UnauthorizedAccessException" />
		public static RegistryKey OpenOrCreateRegistry(RegistryKey root, string path, bool createIfNotFound = true, bool writable = true)
		{
			try
			{
				StaticLogger.TrackEnter();

				// log parameters
				StaticLogger.LogDebug("root: " + (root != null ? root.ToString() : "N/A"));
				StaticLogger.LogDebug("path: " + (path ?? "N/A"));
				StaticLogger.LogDebug("createIfNotFound: " + createIfNotFound);
				StaticLogger.LogDebug("writable: " + writable);

				// check parameters
				if (root == null)
				{
					throw new ArgumentNullException("root");
				}
				if (string.IsNullOrEmpty(path))
				{
					throw new ArgumentException("'path' is not specified.");
				}

				path = path.Replace('/', '\\');
				path = path.TrimStart('\\', '/');

				RegistryKey key = null;
				key = root.OpenSubKey(path, writable);
				if (key == null)
				{
					key = root.CreateSubKey(path, writable ? RegistryKeyPermissionCheck.ReadWriteSubTree : RegistryKeyPermissionCheck.ReadSubTree);
				}

				if (key == null)
				{
					throw new Exception("Failed opening or creating registry key '" + root.ToString() + "\\" + path + "'.");
				}

				return key;
			}
			finally
			{
				StaticLogger.TrackLeave();
			}
		}

		#endregion Registry

		#region Service

		/// <summary>
		/// Makes the sure the specified service is running.
		/// </summary>
		/// <param name="serviceName">The name of the service.</param>
		/// <param name="timeout">The timeout, in milliseconds.</param>
		/// <exception cref="System.ArgumentException" />
		/// <exception cref="System.Exception" />
		/// <exception cref="System.NotSupportedException" />
		public static void MakeSureServiceIsRunning(string serviceName, int timeout = 5000)
		{
			try
			{
				StaticLogger.TrackEnter();

				// check parameters
				if (string.IsNullOrWhiteSpace(serviceName))
				{
					throw new ArgumentException("Please specified a valid service name.", "serviceName");
				}

				lock (CommonUtils.GetLocker(new Tuple<string, string>("MakeSureServiceIsRunning()", serviceName)))
				{
					var controllers = ServiceController.GetServices().Where(sc => sc.ServiceName.Equals(serviceName));
					if (controllers.Count() == 0)
					{
						throw new NotSupportedException("The service '" + serviceName + "' isn't installed.");
					}

					try
					{
						using (var controller = controllers.First())
						{
							switch (controller.Status)
							{
								case ServiceControllerStatus.Running:
									return;

								case ServiceControllerStatus.StartPending:
									break;

								case ServiceControllerStatus.Paused:
									controller.Continue();
									break;

								case ServiceControllerStatus.PausePending:
									controller.WaitForStatus(ServiceControllerStatus.Paused, new TimeSpan(timeout * 10000));
									controller.Continue();
									break;

								case ServiceControllerStatus.Stopped:
									controller.Start();
									break;

								case ServiceControllerStatus.StopPending:
									controller.WaitForStatus(ServiceControllerStatus.Stopped, new TimeSpan(timeout * 10000));
									controller.Start();
									break;
							}
							controller.WaitForStatus(ServiceControllerStatus.Running, new TimeSpan(timeout * 10000));
						}
					}
					catch (System.TimeoutException)
					{
						throw;
					}
					catch (Exception ex)
					{
						throw new Exception("Failed manipulating the service '" + serviceName + "'", ex);
					}
				}
			}
			finally
			{
				StaticLogger.TrackLeave();
			}
		}

		#endregion Service

		#region Debugger

		/// <summary>
		/// Gets nested exception messages.
		/// </summary>
		/// <param name="exception">The root exception.</param>
		/// <returns></returns>
		[DebuggerStepThrough]
		public static string GetExceptionMessages(Exception exception)
		{
			try
			{
				if (exception == null)
				{
					return "N/A";
				}

				Exception ex = exception;
				StringBuilder builder = new StringBuilder();

				if (ex is AggregateException)
				{
					AggregateException aggEx = ex as AggregateException;
					if (aggEx.InnerExceptions != null)
					{
						foreach (var e in aggEx.InnerExceptions)
						{
							builder.AppendLine(string.Join("\r\n==\r\n", aggEx.InnerExceptions.Select(ee => GetExceptionMessages(ee))));
						}
						return builder.ToString();
					}
					else if (aggEx.InnerException != null)
					{
						ex = aggEx.InnerException;
					}
				}

				while (ex != null)
				{
					builder.AppendLine(ex.Message + " (" + (ex.GetType().FullName ?? "N/A") + ", " + (ex.Source ?? "N/A") + ", " + (ex.TargetSite != null ? (ex.TargetSite.Name ?? "N/A") : "N/A") + ")");
					ex = ex.InnerException;
				}

				builder.AppendLine("Stack Trace:");
				builder.AppendLine(exception.StackTrace);

				return builder.ToString();
			}
			catch
			{
				return "N/A";
			}
		}

		/// <summary>
		/// Gets the full name of the specified type.
		/// </summary>
		/// <param name="type">The type.</param>
		/// <returns></returns>
		public static string GetFullName(this Type type)
		{
			// TODO:
			// this function has some issue that the returned full name is not precise

			try
			{
				string fullName = "";
				if (type.IsGenericType)
				{
					Type[] genericParameters = type.GetGenericArguments();
					fullName = GetSpecifiedName(type.Name, ref genericParameters);

					if (type.DeclaringType != null)
					{
						fullName = (type.DeclaringType.ContainsGenericParameters ? GetSpecifiedName(type.DeclaringType.Name, ref genericParameters) : type.DeclaringType.Name) + "." + fullName;
					}
				}
				else
				{
					fullName = type.Name;
					if (type.DeclaringType != null)
					{
						fullName = type.DeclaringType.Name + "." + fullName;
					}
				}

				if (string.IsNullOrEmpty(type.Namespace) == false)
				{
					fullName = type.Namespace + "." + fullName;
				}

				return fullName;
			}
			catch
			{
				return type.FullName;
			}
		}

		private static string GetSpecifiedName(string name, ref Type[] additionalGenericParameters)
		{
			int used = 0;
			Match match = Regex.Match(name, "^(.*)`([0-9]+)$", RegexOptions.Compiled);
			if (match.Success)
			{
				used = Convert.ToInt32(match.Groups[2].Value);
				string fullName = match.Groups[1].Value + "<" + string.Join(",", additionalGenericParameters.Skip(additionalGenericParameters.Length - used).Take(used).Select(item => item.GetFullName())) + ">";
				additionalGenericParameters = additionalGenericParameters.Take(additionalGenericParameters.Length - used).ToArray();
				return fullName;
			}
			else
			{
				return name;
			}
		}

		#endregion Debugger

		#region Process

		/// <summary>
		/// Runs a execution with specified arguments.
		/// </summary>
		/// <param name="exitCode">The exit code.</param>
		/// <param name="executeFile">Execution name or command name.</param>
		/// <param name="arguments">Arguments passed to the execution or command.</param>
		/// <param name="createNoWindow">True if no window is showed.</param>
		/// <param name="useShellExecute">True if to execute in shell.</param>
		/// <param name="waitForExit">True if to wait for exit.</param>
		/// <param name="workingDirectory">The working directory.</param>
		/// <returns>Exit code of the execution</returns>
		public static void RunProcess(out int exitCode, string executeFile, string arguments = null, bool createNoWindow = true, bool useShellExecute = false, bool waitForExit = true, string workingDirectory = null)
		{
			// Initialize output
			exitCode = -1;

			// Check parameters
			if (string.IsNullOrWhiteSpace(executeFile))
			{
				throw new ArgumentNullException("executeFile", "Please specify 'executeFile'.");
			}

			// Execute
			using (Process runProcess = new Process())
			{
				runProcess.StartInfo.CreateNoWindow = createNoWindow;
				runProcess.StartInfo.UseShellExecute = useShellExecute;
				if (useShellExecute && createNoWindow)
				{
					runProcess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
				}

				runProcess.StartInfo.FileName = executeFile;
				runProcess.StartInfo.Arguments = arguments ?? String.Empty;
				if (!string.IsNullOrEmpty(workingDirectory) && Directory.Exists(workingDirectory))
				{
					runProcess.StartInfo.WorkingDirectory = workingDirectory;
				}

				runProcess.Start();
				if (waitForExit)
				{
					runProcess.WaitForExit();
					exitCode = runProcess.ExitCode;
				}
			}
		}

		#endregion Process

		#region Others

		/// <summary>
		/// Invokes the event.
		/// </summary>
		/// <typeparam name="T">The type of the event argument.</typeparam>
		/// <param name="handlers">The handlers.</param>
		/// <param name="sender">The sender.</param>
		/// <param name="args">The arguments.</param>
		[DebuggerStepThrough]
		public static void InvokeEvent<T>(EventHandler<T> handlers, object sender, T args)
			where T : EventArgs
		{
			if (handlers == null)
			{
				return;
			}

			foreach (EventHandler<T> handler in handlers.GetInvocationList())
			{
				if (handler == null)
				{
					continue;
				}

				try
				{
					handler.Invoke(sender, args);
				}
				catch (Exception ex)
				{
					StaticLogger.LogWarn("Unhandled exception of event handler, exception:\r\n" + CommonUtils.GetExceptionMessages(ex));
				}
			}
		}

		/// <summary>
		/// Begins to invoke the event.
		/// </summary>
		/// <typeparam name="T">The type of the event argument.</typeparam>
		/// <param name="handlers">The handlers.</param>
		/// <param name="sender">The sender.</param>
		/// <param name="args">The arguments.</param>
		[DebuggerStepThrough]
		public static void BeginInvokeEvent<T>(EventHandler<T> handlers, object sender, T args)
			where T : EventArgs
		{
			if (handlers == null)
			{
				return;
			}

			foreach (EventHandler<T> handler in handlers.GetInvocationList())
			{
				if (handler == null)
				{
					continue;
				}

				handler.BeginInvoke(sender, args, delegate(IAsyncResult result)
				{
					try
					{
						if (result.IsCompleted)
						{
							handler.EndInvoke(result);
						}
					}
					catch (Exception ex)
					{
						StaticLogger.LogWarn("Unhandled exception of event handler, exception:\r\n" + CommonUtils.GetExceptionMessages(ex));
					}
				}, null);
			}
		}

		private static ConcurrentDictionary<object, object> _lockStore = new ConcurrentDictionary<object, object>();

		/// <summary>
		/// Gets the locker of a specified object.
		/// </summary>
		/// <param name="obj">The object.</param>
		/// <returns></returns>
		public static object GetLocker(object obj)
		{
			if (obj == null)
			{
				return new object();
			}

			Debug.Assert(_lockStore != null);
			return _lockStore.GetOrAdd(obj, new object());
		}

		#endregion Others
	}
}
