﻿namespace AirFusion.Common.Wmi.Cim
{
	/// <summary>
	/// The CIM_PhysicalPackage class represents physical elements that contain or host other components. Examples are a rack enclosure or an adapter card.
	/// </summary>
	[ParameterValue("CIM_PhysicalPackage")]
	public class WmiPhysicalPackage : WmiPhysicalElement
	{
		/// <summary>
		/// Gets the depth of the physical package, in inches.
		/// </summary>
		[ParameterValue("Depth")]
		public float? Depth { get; internal set; }

		/// <summary>
		/// Gets the height of the physical package, in inches.
		/// </summary>
		[ParameterValue("Height")]
		public float? Height { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the the package can be hot-swapped.
		/// A physical package can be hot-swapped if the element can be replaced by a physically different (but equivalent) one while the containing package is turned on.
		/// For example, a fan component may be designed to be hot-swapped. All components that can be hot-swapped are inherently removable and replaceable.
		/// </summary>
		[ParameterValue("HotSwappable")]
		public bool? HotSwappable { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the package is designed to be taken in and out of the physical container in which it is normally found, without impairing the function of the overall packaging.
		/// A package is considered removable even if the power must be off to perform the removal.
		/// If the power can be on and the package removed, then the element is removable and can be hot-swapped.
		/// For example, an upgradeable processor chip is removable.
		/// </summary>
		[ParameterValue("Removable")]
		public bool? Removable { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if it is possible to replace the element with a physically different one.
		/// For example, some computer systems allow the main processor chip to be upgraded to one of a higher clock rating.
		/// In this case, the processor is said to be replaceable. All removable components are inherently replaceable.
		/// </summary>
		[ParameterValue("Replaceable")]
		public bool? Replaceable { get; internal set; }

		/// <summary>
		/// Gets the weight of the physical package, in pounds.
		/// </summary>
		[ParameterValue("Weight")]
		public float? Weight { get; internal set; }

		/// <summary>
		/// Gets the width of the physical package, in inches.
		/// </summary>
		[ParameterValue("Width")]
		public float? Width { get; internal set; }

		/// <summary>
		/// Initializes the object.
		/// </summary>
		public WmiPhysicalPackage()
		{
			this.Depth = null;
			this.Height = null;
			this.HotSwappable = null;
			this.Removable = null;
			this.Replaceable = null;
			this.Weight = null;
			this.Width = null;
		}
	}
}