﻿using System;

namespace AirFusion.Common.Wmi.Cim
{
	/// <summary>
	/// The CIM_UnitaryComputerSystem class represents a desktop, mobile, network computer, server, or other type of single-node computer system.
	/// </summary>
	[ParameterValue("CIM_UnitaryComputerSystem")]
	public class WmiUnitaryComputerSystem : WmiSystem
	{
		/// <summary>
		/// Gets the data needed to find either the initial load device (its key) or the boot service to request the operating system to start.
		/// In addition, the load parameters (that is, a path name and parameters) can also be specified.
		/// </summary>
		[ParameterValue("InitialLoadInfo")]
		public string InitialLoadInfo { get; internal set; }

		/// <summary>
		/// Gets the data that identifies either the initial load device (its key) or the boot service that requested the last operating system load.
		/// In addition, the load parameters (that is, a path name and parameters) can also be specified.
		/// </summary>
		[ParameterValue("LastLoadInfo")]
		public string LastLoadInfo { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the device can be power managed, that is, put into a power-save state.
		/// If FALSE, the integer value 1 ("Not Supported") should be the only entry in the PowerManagementCapabilities array.
		/// </summary>
		[ParameterValue("PowerManagementSupported")]
		public bool? PowerManagementSupported { get; internal set; }

		/// <summary>
		/// Gets the current power state of the computer system and its associated operating system.
		/// </summary>
		[ParameterValue("PowerState")]
		public UInt16? PowerState { get; internal set; }

		/// <summary>
		/// Gets the status that indicates if the unitary computer system can be reset with hardware (for example, with the power and reset buttons). If disabled, hardware reset is not allowed.
		/// </summary>
		[ParameterValue("ResetCapability")]
		public UInt16? ResetCapability { get; internal set; }

		/// <summary>
		/// Initializes the object.
		/// </summary>
		public WmiUnitaryComputerSystem()
		{
			this.InitialLoadInfo = null;
			this.LastLoadInfo = null;
			this.PowerManagementSupported = null;
			this.PowerState = null;
			this.ResetCapability = null;
		}
	}
}