﻿using InTheHand.Net;
using Newtonsoft.Json;
using System;

namespace AirFusion.Common.JsonConverters
{
	internal class BluetoothAddressConverter : JsonConverter
	{
		public override bool CanConvert(Type objectType)
		{
			return typeof(BluetoothAddress) == objectType;
		}

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
		{
			if (CanConvert(objectType) == false)
			{
				throw new ArgumentException("Not supported object type: " + objectType.GetFullName());
			}

			BluetoothAddress ret = null;
			string stringValue = reader.Value != null ? reader.Value.ToString() : null;

			switch (reader.TokenType)
			{
				case JsonToken.Null:
					return ret;
				case JsonToken.String:
					if (!string.IsNullOrEmpty(stringValue))
					{
						BluetoothAddress.TryParse(stringValue, out ret);
					}
					break;
				default:
					throw new ArgumentException("Invalid token type.");
			}

			if (ret == null)
			{
				throw new ArgumentException("Invalid Bluetooth address: " + stringValue);
			}

			return ret;
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
		{
			if (value is BluetoothAddress)
			{
				writer.WriteValue(((BluetoothAddress)value).GetStringFormat());
			}
			else
			{
				writer.WriteNull();
			}
		}
	}
}
