﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the callback argument of StartBrowseService().
		/// </summary>
		public class BrowserCallbackArgs : BaseCallbackArgs
		{
			/// <summary>
			/// Specifies the type of the browser event.
			/// </summary>
			public enum BrowserEventType
			{
				/// <summary>
				/// Signifies a service is found.
				/// </summary>
				Found,

				/// <summary>
				/// Signifies a service is lost.
				/// </summary>
				Lost
			}

			/// <summary>
			/// Gets the instance name.
			/// </summary>
			public string InstanceName { get; internal set; }

			/// <summary>
			/// Gets the index of the interface that the service is found.
			/// </summary>
			public uint? InterfaceIndex { get; internal set; }

			/// <summary>
			/// Gets a value indicating whether the instance is found or lost.
			/// </summary>
			public BrowserEventType? EventType { get; internal set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="BrowserCallbackArgs"/> class.
			/// </summary>
			internal BrowserCallbackArgs()
				: base()
			{
				this.InstanceName = null;
				this.InterfaceIndex = null;
				this.EventType = null;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".InstanceName: " + this.InstanceName);
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + (this.InterfaceIndex.HasValue ? this.InterfaceIndex.Value.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".EventType: " + (this.EventType.HasValue ? this.EventType.Value.GetStringValue() : "N/A"));

				return builder.ToString().TrimEnd();
			}
		}
	}
}
