﻿using AirFusion.Common;
using System;
using System.Net;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class UserDeviceManager
	{
		partial class MulticastDnsAdapter
		{
			/// <summary>
			/// Represents the event data of address updated.
			/// </summary>
			public class AddressUpdatedEventArgs : EventArgs
			{
				/// <summary>
				/// Gets the instance.
				/// </summary>
				public MDnsInstance Instance { get; private set; }

				/// <summary>
				/// Gets the index of the interface.
				/// </summary>
				public uint InterfaceIndex { get; private set; }

				/// <summary>
				/// Gets the new address.
				/// </summary>
				public IPAddress NewAddress { get; private set; }

				/// <summary>
				/// Gets a value indicating whether the address record is removed.
				/// </summary>
				public bool IsRemoved { get; private set; }

				/// <summary>
				/// Prevents a default instance of the <see cref="AddressUpdatedEventArgs"/> class from being created.
				/// </summary>
				AddressUpdatedEventArgs()
				{
				}

				/// <summary>
				/// Initializes a new instance of the <see cref="AddressUpdatedEventArgs"/> class.
				/// </summary>
				/// <param name="instance">The instance.</param>
				/// <param name="interfaceIndex">The index of the interface.</param>
				/// <param name="newAddress">The newer address.</param>
				/// <param name="isRemoved">A value indicating whether the address is removed.</param>
				public AddressUpdatedEventArgs(MDnsInstance instance, uint interfaceIndex, IPAddress newAddress, bool isRemoved)
				{
					if (instance == null)
					{
						throw new ArgumentNullException("instance");
					}

					this.Instance = instance;
					this.InterfaceIndex = interfaceIndex;
					this.NewAddress = newAddress;
					this.IsRemoved = isRemoved;
				}

				/// <summary>
				/// Returns a <see cref="System.String" /> that represents this instance.
				/// </summary>
				/// <returns>
				/// A <see cref="System.String" /> that represents this instance.
				/// </returns>
				public override string ToString()
				{
					StringBuilder builder = new StringBuilder();
					builder.AppendLine(this.GetType().GetFullName() + ".Instance: " + (this.Instance != null ? "\r\n" + this.Instance.ToString() : "N/A"));
					builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + this.InterfaceIndex);
					builder.AppendLine(this.GetType().GetFullName() + ".NewAddress: " + (this.NewAddress != null ? this.NewAddress.ToString() : "N/A"));
					builder.AppendLine(this.GetType().GetFullName() + ".IsRemoved: " + this.IsRemoved);

					return builder.ToString().TrimEnd();
				}
			}
		}
	}
}
