﻿using System;

namespace AirFusion.Manager.DeviceManager
{
	public sealed partial class UserDeviceManager
	{
		/// <summary>
		/// Represents the methods that will handle the event of initialized.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="InitializedEventArgs" /> instance containing the event data.</param>
		public delegate void InitializedEventHandler(object sender, InitializedEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of discovery.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DiscoveryEventArgs" /> instance containing the event data.</param>
		public delegate void DiscoveryEventHandler(object sender, DiscoveryEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of device lost.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DeviceLostEventArgs" /> instance containing the event data.</param>
		public delegate void DeviceLostEventHandler(object sender, DeviceLostEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of device found.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DeviceFoundEventArgs" /> instance containing the event data.</param>
		public delegate void DeviceFoundEventHandler(object sender, DeviceFoundEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of device updated.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DeviceUpdateEventArgs" /> instance containing the event data.</param>
		public delegate void DeviceUpdateEventHandler(object sender, DeviceUpdateEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of pairing invited.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="PairingRequestedEventArgs" /> instance containing the event data.</param>
		public delegate void PairingInvitedEventHandler(object sender, PairingInvitedEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of pairing requested.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="PairingRequestedEventArgs" /> instance containing the event data.</param>
		public delegate void PairingRequestedEventHandler(object sender, PairingRequestedEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of pairing responsed.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="PairingRequestedEventArgs" /> instance containing the event data.</param>
		public delegate void PairingResponsedEventHandler(object sender, PairingResponsedEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of paired.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DevicePairedEventArgs" /> instance containing the event data.</param>
		public delegate void DevicePairedEventHandler(object sender, DevicePairedEventArgs eventArgs);

		/// <summary>
		/// Represents the methods that will handle the event of un-paired.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DeviceUnPairedEventArgs" /> instance containing the event data.</param>
		public delegate void DeviceUnPairedEventHandler(object sender, DeviceUnPairedEventArgs eventArgs);

		/// <summary>
		/// Presents the methods that will handle the error event.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="eventArgs">The <see cref="DeviceUnPairedEventArgs" /> instance containing the event data.</param>
		[Obsolete]
		public delegate void ErrorEventHandler(object sender, ErrorEventArgs eventArgs);
	}
}