﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the event arguments of pairing invited event.
	/// </summary>
	public sealed class PairingInvitedEventArgs : EventArgs
	{
		#region Enumerations

		/// <summary>
		/// Specifies the type of pairing invitation.
		/// </summary>
		public enum PairingInvitationType
		{
			/// <summary>
			/// Reserved.
			/// </summary>
			Reserved,

			/// <summary>
			/// Signifies an unknown invitation type.
			/// </summary>
			Unknown = -1
		}

		#endregion Enumerations

		#region Properties

		/// <summary>
		/// Gets the device.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		/// <summary>
		/// Gets the type of the invitation.
		/// </summary>
		public PairingInvitationType InvitationType { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="PairingInvitedEventArgs" /> class.
		/// </summary>
		internal PairingInvitedEventArgs()
		{
			this.Device = null;
			this.InvitationType = PairingInvitationType.Unknown;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="PairingInvitedEventArgs" /> class.
		/// </summary>
		/// <param name="device">The device.</param>
		/// <param name="invitationType">The type of the pairing invitation.</param>
		internal PairingInvitedEventArgs(AFUserDevice device, PairingInvitationType invitationType = PairingInvitationType.Reserved)
		{
			this.Device = device;
			this.InvitationType = invitationType;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".InvitationType: " + this.InvitationType);

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}