﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the event arguments of pairing requested event.
	/// </summary>
	public sealed class PairingRequestedEventArgs : EventArgs
	{
		#region Enumerations

		/// <summary>
		/// Specifies the type of pairing request.
		/// </summary>
		public enum PairingRequestType
		{
			/// <summary>
			/// Reserved.
			/// </summary>
			Reserved,

			/// <summary>
			/// Signifies an unknown request type.
			/// </summary>
			Unknown = -1
		}

		#endregion Enumerations

		#region Properties

		/// <summary>
		/// Gets the device.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		/// <summary>
		/// Gets the type of the request.
		/// </summary>
		public PairingRequestType RequestType { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="PairingRequestedEventArgs" /> class.
		/// </summary>
		internal PairingRequestedEventArgs()
		{
			this.Device = null;
			this.RequestType = PairingRequestType.Unknown;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="PairingRequestedEventArgs" /> class.
		/// </summary>
		/// <param name="device">The device.</param>
		/// <param name="requestType">The type of the pairing request.</param>
		internal PairingRequestedEventArgs(AFUserDevice device, PairingRequestType requestType = PairingRequestType.Reserved)
		{
			this.Device = device;
			this.RequestType = requestType;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".RequestType: " + this.RequestType);

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}