﻿using AirFusion.Common;
using Newtonsoft.Json;
using System;
using System.Text;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents properties and information of a certain device.
	/// </summary>
	internal partial class DeviceInformation
	{
		/// <summary>
		/// Gets or sets the instance ID.
		/// </summary>
		[JsonProperty("InstanceID"), JsonConverter(typeof(AirFusion.Common.JsonConverters.GuidConverter))]
		public Guid? InstanceID { get; internal set; }

		/// <summary>
		/// Gets or sets the device name.
		/// </summary>
		[JsonProperty("DeviceName")]
		public string DeviceName { get; internal set; }

		/// <summary>
		/// Gets or sets the model name.
		/// </summary>
		[JsonProperty("ModelName")]
		public string ModelName { get; internal set; }

		/// <summary>
		/// Gets or sets the user name.
		/// </summary>
		[JsonProperty("UserName")]
		public string UserName { get; internal set; }

		/// <summary>
		/// Gets or sets the manufacturer of the device.
		/// </summary>
		[JsonProperty("Manufacturer")]
		public string Manufacturer { get; internal set; }

		/// <summary>
		/// Gets or sets the operating system of the device.
		/// </summary>
		[JsonProperty("OS")]
		public string OS { get; internal set; }

		/// <summary>
		/// Gets or sets the version of the operating system of the device.
		/// </summary>
		[JsonProperty("OSVersion")]
		public string OSVersion { get; internal set; }

		/// <summary>
		/// Gets or sets the serial number.
		/// </summary>
		[JsonProperty("SerialNumber")]
		public string SerialNumber { get; internal set; }

		/// <summary>
		/// Gets or sets the application version.
		/// </summary>
		[JsonProperty("ApplicationVersion")]
		public string ApplicationVersion { get; internal set; }

		/// <summary>
		/// Gets or sets the value indicating whether the application has system permission.
		/// </summary>
		[JsonProperty("HasSystemPermission")]
		public bool? HasSystemPermission { get; internal set; }

		/// <summary>
		/// Gets or sets the display information of the device.
		/// </summary>
		[JsonProperty("Display")]
		public DisplayInfo Display { get; internal set; }

		// 2015-05-20 Seoker: remove Bluetooth related functions since it's defeatured
		/*
		/// <summary>
		/// Gets or sets the information of Bluetooth.
		/// </summary>
		[JsonProperty("Bluetooth")]
		public BluetoothInfo Bluetooth { get; internal set; }
		*/

		/// <summary>
		/// Gets or sets the information of AirFusion services.
		/// </summary>
		[JsonProperty("Services")]
		public ServicesInfo Services { get; internal set; }

		#region Private/Protected/Internal

		/// <summary>
		/// Initializes a new instance of the <see cref="DeviceInformation"/> class.
		/// </summary>
		internal DeviceInformation()
		{
			this.InstanceID = null;
			this.DeviceName = null;
			this.ModelName = null;
			this.UserName = null;
			this.Manufacturer = null;
			this.OS = null;
			this.OSVersion = null;
			this.SerialNumber = null;
			this.ApplicationVersion = null;
			this.HasSystemPermission = null;
			this.Display = null;
			//this.Bluetooth = null; // 2015-05-20 Seoker: remove Bluetooth related functions since it's defeatured
			this.Services = null;
		}

		#endregion Private/Protected/Internal

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".InstanceID: " + (this.InstanceID.HasValue ? this.InstanceID.Value.GetStringFormat() : "N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".DeviceName: " + this.DeviceName);
			builder.AppendLine(this.GetType().GetFullName() + ".ModelName: " + this.ModelName);
			builder.AppendLine(this.GetType().GetFullName() + ".UserName: " + this.UserName);
			builder.AppendLine(this.GetType().GetFullName() + ".Manufacturer: " + this.Manufacturer);
			builder.AppendLine(this.GetType().GetFullName() + ".OS: " + this.OS);
			builder.AppendLine(this.GetType().GetFullName() + ".OSVersion: " + this.OSVersion);
			builder.AppendLine(this.GetType().GetFullName() + ".SerialNumber: " + this.SerialNumber);
			builder.AppendLine(this.GetType().GetFullName() + ".ApplicationVersion: " + this.ApplicationVersion);
			builder.AppendLine(this.GetType().GetFullName() + ".HasSystemPermission: " + (this.HasSystemPermission.HasValue ? this.HasSystemPermission.Value.ToString() : "N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".Display: " + (this.Display != null ? "\r\n" + this.Display.ToString() : "N/A"));
			//builder.AppendLine(this.GetType().GetFullName() + ".Bluetooth: " + (this.Bluetooth != null ? "\r\n" + this.Bluetooth.ToString() : "N/A")); // 2015-05-20 Seoker: remove Bluetooth related functions since it's defeatured
			builder.AppendLine(this.GetType().GetFullName() + ".Services: " + (this.Services != null ? "\r\n" + this.Services.ToString() : "N/A"));

			return builder.ToString().TrimEnd();
		}
	}
}
