﻿using AirFusion.Common;
using AirFusion.Manager.DeviceManager;
using System;
using System.Text;

namespace AirFusion.Manager.FeatureHelper
{
	/// <summary>
	/// Represents the event arguments of display configuration updated event.
	/// </summary>
	public sealed class DisplayConfigurationUpdatedEventArgs : EventArgs
	{
		#region Properties

		/// <summary>
		/// Gets the device.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		/// <summary>
		/// Gets the display configuration.
		/// </summary>
		public AFUserDevice.DisplayInfo DisplayConfiguration { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="DisplayConfigurationUpdatedEventArgs"/> class.
		/// </summary>
		internal DisplayConfigurationUpdatedEventArgs()
		{
			this.Device = null;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="DisplayConfigurationUpdatedEventArgs" /> class.
		/// </summary>
		/// <param name="device">The device.</param>
		/// <param name="displayConfiguration">The display configuration.</param>
		internal DisplayConfigurationUpdatedEventArgs(AFUserDevice device, AFUserDevice.DisplayInfo displayConfiguration)
		{
			this.Device = device;
			this.DisplayConfiguration = displayConfiguration;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".DisplayConfiguration: " + (this.DisplayConfiguration != null ? this.DisplayConfiguration.ToString() : "N/A"));

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}