﻿using AirFusion.Common;
using AirFusion.Manager.DeviceManager;
using AirFusion.WebService;
using AirFusion.WebService.Adapters;
using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace AirFusion.Manager.FeatureHelper
{
	/// <summary>
	/// Represents the input control helper.
	/// </summary>
	public sealed partial class InputControlHelper
	{
		#region Statics Properties/Variables/Methods

		internal static IDictionary<UserDeviceManager, InputControlHelper> DeviceManagerMapping = new Dictionary<UserDeviceManager, InputControlHelper>();

		#endregion Statics Properties/Variables/Methods

		#region Properties

		/// <summary>
		/// Gets the input control service.
		/// </summary>
		internal InputControlService InputControlService { get; private set; }

		#endregion Properties

		#region Private/Protected/Internal Variables

		private UserDeviceManager _deviceManager;

		#endregion Private/Protected/Internal Variables

		#region Public Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="InputControlHelper"/> class.
		/// </summary>
		internal InputControlHelper(UserDeviceManager deviceManager, WebServicesManager webManager)
		{
			try
			{
				StaticLogger.TrackEnter();

				// check parameters
				if (deviceManager == null)
				{
					throw new ArgumentNullException("deviceManager");
				}
				if (webManager == null)
				{
					throw new ArgumentNullException("webManager");
				}

				// debug assertion
				Debug.Assert(DeviceManagerMapping != null);

				// check prerequisite
				if (DeviceManagerMapping.ContainsKey(deviceManager))
				{
					throw new InvalidOperationException("The specified device manager already have a input control helper.");
				}

				this._deviceManager = deviceManager;
				DeviceManagerMapping.Add(deviceManager, this);

				// input control service
				{
					this.InputControlService = new InputControlService(webManager);
				}
			}
			finally
			{
				StaticLogger.TrackLeave();
			}
		}

		#endregion Public Methods
	}
}
