﻿using AirFusion.Manager.FeatureHelper;
using Newtonsoft.Json;
using System;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents the web service of the screencast daemon status (request) (GET /screencast/status).
	/// </summary>
	internal class ScreencastDaemonStatusAdapter : WebServiceAdapter<DBNull, ScreencastDaemonStatusAdapter.ScreencastDaemonStatusInfo>
	{
		#region Definitions

		internal const string WebServiceUri = "/screencast/status";
		internal static HttpMethod WebServiceMethod = HttpMethod.Get;

		#endregion Definitions

		#region Classes

		/// <summary>
		/// Represents the status of screencast daemon.
		/// </summary>
		internal class ScreencastDaemonStatusInfo
		{
			/// <summary>
			/// Gets the status of screencast daemon.
			/// </summary>
			[JsonProperty("CurrentStatus"), JsonConverter(typeof(AirFusion.Common.JsonConverters.EnumConverter))]
			public DaemonStatus? CurrentStatus { get; set; }
		}

		#endregion Classes

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="ScreencastDaemonStatusAdapter"/> class.
		/// </summary>
		/// <param name="webServiceManager">The web service manager.</param>
		public ScreencastDaemonStatusAdapter(WebServicesManager webServiceManager)
			: base(webServiceManager)
		{
		}

		/// <summary>
		/// Gets the status of the screencast daemon from target device asynchronously.
		/// </summary>
		/// <param name="address">The IP address of the target device.</param>
		/// <param name="port">The port number of the target device.</param>
		/// <returns>The response.</returns>
		/// <exception cref="AirFusion.WebService.Adapters.WebServiceException" />
		/// <exception cref="System.ArgumentNullException" />
		/// <exception cref="System.Exception" />
		/// <exception cref="System.InvalidOperationException" />
		/// <exception cref="System.TimeoutException" />
		public async Task<WebServiceResponse> GetStatusAsync(IPAddress address, ushort port)
		{
			// check parameters
			if (address == null)
			{
				throw new ArgumentNullException("address");
			}

			var request = WebServiceRequest.CreateLocalRequest(
				WebServiceMethod,
				new UriBuilder(this.Scheme, address.ToString(), (int)port, WebServiceUri).Uri);

			return await this.SendRequestAsync(request, this.Timeout).ConfigureAwait(continueOnCapturedContext: false);
		}

		#endregion Methods
	}
}
