﻿using System;
using System.Diagnostics;
using System.Net;
using System.Net.Http;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents the exception of a web service request.
	/// </summary>
	public class WebServiceException : Exception
	{
		/// <summary>
		/// Gets the response message from server.
		/// </summary>
		public HttpResponseMessage ResponseMessage { get; protected set; }

		/// <summary>
		/// Gets the status code.
		/// </summary>
		public HttpStatusCode StatusCode
		{
			get
			{
				Trace.Assert(this.ResponseMessage != null, "this.ResponseMessage = null");
				return this.ResponseMessage.StatusCode;
			}
		}

		/// <summary>
		/// Gets the reason phrase which typically is sent by servers together with the status code.
		/// </summary>
		public string ReasonPhrase
		{
			get
			{
				Trace.Assert(this.ResponseMessage != null, "this.ResponseMessage = null");
				return this.ResponseMessage.ReasonPhrase;
			}
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="WebServiceException"/> class.
		/// </summary>
		/// <param name="responseMessage">The response message.</param>
		public WebServiceException(HttpResponseMessage responseMessage)
			: base("Server returned: " + (int)responseMessage.StatusCode + " " + responseMessage.ReasonPhrase)
		{
			if (responseMessage == null)
			{
				throw new ArgumentNullException("responseMessage");
			}

			this.ResponseMessage = responseMessage;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="WebServiceException"/> class.
		/// </summary>
		/// <param name="responseMessage">The response message.</param>
		/// <param name="message">The message that describes the error.</param>
		public WebServiceException(HttpResponseMessage responseMessage, string message)
			: base(message)
		{
			if (responseMessage == null)
			{
				throw new ArgumentNullException("responseMessage");
			}

			this.ResponseMessage = responseMessage;
		}
	}
}
