// PullSrcFilter.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include "PullSrcFilter.h"
#include "Ws2tcpip.h"

#pragma region PullSrcFilter Class

CUnknown * WINAPI PullSrcFilter::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
	ASSERT(phr);

	CUnknown *punk = new PullSrcFilter(lpunk, phr);
	if(punk == NULL)
	{
		if(phr)
			*phr = E_OUTOFMEMORY;
	}
	return punk;
} // CreateInstance

PullSrcFilter::PullSrcFilter(LPUNKNOWN lpunk, HRESULT *phr)
	: CBaseFilter(NAME("Pull Source Filter"), lpunk, &m_csFilter, CLSID_PullSrcFilter, NULL)
{
	SourceFilterLogManager::CheckLogOutputStatus();
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::PullSrcFilter\n"));

	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	wVersionRequested = MAKEWORD(2, 2);

	err = WSAStartup(wVersionRequested, &wsaData);
	if (err != 0)
	{
		SourceFilterLogManager::OutputLog(_T("WSAStartup failed with error\n"));
	}

	// Assign media type
	m_mt.majortype = MEDIATYPE_Stream;
	m_mt.subtype   = MEDIASUBTYPE_MPEG2_TRANSPORT;

	m_OutputPin = new PullOutputPin(phr, this, &m_csFilter);

	m_bReceiving   = false;
	m_hSocket      = INVALID_SOCKET;
	m_hMutex = CreateMutex(NULL, FALSE, _T("QUEUE_MUTEX"));

	m_pRingBuffer = new BYTE[RINGBUFFERSIZE];
	m_idxRecvWritePos = 0;
	m_idxRecvWriteRound = 0;
	m_idxOffset = 0;
	mRcvThread = NULL;

	//Johnson 0112 add (default type is UDP type)
	m_iConnectType = ConnectionType::Others;

	//Johnson 0122 add
	m_bSendToUICheck = true;

	//Johnon 0212 add
	m_bSendDummyinfo = false;

	//Johnson 0415 add
	m_bSavePacketToFile = false;

	//For Debug use
	//InitialSetting(ConnectionType::UDP);
}

PullSrcFilter::~PullSrcFilter()
{
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::~PullSrcFilter()\n"));

	DeleteReceiver();

	if (m_pRingBuffer) delete m_pRingBuffer;

	if (mRcvThread != NULL)
	{
		TerminateThread(mRcvThread, 0);
		CloseHandle(mRcvThread);

		//delete mRcvThread;
		//WaitForSingleObject(mRcvThread, INFINITE);
		mRcvThread = NULL;
	}

	WSACleanup();

	delete m_OutputPin;
}

STDMETHODIMP PullSrcFilter::NonDelegatingQueryInterface(REFIID riid, void **pObj)
{
	/*SourceFilterLogManager::OutputLog(_T("PullSrcFilter::NonDelegatingQueryInterface\n"));*/

	/*if (pObj == NULL)
	{
	SourceFilterLogManager::OutputLog(_T("E_POINTER\n"));

	return E_POINTER;
	}
	else if (IsEqualIID(riid,IID_IBaseFilter))
	{
	SourceFilterLogManager::OutputLog(_T("IsEqualIID(riid,IID_IBaseFilter)\n"));

	*pObj = this;
	AddRef();
	return S_OK;
	}
	else if (IsEqualIID(riid,IID_IUnknown))
	{
	SourceFilterLogManager::OutputLog(_T("IsEqualIID(riid,IID_IUnknown)\n"));

	*pObj = this;
	AddRef();
	return S_OK;
	}
	else */
	if (IsEqualIID(riid,__uuidof(ISourceFilter)))
	{
		SourceFilterLogManager::OutputLog(_T("IsEqualIID(riid,__uuidof(ISourceFilter))\n"));

		*pObj = (ISourceFilter *) this;
		AddRef();
		return S_OK;
	}
	else
	{
		/*SourceFilterLogManager::OutputLog(_T("E_NOINTERFACE\n"));*/

		// always set [out] parameter
		/**pObj = NULL;
		return E_NOINTERFACE;*/
		return CBaseFilter::NonDelegatingQueryInterface(riid,pObj);
	}
}

STDMETHODIMP PullSrcFilter::InitialSetting(int _enumType = 0)
{
	//1217 Johnson add to read registry key value
	HKEY hKey;
	LONG Ret;
	DWORD dwReturn = 0;
	DWORD dwBufSize = sizeof(DWORD);

	Ret = RegOpenKeyExW(HKEY_CURRENT_USER, L"SOFTWARE\\OEM\\AcerEXTEND\\SourceFilter", NULL, KEY_ALL_ACCESS, &hKey);

	if(_enumType == ConnectionType::TCP)
	{
		SourceFilterLogManager::OutputLog(_T("PullSrcFilter::InitialSetting->TCP\n"));

		m_iConnectType = ConnectionType::TCP;
		m_hClientSocket = INVALID_SOCKET;
	}
	else
	{
		SourceFilterLogManager::OutputLog(_T("PullSrcFilter::InitialSetting->UDP\n"));

		//Johnson 0112 Add
		//Record ConnectType is UDP Type
		m_iConnectType = ConnectionType::UDP;

		//Johnson 1210 Add
		m_iPreRTPCount = 0;
		m_iTotalRTPCount = 0;
		m_bNeedToSkip = false;
		m_bNetworkTooBad = false;

		//Johnson 0106 add
		m_iPreRTPTimeStamp = 0;

		//m_bSendToUICheck = true;

		if (Ret == ERROR_SUCCESS)
			Ret = RegQueryValueEx(hKey, L"ReferenceCount", NULL, NULL, reinterpret_cast<LPBYTE>(&dwReturn), &dwBufSize);

		if (Ret == ERROR_SUCCESS)
			m_iMAXReferencePacket = (float)dwReturn;
		else
			m_iMAXReferencePacket = 3000;

		Ret = RegQueryValueEx(hKey, L"SuccessRate", NULL, NULL, reinterpret_cast<LPBYTE>(&dwReturn), &dwBufSize);

		if (Ret == ERROR_SUCCESS)
			m_fSuccessRate = (float)dwReturn;
		else
			m_fSuccessRate = 0.8;

		Ret = RegQueryValueEx(hKey, L"RecvBuffSize", NULL, NULL, reinterpret_cast<LPBYTE>(&dwReturn), &dwBufSize);

		if (Ret == ERROR_SUCCESS)
			m_iRecvBufferSize = (DWORD)dwReturn;
		else
			m_iRecvBufferSize = 2048;

		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("RecvBufferSize Registry key value is %d\n"), m_iRecvBufferSize);
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
	}

	//0415 Johnson add for save Video
	Ret = RegQueryValueEx(hKey, L"SavePacket", NULL, NULL, reinterpret_cast<LPBYTE>(&dwReturn), &dwBufSize);

	if (Ret == ERROR_SUCCESS)
	{
		m_bSavePacketToFile = true;

		SYSTEMTIME stLocal;
		::GetLocalTime(&stLocal);
		wchar_t year[5],month[3],day[3],hour[3],minute[3],second[3];
		_itow_s(stLocal.wYear  ,year,10);
		_itow_s(stLocal.wMonth ,month,10);
		_itow_s(stLocal.wDay   ,day,10);
		_itow_s(stLocal.wHour  ,hour,10);
		_itow_s(stLocal.wMinute,minute,10);
		_itow_s(stLocal.wSecond,second,10);
		m_sSavePacketPath.append(L"C:\\ProgramData\\OEM\\AcerEXTEND\\");
		m_sSavePacketPath.append(year);
		m_sSavePacketPath.append(month);
		m_sSavePacketPath.append(day);
		m_sSavePacketPath.append(hour);
		m_sSavePacketPath.append(minute);
		m_sSavePacketPath.append(second);
		m_sSavePacketPath.append(L"Log.ts");

		SourceFilterLogManager::OutputLog(_T("SaveFilte to Local Folder\n"));

		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("%ls"),m_sSavePacketPath.c_str());
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
	}
	else
		m_bSavePacketToFile = false;

	//For Debug use
	//Setting Finish, Receiving data
	//StartReceiving();
	return S_OK;
}

STDMETHODIMP PullSrcFilter::ReConnecting(int _enumType)
{
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::ReConnecting Start\n"));

	DeleteReceiver();
	//if (m_pRingBuffer) delete m_pRingBuffer;
	if (mRcvThread != NULL)
	{
		/*TerminateThread(mRcvThread, 0);
		CloseHandle(mRcvThread);*/
		//delete mRcvThread;
		//WaitForSingleObject(mRcvThread, INFINITE);
		mRcvThread = NULL;
	}
	m_bReceiving   = false;
	m_hSocket      = INVALID_SOCKET;

	/*m_pRingBuffer = new BYTE[RINGBUFFERSIZE];
	m_idxRecvWritePos = 0;
	m_idxRecvWriteRound = 0;
	m_idxOffset = 0;*/

	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::ReConnecting End, Prepare InitialSetting\n"));

	//Johnson 0122 add
	m_bSendToUICheck = true;

	//Johnon 0212 add
	m_bSendDummyinfo = false;

	InitialSetting(_enumType);
	return S_OK;
}

STDMETHODIMP PullSrcFilter::SendDummyData(void)
{
	m_bSendDummyinfo = true;

	TerminateThread(mRcvThread, 0);

	if(m_iConnectType == ConnectionType::UDP)
	{
		closesocket(m_hSocket);
		m_hSocket = INVALID_SOCKET;
	}
	else if(m_iConnectType == ConnectionType::TCP)
	{
		closesocket(m_hSocket);
		closesocket(m_hClientSocket);
		m_hSocket = INVALID_SOCKET;
		m_hClientSocket = INVALID_SOCKET;
	}
	return S_OK;
}

//STDMETHODIMP PullSrcFilter::Stop()
//{
//	SourceFilterLogManager::OutputLog(_T("Start PullSrcFilter::Stop\n"));
//
//	DeleteReceiver();
//
//	if (mRcvThread != NULL)
//	{
//		TerminateThread(mRcvThread, 0);
//		CloseHandle(mRcvThread);
//		//delete mRcvThread;
//		//WaitForSingleObject(mRcvThread, INFINITE);
//		mRcvThread = NULL;
//	}
//
//	//if (m_pRingBuffer) delete m_pRingBuffer;
//
//	WSACleanup();
//
//	SourceFilterLogManager::OutputLog(_T("End PullSrcFilter::Stop\n"));
//	StopConnection = true;
//
//	return S_OK;
//}

int PullSrcFilter::GetPinCount()
{
	return 1;
}

CBasePin * PullSrcFilter::GetPin(int n)
{
	if((GetPinCount() > 0) && (n == 0))
	{
		return m_OutputPin;
	}
	else
	{
		return NULL;
	}
}

STDMETHODIMP PullSrcFilter::StartReceiving(void)
{
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::StartReceiving\n"));

	//Repressent nerver call initial setting function, we need to initial these information
	if(m_iConnectType == ConnectionType::Others)
	{
		SourceFilterLogManager::OutputLog( L"PullSrcFilter::StartReceiving m_iConnectType is Others\n");

		InitialSetting(ConnectionType::UDP);
	}

	if (m_hSocket == INVALID_SOCKET)
	{
		if(m_iConnectType == ConnectionType::UDP)
		{
			SourceFilterLogManager::OutputLog(_T("PullSrcFilter::StartReceiving CreateUDPReceiver\n"));
			CreateUDPReceiver();
		}
		else if(m_iConnectType == ConnectionType::TCP)
		{
			SourceFilterLogManager::OutputLog(_T("PullSrcFilter::StartReceiving CreateUDPReceiver\n"));
			CreateTCPReceiver();
		}
	}

	if (m_hSocket != INVALID_SOCKET)
	{
		if (m_bReceiving)
		{
			return S_OK;;
		}

		SourceFilterLogManager::OutputLog(_T("PullSrcFilter::StartReceiving Thread Start\n"));

		m_bReceiving = true;
		DWORD threadID = 0;
		mRcvThread = CreateThread(NULL, 0, ReceivingThrd, this, 0, &threadID);
	}

	return S_OK;
}

void PullSrcFilter::CreateUDPReceiver(void)
{
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::CreateUDPReceiver start\n"));

	DeleteReceiver();

	m_hSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (m_hSocket != INVALID_SOCKET)
	{
		BOOL flag = TRUE;
		int ret = setsockopt(m_hSocket, SOL_SOCKET, SO_REUSEADDR,
			(char *) &flag, sizeof(flag));

		if (ret == SOCKET_ERROR)
		{
			DeleteReceiver();

			SourceFilterLogManager::OutputLog(_T("setsockopt, SOCKET_ERROR\n"));

			return;
		}

		SOCKADDR_IN  addr;
		addr.sin_family      = AF_INET;
		addr.sin_addr.s_addr = htonl(INADDR_ANY);
		addr.sin_port        = htons(BASE_SOCKET_PORT);
		ret = bind(m_hSocket, (struct sockaddr*) &addr, sizeof(addr));
		int WSAERROR = WSAGetLastError();
		if (ret == SOCKET_ERROR)
		{
			DeleteReceiver();

			SourceFilterLogManager::OutputLog(_T("bind, SOCKET_ERROR\n"));

			return;
		}

		//Johnson Add
		int RcvBufferSize = m_iRecvBufferSize * 1024;
		ret = setsockopt(m_hSocket,SOL_SOCKET,SO_RCVBUF,(const char*)&RcvBufferSize,sizeof(int));
		if (ret == SOCKET_ERROR)
		{
			DeleteReceiver();

			SourceFilterLogManager::OutputLog(_T("Set SO_RCVBUF Error\n"));

			return;
		}
	}
	else
	{
		SourceFilterLogManager::OutputLog(_T("PullSrcFilter::CreateUDPReceiver Error\n"));
	}

	SourceFilterLogManager::OutputLog(_T("Start UDP Transimission\n"));
}

void PullSrcFilter::CreateTCPReceiver(void)
{
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::CreateTCPReceiver start\n"));

	struct addrinfo *result = NULL;
	struct addrinfo hints;

	ZeroMemory(&hints, sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;
	hints.ai_flags = AI_PASSIVE;

	int iResult;

	iResult = getaddrinfo(NULL, DEFAULT_PORT, &hints, &result);
	if (iResult != 0) {
		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("getaddrinfo failed with error: %d\n"), iResult);
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		WSACleanup();
		return;
	}

	// Create a SOCKET for connecting to server
	m_hSocket = socket(result->ai_family, result->ai_socktype, result->ai_protocol);
	if (m_hSocket == INVALID_SOCKET) {
		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("socket failed with error: %ld\n"), WSAGetLastError());
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		freeaddrinfo(result);
		WSACleanup();
		return;
	}

	// Setup the TCP listening socket
	iResult = bind(m_hSocket, result->ai_addr, (int)result->ai_addrlen);
	if (iResult == SOCKET_ERROR) {
		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("bind failed with error: %d\n"), WSAGetLastError());
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		freeaddrinfo(result);
		closesocket(m_hSocket);
		WSACleanup();
		return;
	}

	freeaddrinfo(result);

	iResult = listen(m_hSocket, SOMAXCONN);
	if (iResult == SOCKET_ERROR) {
		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("listen failed with error: %d\n"), WSAGetLastError());
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		closesocket(m_hSocket);
		WSACleanup();
		return;
	}

	SourceFilterLogManager::OutputLog(_T("Start TCP Transimission\n"));
}

DWORD PullSrcFilter::ReceivingThrd(void * pParam)
{
	SourceFilterLogManager::OutputLog(_T("PullSrcFilter::ReceivingThrd start\n"));

	PullSrcFilter * pTarget = (PullSrcFilter *) pParam;
	if (pTarget != NULL)
	{
		pTarget->ReceivingLoop();
	}
	return 0;
}

int PullSrcFilter::Receive(SOCKET inputSocket, char * outBuf, int inLen, int* outLen)
{
	if (inputSocket)
	{
		// Check socket for readability
		TIMEVAL tv;
		tv.tv_sec  = 0;
		tv.tv_usec = 0;
		fd_set rfds;
		FD_ZERO(&rfds);
		FD_SET(inputSocket, &rfds);
		//Johnson modify TIMEVAL from 0 to NULL, the method will always wait for data can get
		if (select(inputSocket+1,&rfds,NULL,NULL,NULL))
			//if (select(1, &rfds, NULL, NULL,&tv))
		{
			int    lenret = 0;
			char * pBuf   = outBuf;

			lenret = recv(inputSocket, pBuf, inLen, 0);

			if (lenret == SOCKET_ERROR)
			{
				return E_SOCKET_FAIL;
			}
			else if (lenret == 0)
			{
				return E_SOCKET_CLOSE;
			}

			*outLen = lenret;

			return S_SOCKET_OK;
		}
		else
		{
			return E_SOCKET_NOT_READY;
		}
	}
	return E_SOCKET_FAIL;
}

void PullSrcFilter::ReceivingLoop(void)
{
	SourceFilterLogManager::OutputLog(_T("Start ReceivingLoop\n"));

	const int RECVSIZE = 4096;
	int iRecvLen = 0, nret = 0;
	char buff[RECVSIZE];

#pragma region Connection Type is UDP
	if(m_iConnectType == ConnectionType::UDP)
	{
		SourceFilterLogManager::OutputLog(_T("Start ReceivingLoop UDP method\n"));

#pragma region Parameter initial
		HWND _pAirFusionTray;
		int _iCurrentRTPCount, _iCurrentRTPTimeStamp, _timeShow = 1000;
		bool _bCheckCount = false, _bSendToUICheck = true;
		LONGLONG _LTotalSize = 0;
		DWORD RTPtick1 = GetTickCount();
		bool _bSequenceErrorSKip = false;
#pragma endregion

		while (m_bReceiving)
		{
			int count = 0;
			// Receive the message header
			while ((nret = Receive(m_hSocket, buff, RECVSIZE, &iRecvLen)) == E_SOCKET_NOT_READY)
			{
				Sleep(10);
				count++;
			}
			// When errors occur or socket closed, terminate this loop
			if (nret == E_SOCKET_FAIL || nret == E_SOCKET_CLOSE)
			{
				SourceFilterLogManager::OutputLog(_T("Socket error, E_SOCKET_FAIL or E_SOCKET_CLOSE\n"));
				break;
			}
			else if (iRecvLen > WFD_PACK_HEADER_SIZE)
			{
				//Calculate Total Receive Data size
				unsigned int TdataLen = (unsigned int)(iRecvLen - WFD_PACK_HEADER_SIZE);
				_LTotalSize += TdataLen;

				//For debug use, will show total receive data size every 1s
				if(GetTickCount() - RTPtick1 > _timeShow)
				{
					swprintf_s(SourceFilterLogManager::m_logBuffer, _T("SrcFilter total get data size(byte) and time (%lld, %d) and Message is %d\n"), _LTotalSize, GetTickCount(), m_bNetworkTooBad?1:0);
					SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
					_timeShow += 1000;
				}

				//For debug use, save data to file
				/*char* realData = (char*)malloc(sizeof(char)*TdataLen);
				memcpy(realData, buff + WFD_PACK_HEADER_SIZE, TdataLen);
				FILE * pFile;
				fopen_s(&pFile, "C:\\ProgramData\\UDP_OriLog.ts", "ab+");
				if (pFile != NULL)
				{
				fwrite(realData, sizeof(char), TdataLen, pFile);
				fclose(pFile);
				}*/

				//For debug use, show RTP time Stamp and real data arrival time difference
				/*DWORD RTPtick1 = 0;
				RTPtick1 = GetTickCount();
				int _iRTPTimeStamp = (static_cast<unsigned char>(buff[4])) * 256 * 256 *256 + (static_cast<unsigned char>(buff[5])) * 256 * 256 + (static_cast<unsigned char>(buff[6])) * 256 + (static_cast<unsigned char>(buff[7]));

				swprintf_s(SourceFilterLogManager::m_logBuffer, _T("Time Stamp and TickCount difference value is %d\n"), RTPtick1 - _iRTPTimeStamp);
				SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);*/

#pragma region about RTP TimeStamp info
				//Cal RTP Time Stamp value(in RTP header 4,5,6 and 7)
				_iCurrentRTPTimeStamp = (static_cast<unsigned char>(buff[4])) * 256 * 256 *256 + (static_cast<unsigned char>(buff[5])) * 256 * 256 + (static_cast<unsigned char>(buff[6])) * 256 + (static_cast<unsigned char>(buff[7]));

				//if previous RTP Time Stamp not equal zero(repressent phone have been send data) and
				//Current RTP Time Stamp and Previous Time Stamp differnece is big than 50000(reference by Wireshark result)
				//Reset all UDP parameter(for phone is disconnection scenario?we need to reset all parameter otherwi)
				if(m_iPreRTPTimeStamp != 0)
				{
					if(_iCurrentRTPTimeStamp - m_iPreRTPTimeStamp > 50000)
					{
						SourceFilterLogManager::OutputLog(_T("Reconnecting Start, need to reset value\n"));

						_bCheckCount = false;
						m_bNeedToSkip = false;
						m_iPreRTPCount = 0;
						_iCurrentRTPCount = 0;

						while(!m_qRecentCount.empty())
							m_qRecentCount.pop();
					}
				}
				m_iPreRTPTimeStamp = _iCurrentRTPTimeStamp;
#pragma endregion

#pragma region Johnson add. When UDP packet loss, we will skip the frame until next key frame arrival
				//Check MPEG TS header contain PES start packet(Payload Unit Start Indicator(in sencond byte), so need to Add RTP header lenght)
				if (static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + 1] & 0x40) == 0x40 && m_bNeedToSkip)
				{
					//check video stream(0x1011)
					if ((static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + 1] & 0x1F) == 0x10) && (static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + 2]) == 0x11))
					{
						//check this is PES header value(0x00 00 01)
						if ((static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + 4]) == 0x00) && (static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + 5]) == 0x00) &&
							(static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + 6]) == 0x01))
						{
							for(int k = 7; k<187 ; k++)
							{
								if((static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + k]) == 0x00) && (static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + k + 1]) == 0x00) && (static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + k + 2]) == 0x00) &&
									(static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + k + 3]) == 0x01) && (static_cast<unsigned char>(buff[WFD_PACK_HEADER_SIZE + k + 4]) == 0x65))
								{
									m_bNeedToSkip = false;
									SourceFilterLogManager::OutputLog(_T("*****Start Recvive data from Socket by I Frame*****\n"));
									break;
								}
							}
						}
					}
				}

				//Record RTP sequence count numbher
				_iCurrentRTPCount = ((static_cast<unsigned char>(buff[2])) * 256 + (static_cast<unsigned char>(buff[3])));

				if(_iCurrentRTPCount > m_iMAXReferencePacket)
					_bCheckCount = true;

				//Johnson add for skip frame
				if(m_iPreRTPCount != 0 && _iCurrentRTPCount != 0)
				{
					//cal difference between current RTP count and previous RTP count
					if ((_iCurrentRTPCount - m_iPreRTPCount) != 1)
					{
						//0130 Johnson add enhance user experience, when AirFusion initial get data, if have any data loss, we should not skip any info, just to show it.
						/*if(_bCheckCount)
						m_bNeedToSkip = true;*/

						swprintf_s(SourceFilterLogManager::m_logBuffer, _T("********************************************\n Current Packet number is (%d, %d, %d) \n Packet Loss %d\n Total RTP Count %d\n********************************************\n"), (static_cast<unsigned char>(buff[2])));
						SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
						m_iTotalRTPCount += (_iCurrentRTPCount - m_iPreRTPCount);
					}
					else
						m_iTotalRTPCount++;
				}
				else
					m_iTotalRTPCount++;

				if((_iCurrentRTPCount < m_iPreRTPCount) && _iCurrentRTPCount != 0)
				{
					_bSequenceErrorSKip = true;
					swprintf_s(SourceFilterLogManager::m_logBuffer, _T("********************************************\nCurrent Packet, Previous number is (%d, %d) \n********************************************\n"), _iCurrentRTPCount, m_iPreRTPCount);
					SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
				}
				else
				{
					//Record current RTP count
					m_iPreRTPCount = _iCurrentRTPCount;
					if(_bSequenceErrorSKip)
					{
						SourceFilterLogManager::OutputLog(_T("********************************************\nThe Sequence Number is Right, continue to receive \n********************************************\n"));
					}
					_bSequenceErrorSKip = false;
				}

				if(!m_bNeedToSkip && !_bSequenceErrorSKip)
					m_qRecentCount.push(m_iPreRTPCount);

				//avoid queue overflow
				while(m_qRecentCount.size() > m_iMAXReferencePacket)
					m_qRecentCount.pop();

				//if(m_bNeedToSkip)
				{
					//represent normal case
					if(m_iPreRTPCount - m_qRecentCount.front() >= 0)
					{
						while (!m_qRecentCount.empty() && m_iPreRTPCount - m_qRecentCount.front() > m_iMAXReferencePacket)
						{
							m_qRecentCount.pop();
						}
					}
					else //for overflow case(cycle count is 65536)
					{
						while (!m_qRecentCount.empty() && m_iPreRTPCount + 65536 - m_qRecentCount.front() > m_iMAXReferencePacket)
						{
							m_qRecentCount.pop();
						}
					}
					if(m_qRecentCount.size() >= m_iMAXReferencePacket * m_fSuccessRate)
					{
						if(m_bNetworkTooBad)
						{
							SendResultMessage(true);
							SourceFilterLogManager::OutputLog(_T("Good Network : m_qRecentCount.size() >= m_iMAXReferencePacket * m_fSuccessRate \n"));
						}
						m_bNetworkTooBad = false;
					}
					else if((_bCheckCount && m_qRecentCount.size() < m_iMAXReferencePacket * m_fSuccessRate) || _bSequenceErrorSKip || (_bCheckCount == false && m_qRecentCount.size() < m_iPreRTPCount * m_fSuccessRate))
					{
						if(!m_bNetworkTooBad)
						{
							SendResultMessage(false);
							SourceFilterLogManager::OutputLog(_T("Bad Network\n"));
						}
						m_bNetworkTooBad = true;

						if(_bCheckCount)
							m_qRecentCount.push(m_iPreRTPCount);
					}
					else
					{
						if(m_bNetworkTooBad)
						{
							SendResultMessage(true);
							SourceFilterLogManager::OutputLog(_T("Good Network\n"));
						}
						m_bNetworkTooBad = false;
					}
				}
#pragma endregion

				if((m_bNeedToSkip == false || m_bNetworkTooBad) && _bSequenceErrorSKip == false)
				{
					unsigned int dataLen = (unsigned int)(iRecvLen - WFD_PACK_HEADER_SIZE);

					SaveToRingQueue(buff, dataLen);

					//if (m_idxRecvWritePos + dataLen >= RINGBUFFERSIZE)
					//{
					//	// exceed buffer size, wrap to start
					//	unsigned int iOverSize = m_idxRecvWritePos + dataLen - RINGBUFFERSIZE;
					//	memcpy(m_pRingBuffer + m_idxRecvWritePos, buff + WFD_PACK_HEADER_SIZE, dataLen-iOverSize);
					//	memcpy(m_pRingBuffer, buff + WFD_PACK_HEADER_SIZE + dataLen - iOverSize, iOverSize);
					//	m_idxRecvWritePos = iOverSize;
					//	m_idxRecvWriteRound++;
					//}
					//else
					//{
					//	memcpy(m_pRingBuffer + m_idxRecvWritePos, buff + WFD_PACK_HEADER_SIZE, dataLen);	// remove RTP header
					//	m_idxRecvWritePos += dataLen;
					//}

					//if(m_idxRecvWritePos > 4096 & _bSendToUICheck)
					//{
					//	_pAirFusionTray = FindWindow(NULL,L"AirFusionTray");
					//	if(_pAirFusionTray != NULL)
					//	{
					//		PostMessage(_pAirFusionTray,WM_CHECKRECEIVEDATA,NULL,NULL);
					//		_bSendToUICheck = false;
					//	}
					//}
				}
			}
			else
			{
				SourceFilterLogManager::OutputLog(_T("ERRORR!!!!!!!!!!!!!\n"));
				break;
			}
		}
	}
#pragma endregion

#pragma region Connection Type is TCP
	else if(m_iConnectType == ConnectionType::TCP)
	{
		SourceFilterLogManager::OutputLog(_T("Start ReceivingLoop TCP create Server\n"));

		//0527 Johnon add for double check send event to UI info data recevice complete
		m_bSendToUICheck = true;
		m_idxRecvWritePos = 0;

		// Accept a client socket
		while((m_hClientSocket = accept(m_hSocket, NULL, NULL)) != INVALID_SOCKET)
		{
			TCPListenSocket(m_hClientSocket);
			while((nret = recv(m_hClientSocket, buff, 2, MSG_WAITALL)) >= 0) //2 + 12byte(2 is record total byte and 12 is RTP HEADER byte)
			{
				int _iRTPPacketLength = (static_cast<unsigned char>(buff[0])) * 256 + (static_cast<unsigned char>(buff[1]));

				if(CheckRTPPacketLength(_iRTPPacketLength))
				{
					int _realSize = recv(m_hClientSocket, buff, _iRTPPacketLength, MSG_WAITALL);
					if(_realSize == _iRTPPacketLength)
					{
						SaveToRingQueue(buff, _iRTPPacketLength-(DEF_RTP_PACKET_HEADER_SIZE + DEF_MPEG_TS_HEADER_SIZE));
					}
					else
					{
						_stprintf_s(SourceFilterLogManager::m_logBuffer, _T("TCP Size Error, Packet length : %d, Real size : %d"), _iRTPPacketLength, _realSize);
						SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
						PostSourceFilterDisconnectMessage();
						break;
					}
				}
				else
				{
					_stprintf_s(SourceFilterLogManager::m_logBuffer, _T("RTP packet error, Packet length : %d"), _iRTPPacketLength);
					SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
					PostSourceFilterDisconnectMessage();
					break;
				}
			}
			_stprintf_s(SourceFilterLogManager::m_logBuffer, _T("recv failed with error: %d\n"), WSAGetLastError());
			SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
			closesocket(m_hClientSocket);
			m_hClientSocket = INVALID_SOCKET;
		}

		//char* recvbuf;
		//recvbuf = new char[DEFAULT_BUFLEN];
		//int recvbuflen = DEFAULT_BUFLEN;
		//int previous_value = 0, header_value = 0;
		//bool _bChangeLength = false;
		//int iReceiveLength = 0, _iPreviousRTPCount = 0;

		//// Receive until the peer shuts down the connection
		//do {
		//	iReceiveLength = recv(m_hClientSocket, recvbuf + header_value, recvbuflen - header_value, 0);
		//	iReceiveLength = iReceiveLength + header_value;
		//	int _iRTPPacketLength = iReceiveLength;
		//
		//  swprintf_s(SourceFilterLogManager::m_logBuffer, _T("ReceiveLength and Header value is (%d, %d)\n"), iReceiveLength, header_value);
		//  SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);

		//	header_value = 0;

		//	if (iReceiveLength > 0)
		//	{
		//		for (int i = previous_value + header_value; i < iReceiveLength; i++)
		//		{
		//			if (i + 2 > iReceiveLength)//check length overflow
		//			{
		//				previous_value = (static_cast<unsigned char>(recvbuf[i])) * 256;
		//				_bChangeLength = true;
		//				break;
		//			}

		//			previous_value = 0;
		//			header_value = 0;
		//			_bChangeLength = false;

		//			int _varLength = (static_cast<unsigned char>(recvbuf[i])) * 256 + (static_cast<unsigned char>(recvbuf[i + 1])) - 12; //12 is UDP+RTP header

		//			//12 is UDP + RTP header and 2 is unnecessary byte(this byte record the packet length)
		//			_iRTPPacketLength = _iRTPPacketLength - _varLength - 12 - 2;

		//			if (_iRTPPacketLength < 0) //Repressent UDP+RTP header overflow
		//			{
		//				_iRTPPacketLength = _iRTPPacketLength + _varLength + 14;
		//				memcpy(recvbuf, recvbuf + i, _iRTPPacketLength);
		//				header_value = _iRTPPacketLength;
		//				break;
		//			}

		//			//Double check the packet is belong MPEG2-TS packet
		//			if ((unsigned)recvbuf[i + 14] == 0x47)
		//			{
		//				//Record RTP sequence count number
		//				/*int _CurrentRTPCount = ((static_cast<unsigned char>(recvbuf[i + 4])) * 256 + (static_cast<unsigned char>(recvbuf[i + 5])));
		//				if(_CurrentRTPCount != 0 && (_CurrentRTPCount - _iPreviousRTPCount) != 1)
		//				{
		//					SourceFilterLogManager::OutputLog(_T("Packet Loss Error\n"));
		//				}
		//				_iPreviousRTPCount = _CurrentRTPCount;
		//				swprintf_s(SourceFilterLogManager::m_logBuffer, _T("RTP Sequence Numbaer: %d\n"), _CurrentRTPCount);
		//				SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);

		//				char* realData = (char*)malloc(sizeof(char)*_varLength);
		//				memcpy(realData, recvbuf + i + 14, _varLength);
		//				FILE * pFile;
		//				fopen_s(&pFile, "C:\\ProgramData\\0112TCPInfo.ts", "ab+");
		//				if (pFile != NULL)
		//				{
		//					fwrite(realData, sizeof(char), _varLength, pFile);
		//					fclose(pFile);
		//					swprintf_s(SourceFilterLogManager::m_logBuffer, _T("Bytes received: %d\n"), _varLength);
		//					SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		//				}*/

		//				if (m_idxRecvWritePos + _varLength >= RINGBUFFERSIZE)
		//				{
		//					// exceed buffer size, wrap to start
		//					unsigned int iOverSize = m_idxRecvWritePos + _varLength - RINGBUFFERSIZE;
		//					memcpy(m_pRingBuffer + m_idxRecvWritePos, recvbuf + i + 14, _varLength - iOverSize);
		//					memcpy(m_pRingBuffer, recvbuf + _varLength - iOverSize + 14, iOverSize);
		//					m_idxRecvWritePos = iOverSize;
		//					m_idxRecvWriteRound++;
		//				}
		//				else
		//				{
		//					char* realData = (char*)malloc(sizeof(char)* _varLength);
		//					memcpy(m_pRingBuffer + m_idxRecvWritePos, recvbuf + i + 14, _varLength);	// remove RTP header
		//					m_idxRecvWritePos += _varLength;
		//				}
		//			}
		//			else
		//				int k= 0;
		//			i = i + _varLength + 14 - 1;
		//		}
		//	}
		//	else
		//	{
		//		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("recv failed with error: %d\n"), WSAGetLastError());
		//		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		//		//printf("recv failed with error: %d\n", WSAGetLastError());
		//		closesocket(m_hClientSocket);
		//		closesocket(m_hSocket);
		//		//WSACleanup();
		//	}
		//} while (iReceiveLength >= 0);

		// shutdown the connection since we're done
		//iReceiveLength = shutdown(m_hClientSocket, SD_SEND);
		//if (iReceiveLength == SOCKET_ERROR) {
		//	printf("shutdown failed with error: %d\n", WSAGetLastError());
		//	closesocket(m_hClientSocket);
		//	closesocket(m_hSocket);
		//	//WSACleanup();
		//	return;
		//}

		//if(recvbuf) delete recvbuf;

		// cleanup
		closesocket(m_hClientSocket);
		//WSACleanup();
	}
#pragma endregion

	SourceFilterLogManager::OutputLog(_T("End ReceivingLoop\n"));

	m_bReceiving = false;
}

void PullSrcFilter::TCPListenSocket(SOCKET ClientSocket)
{
	if (ClientSocket == INVALID_SOCKET)
	{
		swprintf_s(SourceFilterLogManager::m_logBuffer, _T("accept failed with error: %d\n"), WSAGetLastError());
		SourceFilterLogManager::OutputLog(SourceFilterLogManager::m_logBuffer);
		closesocket(ClientSocket);
		WSACleanup();
	}

	int iRecvBufferSize = 20 * 1024 * 1024;
	setsockopt(m_hSocket,SOL_SOCKET,SO_RCVBUF,(const char*)&iRecvBufferSize,sizeof(int));
	setsockopt(ClientSocket,SOL_SOCKET,SO_RCVBUF,(const char*)&iRecvBufferSize,sizeof(int));

	SourceFilterLogManager::OutputLog(_T("Start ReceivingLoop TCP create Server done, start receive data\n"));
}

void PullSrcFilter::SaveToRingQueue(char * sockBuf, unsigned dataLen)
{
	if(m_bSavePacketToFile)
	{
		char str[255];
		sprintf_s(str,"%ls",m_sSavePacketPath.c_str());

		char* realData = (char*)malloc(sizeof(char)*dataLen);
		memcpy(realData, sockBuf + WFD_PACK_HEADER_SIZE, dataLen);
		FILE * pFile;
		fopen_s(&pFile, str, "ab+");
		if (pFile != NULL)
		{
			fwrite(realData, sizeof(char), dataLen, pFile);
			fclose(pFile);
		}
	}

	if (m_idxRecvWritePos + dataLen >= RINGBUFFERSIZE)
	{
		// exceed buffer size, wrap to start
		unsigned int iOverSize = m_idxRecvWritePos + dataLen - RINGBUFFERSIZE;
		memcpy(m_pRingBuffer + m_idxRecvWritePos, sockBuf + WFD_PACK_HEADER_SIZE, dataLen-iOverSize);
		memcpy(m_pRingBuffer, sockBuf + WFD_PACK_HEADER_SIZE + dataLen - iOverSize, iOverSize);
		m_idxRecvWritePos = iOverSize;
		m_idxRecvWriteRound++;
	}
	else
	{
		memcpy(m_pRingBuffer + m_idxRecvWritePos, sockBuf + WFD_PACK_HEADER_SIZE, dataLen);	// remove RTP header
		m_idxRecvWritePos += dataLen;
	}

	if(m_idxRecvWritePos > 4096 & m_bSendToUICheck)
	{
		HWND pAirFusionTray;
		pAirFusionTray = FindWindow(NULL,L"AirFusionTray");
		if(pAirFusionTray != NULL)
		{
			PostMessage(pAirFusionTray,WM_CHECKRECEIVEDATA,NULL,NULL);
			m_bSendToUICheck = false;

			SourceFilterLogManager::OutputLog(_T("PullSrcFilter::SaveToRingQueue send finish info to UI\n"));

			//When first Launch , need to send Message to UI info the network is good
			SourceFilterLogManager::OutputLog(_T("*************The network is good****************\n"));
			PostMessage(pAirFusionTray,WM_GOODNETWORK,NULL,NULL);
		}
	}
}

void PullSrcFilter::SendResultMessage(bool bNetworkStatus)
{
	HWND pAirFusionTray;
	pAirFusionTray = FindWindow(NULL,L"AirFusionTray");
	if(pAirFusionTray != NULL)
	{
		if(bNetworkStatus)
		{
			SourceFilterLogManager::OutputLog(_T("*************The network is good****************\n"));
			PostMessage(pAirFusionTray,WM_GOODNETWORK,NULL,NULL);
		}
		else
		{
			SourceFilterLogManager::OutputLog(_T("*************The network is too bad, cancle skip frame method, just to show it****************\n"));
			PostMessage(pAirFusionTray,WM_BADNETWORK,NULL,NULL);
		}
	}
}

void PullSrcFilter::DeleteReceiver(void)
{
	if (m_hSocket != INVALID_SOCKET)
	{
		closesocket(m_hSocket);
		m_hSocket = INVALID_SOCKET;
	}
}

void PullSrcFilter::QueuePush(PMPEGTS_PACK inData)
{
	WaitForSingleObject(m_hMutex, INFINITE);

	m_dQueue.push_back(inData);

	ReleaseMutex(m_hMutex);
}

PMPEGTS_PACK PullSrcFilter::QueuePop()
{
	WaitForSingleObject(m_hMutex, INFINITE);

	if(m_dQueue.empty())
	{
		ReleaseMutex(m_hMutex);
		return NULL;
	}
	else
	{
		PMPEGTS_PACK target = *(m_dQueue.begin());
		m_dQueue.pop_front();

		ReleaseMutex(m_hMutex);
		return target;
	}
}

/// <summary>
/// Check the length of the RTP packet.
/// </summary>
/// <param name="packetLength">Length of the RTP packet.</param>
/// <returns>True: Status correct; False: Status incorrect</returns>
bool PullSrcFilter::CheckRTPPacketLength(int packetLength)
{
	if((packetLength > (DEF_RTP_PACKET_HEADER_SIZE + DEF_MPEG_TS_HEADER_SIZE)) 
		&& (((packetLength - (DEF_RTP_PACKET_HEADER_SIZE + DEF_MPEG_TS_HEADER_SIZE)) % MPEGTS_PACK_SIZE) == 0) 
		&& ((packetLength - (DEF_RTP_PACKET_HEADER_SIZE + DEF_MPEG_TS_HEADER_SIZE)) / MPEGTS_PACK_SIZE) <= DEF_MPEG_TS_MAXIMUM_PACKET_COUNT)
	{
		return TRUE;
	}
	return FALSE;
}

/// <summary>
/// Posts the source filter disconnect message.
/// </summary>
void PullSrcFilter::PostSourceFilterDisconnectMessage()
{
	HWND windowHandler;
	windowHandler = FindWindow(NULL,L"AirFusionTray");
	if(windowHandler != NULL)
	{
		PostMessage(windowHandler,WM_TCPDisconnected,NULL,NULL);
	}
}

#pragma endregion
