﻿using AirFusion.WebService;
using Microsoft.WindowsAPICodePack.Shell.PropertySystem;
using MS.WindowsAPICodePack.Internal;
using ShortcutHelpers;

using Windows.Data.Xml.Dom;
using Windows.UI.Notifications;

using System;
using System.Drawing;
using System.IO;

using AirFusion.Common;

namespace AirFusion.Manager.ToastManager
{
    public partial class AFToastManager
    {
        #region event handler definition
        public delegate void OnToastActivatedEventHandler(ToastNotification sender, object e);
        public delegate void OnToastToastDismissedEventHandler(ToastNotification sender, ToastDismissalReason reason);
        public delegate void OnToastToastFailedEventHandler(ToastNotification sender, ToastFailedEventArgs e);

        public event OnToastActivatedEventHandler AFToastActivated;
        public event OnToastToastDismissedEventHandler AFToastDismissed;
        public event OnToastToastFailedEventHandler AFToastFailed;
        #endregion

        public class ToastPresentationItem
        {
            public System.Windows.Media.Imaging.BitmapSource BitmapIcon { get; set; }
            public string Title { get; set; }
            public string[] Content { get; set; }

            public string NotificationItemKey { get; set; }

            internal ToastPresentationItem()
            {
                BitmapIcon = null;
                Title = String.Empty;
                NotificationItemKey = String.Empty;
                Content = null;
            }
        }

        public AFToastManager()
        {
            TryCreateShortcut();
        }

        #region public function
        public ToastPresentationItem CreateAndShowToast(string notificationKey, AFNotificationPostAdapter.AFNotificationPost newPostItem)
        {
            ToastPresentationItem item = NotificationItemParser(notificationKey, newPostItem);

            ToastTemplateType toastType = GetToastType(newPostItem.Icon, newPostItem.Texts.Length);

            XmlDocument toastXml = ToastXmlGenerator(toastType, item);

            //Generate Toast
            //Create the toast and attach event listeners
            ToastNotification toast = new ToastNotification(toastXml);
            toast.Activated += ToastActivated;
            toast.Dismissed += ToastDismissed;
            toast.Failed += ToastFailed;

            // Show the toast. Be sure to specify the AppUserModelId on your application's shortcut!
            ToastNotificationManager.CreateToastNotifier(APP_ID).Show(toast);

            return item;
        }
        #endregion

        #region private function
        private bool TryCreateShortcut()
        {
            String shortcutPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Microsoft\\Windows\\Start Menu\\Programs\\AirFusion.lnk";
            if (!File.Exists(shortcutPath))
            {
                InstallShortcut(shortcutPath);
                return true;
            }
            return false;
        }

        private void InstallShortcut(String shortcutPath)
        {
            try
            {
                AFStaticLogger.TrackEnter();

                // Find the path to the current executable
                String exePath = System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName;
                IShellLinkW newShortcut = (IShellLinkW)new CShellLink();

                // Create a shortcut to the exe
                ShortcutHelpers.ErrorHelper.VerifySucceeded(newShortcut.SetPath(exePath));
                ShortcutHelpers.ErrorHelper.VerifySucceeded(newShortcut.SetArguments(""));

                // Open the shortcut property store, set the AppUserModelId property
                IPropertyStore newShortcutProperties = (IPropertyStore)newShortcut;

                using (PropVariant appId = new PropVariant(APP_ID))
                {
                    ShortcutHelpers.ErrorHelper.VerifySucceeded(newShortcutProperties.SetValue(SystemProperties.System.AppUserModel.ID, appId));
                    ShortcutHelpers.ErrorHelper.VerifySucceeded(newShortcutProperties.Commit());
                }

                // Commit the shortcut to disk
                IPersistFile newShortcutSave = (IPersistFile)newShortcut;

                ShortcutHelpers.ErrorHelper.VerifySucceeded(newShortcutSave.Save(shortcutPath, true));
            }
            finally
            {
                AFStaticLogger.TrackLeave();
            }
        }


        private ToastTemplateType GetToastType(string iconText, int numOfText)
        {
            ToastTemplateType toastType = ToastTemplateType.ToastImageAndText01;
            if (String.IsNullOrWhiteSpace(iconText)) //only text, no image exist
            {
                switch (numOfText)
                {
                    case 1:
                        toastType = ToastTemplateType.ToastText01;
                        break;
                    case 2:
                        toastType = ToastTemplateType.ToastText02;
                        break;
                    case 3:
                    default:
                        toastType = ToastTemplateType.ToastText04;
                        break;
                }
            }
            else
            {
                switch (numOfText)
                {
                    case 1:
                        toastType = ToastTemplateType.ToastImageAndText01;
                        break;
                    case 2:
                        toastType = ToastTemplateType.ToastImageAndText02;
                        break;
                    case 3:
                    default:
                        toastType = ToastTemplateType.ToastImageAndText04;
                        break;
                }
            }

            return toastType;
        }

        private ToastPresentationItem NotificationItemParser(string notificaitonKey, AFNotificationPostAdapter.AFNotificationPost newPostItem)
        {
            try
            {
                AFStaticLogger.TrackEnter();


                #region Generate NotificationPresentationItem
                ToastPresentationItem toastItem = new ToastPresentationItem();
                toastItem.NotificationItemKey = notificaitonKey;

                if (newPostItem.Texts.Length >= 1)
                {
                    toastItem.Title = newPostItem.Texts[0];
                    if (toastItem.Content == null)
                    {
                        toastItem.Content = new string[newPostItem.Texts.Length - 1];
                    }

                    for (int index = 1; index < newPostItem.Texts.Length; ++index)
                    {
                        toastItem.Content[index - 1] = newPostItem.Texts[index];
                    }
                }

                //Toast image
                if (!String.IsNullOrWhiteSpace(newPostItem.Icon))
                {
                    byte[] byteBuffer = Convert.FromBase64String(newPostItem.Icon);
                    MemoryStream memoryStream = new MemoryStream(byteBuffer);
                    memoryStream.Position = 0;

                    Bitmap bitmapIcon = (Bitmap)Bitmap.FromStream(memoryStream);
                    toastItem.BitmapIcon = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(
                                bitmapIcon.GetHbitmap(),
                                IntPtr.Zero,
                                System.Windows.Int32Rect.Empty,
                                System.Windows.Media.Imaging.BitmapSizeOptions.FromWidthAndHeight(bitmapIcon.Width, bitmapIcon.Height));

                    toastItem.BitmapIcon.Freeze();

                    bitmapIcon.Save(toastItem.NotificationItemKey + ".png");
                    memoryStream.Close();
                    memoryStream = null;
                    byteBuffer = null;
                }
                #endregion

                return toastItem;
            }
            finally
            {
                AFStaticLogger.TrackLeave();
            }
        }

        private XmlDocument ToastXmlGenerator(ToastTemplateType toastType, ToastPresentationItem toastItem)
        {
            try
            {
                AFStaticLogger.TrackEnter();

                // Get a toast XML template
                XmlDocument toastXml = ToastNotificationManager.GetTemplateContent(toastType);

                // Fill in the text elements
                XmlNodeList stringElements = toastXml.GetElementsByTagName("text");
                stringElements[0].AppendChild(toastXml.CreateTextNode(toastItem.Title));
                for (int index = 1; index < stringElements.Length; ++index)
                {
                    stringElements[index].AppendChild(toastXml.CreateTextNode(toastItem.Content[index - 1]));
                }

                // Specify the absolute path to an image   
                if (toastItem.BitmapIcon != null)
                {
                    string imagePath = toastItem.NotificationItemKey + ".png";
                    imagePath = "file:///" + Path.GetFullPath(imagePath);
                    XmlNodeList imageElements = toastXml.GetElementsByTagName("image");
                    imageElements[0].Attributes.GetNamedItem("src").NodeValue = imagePath;
                }

                XmlNodeList toastNodes = toastXml.GetElementsByTagName("toast");
                toastNodes[0].AppendChild(toastXml.CreateElement(AFToastXmlTag));
                XmlNodeList AFNotificationNodes = toastXml.GetElementsByTagName(AFToastXmlTag);
                AFNotificationNodes[0].AppendChild(toastXml.CreateTextNode(toastItem.NotificationItemKey));

                //toast XML example
                /*
                var xml = @"<toast>
                    <visual>
                        <binding template=""ToastText02"" branding=""none"">
                            <text id=""1"">Sean</text>
                            <text id=""2"">Tsai</text>  
                        </binding>  
                    </visual>
                    <airFusion>8888888</airFusion>
                </toast>";
                var xmlDoc = new XmlDocument();
                xmlDoc.LoadXml(xml);
                */
                return toastXml;
            }
            finally
            {
                AFStaticLogger.TrackLeave();
            }
        }
        #endregion

        #region event function
        private void ToastActivated(ToastNotification sender, object e)
        {
            if (AFToastActivated != null)
            {
                AFToastActivated(sender, e);
            }
        }

        private void ToastDismissed(ToastNotification sender, ToastDismissedEventArgs e)
        {
            String outputText = "";
            switch (e.Reason)
            {
                case ToastDismissalReason.ApplicationHidden:
                    outputText = "The app hid the toast using ToastNotifier.Hide";
                    break;
                case ToastDismissalReason.UserCanceled:
                    outputText = "The user dismissed the toast";
                    break;
                case ToastDismissalReason.TimedOut:
                    outputText = "The toast has timed out";
                    break;
            }

            if (AFToastDismissed != null)
            {
                AFToastDismissed(sender, e.Reason);
            }
        }

        private void ToastFailed(ToastNotification sender, ToastFailedEventArgs e)
        {
            if (AFToastFailed != null)
            {
                AFToastFailed(sender, e);
            }
        }
        #endregion
    }
}
